/*********************************************************************
 *                    Flowcode TCP IP Component Code
 *
 * File: PIC_TCP_IP.c
 *
 * (c) 2009 Matrix Multimedia Ltd.
 * http://www.matrixmultimedia.com
 *
 * Software License Agreement
 *
 * The software supplied herewith by Matrix Multimedia Ltd (the
 * Company) for its Flowcode graphical programming language is
 * intended and supplied to you, the Companys customer, for use
 * solely and exclusively on the Company's products. The software
 * is owned by the Company, and is protected under applicable
 * copyright laws. All rights are reserved. Any use in violation
 * of the foregoing restrictions may subject the user to criminal
 * sanctions under applicable laws, as well as to civil liability
 * for the breach of the terms and conditions of this licence.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * Changelog:
 *
 *  date  | by | description
 * -------+----+-----------------------------------------------------
 * 240407 | ST | Created
 * 210607 | BR | Externalised
 * 130707 | ST | Made to work with the CTL file
 * 110608 | BR | Added function Rx_Data_Size
 * 180309 | BR | Changed delays to be WDT compatible
 * 250309 | BR | Moved private functions from AddDefines section
 * 130509 | BR | Added % definitions to DefinesCode section
 * 100909 | BR | Added HiTECH compatible definitions
 * 131109 | BR | Fixed #pragma error HITECH problem
 * 040110 | BR | Fixed a problem with targets that have a 16-bit timer0 peripheral
 * 150110 | ST | Added <tmr0l> alternative definition and changed "random" port value to use "tmr0" instead of "TMR0" in <MAC_Reset_Pointers>
 * 230311 | BR | Changed code to work for hitech and boostc without ifdefs
 * 051011 | ST | Added SPI support for Wiznet810
 * 180412 | ST | Added substitutions from the component (hw type, cs pins)
 * 180412 | ST | INT pin now has its own port/tris vals
 ********************************************************************/

/*********************************************************************

 Return & parameter types:
   void
   char
   short
   char*

 Pin directions:
   0 = OUTPUT
   2 = INPUT
   3 = BIDIRECTIONAL

**********************************************************************

[Settings]
CLSID={8934F95A-3D31-4061-B325-475BB8E0F23D}
IsAnalogue=0
MultipleAllowed=1
Description=TCP IP Flowcode Component

**********************************************************************

[Pins]
Count=2
1=Interrupt
2=CS

[PinPort]
# PORTA = 0
# PORTB = 1
# PORTC = 2
# PORTD = 3
# PORTE = 4
1=0
2=0

[PinDesiredBit]
1=0
2=3

[PinDirection]
# DIR_INPUT = 2
# DIR_OUTPUT = 0
# DIR_BIDIRECTIONAL = 3
1=2
2=3

[PinMustUsePort]
# boolean
1=1
2=1

[SamePortAsPinX]
# -1 = any pin
# otherwise, the pin number
1=-1
2=-1

[PinMustUsePin]
# boolean
1=1
2=1

[PinValue]
# boolean (0 = off, 1 = on)
1=0
2=0

**********************************************************************

[MacroNames]
Count=27
1=Initialize
2=Create_MAC_Socket
3=Create_IP_Socket
4=Create_UDP_Socket
5=Create_TCP_Socket
6=Set_Destination
7=Set_My_IP
8=GetSocketStatus
9=TCP_Listen
10=TCP_Connect
11=TCP_Close
12=Tx_start
13=Tx_sendbyte
14=Tx_sendmymac
15=Tx_sendmyip
16=Tx_end
17=Rx_data_available
18=Rx_data_size
19=Rx_readheader
20=Rx_readbyte
21=Rx_skipbytes
22=Rx_match2bytes
23=Rx_match4bytes
24=Rx_match6bytes
25=Rx_match_mymac
26=Rx_match_myip
27=Rx_flush_data

[MacroReturns]
1=void
2=char
3=char
4=char
5=char
6=char
7=void
8=char
9=void
10=char
11=void
12=void
13=void
14=void
15=void
16=void
17=char
18=short
19=char
20=char
21=void
22=char
23=char
24=char
25=char
26=char
27=void

[MacroIsPrivate]
1=0
2=0
3=0
4=0
5=0
6=0
7=0
8=0
9=0
10=0
11=0
12=0
13=0
14=0
15=0
16=0
17=0
18=0
19=0
20=0
21=0
22=0
23=0
24=0
25=0
26=0
27=0

[MacroParameters_Initialize]
Count=0

[MacroParamTypes_Initialize]


[MacroParameters_Create_MAC_Socket]
Count=3
1=promiscuous
2=broadcast
3=error

[MacroParamTypes_Create_MAC_Socket]
1=char
2=char
3=char

[MacroParameters_Create_IP_Socket]
Count=3
1=channel
2=protocol
3=broadcast

[MacroParamTypes_Create_IP_Socket]
1=char
2=char
3=char

[MacroParameters_Create_UDP_Socket]
Count=3
1=channel
2=src_port_hi
3=src_port_lo

[MacroParamTypes_Create_UDP_Socket]
1=char
2=char
3=char

[MacroParameters_Create_TCP_Socket]
Count=3
1=channel
2=src_port_hi
3=src_port_lo

[MacroParamTypes_Create_TCP_Socket]
1=char
2=char
3=char

[MacroParameters_Set_Destination]
Count=7
1=channel
2=dst_ip0
3=dst_ip1
4=dst_ip2
5=dst_ip3
6=dst_port_hi
7=dst_port_lo

[MacroParamTypes_Set_Destination]
1=char
2=char
3=char
4=char
5=char
6=char
7=char

[MacroParameters_Set_My_IP]
Count=4
1=ip0
2=ip1
3=ip2
4=ip3

[MacroParamTypes_Set_My_IP]
1=char
2=char
3=char
4=char

[MacroParameters_GetSocketStatus]
Count=1
1=channel

[MacroParamTypes_GetSocketStatus]
1=char

[MacroParameters_TCP_Listen]
Count=1
1=channel

[MacroParamTypes_TCP_Listen]
1=char

[MacroParameters_TCP_Connect]
Count=7
1=channel
2=dst_ip0
3=dst_ip1
4=dst_ip2
5=dst_ip3
6=dst_port_hi
7=dst_port_lo

[MacroParamTypes_TCP_Connect]
1=char
2=char
3=char
4=char
5=char
6=char
7=char

[MacroParameters_TCP_Close]
Count=1
1=channel

[MacroParamTypes_TCP_Close]
1=char

[MacroParameters_Tx_start]
Count=1
1=channel

[MacroParamTypes_Tx_start]
1=char

[MacroParameters_Tx_sendbyte]
Count=2
1=channel
2=data

[MacroParamTypes_Tx_sendbyte]
1=char
2=char

[MacroParameters_Tx_sendmymac]
Count=1
1=channel

[MacroParamTypes_Tx_sendmymac]
1=char

[MacroParameters_Tx_sendmyip]
Count=1
1=channel

[MacroParamTypes_Tx_sendmyip]
1=char

[MacroParameters_Tx_end]
Count=1
1=channel

[MacroParamTypes_Tx_end]
1=char

[MacroParameters_Rx_data_available]
Count=1
1=channel

[MacroParamTypes_Rx_data_available]
1=char

[MacroParameters_Rx_data_size]
Count=1
1=channel

[MacroParamTypes_Rx_data_size]
1=char

[MacroParameters_Rx_readheader]
Count=2
1=channel
2=idx

[MacroParamTypes_Rx_readheader]
1=char
2=char

[MacroParameters_Rx_readbyte]
Count=1
1=channel

[MacroParamTypes_Rx_readbyte]
1=char

[MacroParameters_Rx_skipbytes]
Count=2
1=channel
2=count

[MacroParamTypes_Rx_skipbytes]
1=char
2=char

[MacroParameters_Rx_match2bytes]
Count=3
1=channel
2=byte1
3=byte2

[MacroParamTypes_Rx_match2bytes]
1=char
2=char
3=char

[MacroParameters_Rx_match4bytes]
Count=5
1=channel
2=byte1
3=byte2
4=byte3
5=byte4

[MacroParamTypes_Rx_match4bytes]
1=char
2=char
3=char
4=char
5=char

[MacroParameters_Rx_match6bytes]
Count=7
1=channel
2=byte1
3=byte2
4=byte3
5=byte4
6=byte5
7=byte6

[MacroParamTypes_Rx_match6bytes]
1=char
2=char
3=char
4=char
5=char
6=char
7=char

[MacroParameters_Rx_match_mymac]
Count=1
1=channel

[MacroParamTypes_Rx_match_mymac]
1=char

[MacroParameters_Rx_match_myip]
Count=1
1=channel

[MacroParamTypes_Rx_match_myip]
1=char

[MacroParameters_Rx_flush_data]
Count=1
1=channel

[MacroParamTypes_Rx_flush_data]
1=char

 ********************************************************************/

/********************************************************************
 * ADDITIONAL CODE
 ********************************************************************/

/*DefinesCode_Start*/

/**** Macro Substitutions ****
a = COMP_X
b = W3100A_WR (not used)
c = I2C_INT
d = I2C_INTMASK
e = I2C_BAUD
f = GAR
g = SMR
h = SHAR
i = SIPR
j = CS pin
k = CS port letter
l = INT port letter
m = HW type (0 = W3100, 1 = W5100)
******************************/

#ifndef MX_TCPIP_STATIC_DEFINES
  #define MX_TCPIP_STATIC_DEFINES

//I'll start with the defines that are common to all component instances:
#ifdef _BOOSTC
	#define MX_MAC_SHORTDELAY nop();nop();nop();nop();nop();nop();nop();nop();nop();nop();
#endif

#ifdef HI_TECH_C
	#define MX_MAC_SHORTDELAY asm("nop");asm("nop");asm("nop");asm("nop");asm("nop");asm("nop");asm("nop");asm("nop");asm("nop");asm("nop");
#endif


#ifdef _BOOSTC
	#ifndef TMR0
		#ifndef TMR0L
			#error "No definition exists for TMR0 or tmr0l"
		#else
			#define tmr0 tmr0l
		#endif
	#endif
#endif
#ifdef HI_TECH_C
	#ifndef tmr0
		#ifndef tmr0l
			#error "No definition exists for TMR0 or tmr0l"
		#else
			#define tmr0 tmr0l
		#endif
	#endif
#endif

#define MX_TCPIP_HWTYPE_W3100	0		//uses I2C
#define MX_TCPIP_HWTYPE_W5100	1		//uses SPI
#define MX_TCPIP_HWTYPE_MIAC	2		//uses CAN

//TODO: these will come from the component
#define MX_TCPIP_HWTYPE		%m
#define %a_CS_PIN			%j
#define %a_CS_PORT			port%k
#define %a_CS_TRIS			tris%k


#if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)

	#define IRTR_HI        0x03
	#define IRTR_LO        0xE8
	#define RCR_VAL        100

	// Register definitions
	#define REG_Cx_CR(i)    (i + 0x00)
	#define REG_Cx_ISR(i)   (i + 0x04)
	#define REG_IR          0x08
	#define REG_IMR         0x09

	#define REG_IDM_OR      0x0C
	#define REG_IDM_AR0     0x0D
	#define REG_IDM_AR1     0x0E
	#define REG_IDM_DR      0x0F

	#define RX_PTR_BASE     0x10
	#define RX_PTR_SIZE     0x0C
	#define REG_Cx_RW_PR(i) (RX_PTR_BASE + (RX_PTR_SIZE*i))
	#define REG_Cx_RR_PR(i) (RX_PTR_BASE + (RX_PTR_SIZE*i) + 0x04)
	#define REG_Cx_TA_PR(i) (RX_PTR_BASE + (RX_PTR_SIZE*i) + 0x08)

	#define TX_PTR_BASE     0x40
	#define TX_PTR_SIZE     0x0C
	#define REG_Cx_TW_PR(i) (TX_PTR_BASE + (TX_PTR_SIZE*i))
	#define REG_Cx_TR_PR(i) (TX_PTR_BASE + (TX_PTR_SIZE*i) + 0x04)

	#define REG_GAR         0x80
	#define REG_SMR         0x84
	#define REG_SHAR        0x88
	#define REG_SIPR        0x8E
	#define REG_IRTR        0x92
	#define REG_RCR         0x94
	#define REG_RMSR        0x95
	#define REG_TMSR        0x96

	#define SOCK_BASE       0xA0
	#define SOCK_SIZE       0x18
	#define REG_Cx_SSR(i)   (SOCK_BASE + (SOCK_SIZE*i))
	#define REG_Cx_SOPR(i)  (SOCK_BASE + (SOCK_SIZE*i) + 0x01)
	#define REG_Cx_DIR(i)   (SOCK_BASE + (SOCK_SIZE*i) + 0x08)
	#define REG_Cx_DPR(i)   (SOCK_BASE + (SOCK_SIZE*i) + 0x0C)
	#define REG_Cx_SPR(i)   (SOCK_BASE + (SOCK_SIZE*i) + 0x0E)
	#define REG_Cx_IPR(i)   (SOCK_BASE + (SOCK_SIZE*i) + 0x10)
	#define REG_Cx_TOSR(i)  (SOCK_BASE + (SOCK_SIZE*i) + 0x11)
	#define REG_Cx_MSSR(i)  (SOCK_BASE + (SOCK_SIZE*i) + 0x12)

	//shadow registers
	#define SHADOW_REG_HI    0x01
	#define SHADOW_BASE      0xE0
	#define Cx_SRW_PTR(i)    (SHADOW_BASE + (3*i))
	#define Cx_SRR_PTR(i)    (SHADOW_BASE + (3*i) + 0x01)
	#define Cx_STA_PTR(i)    (SHADOW_BASE + (3*i) + 0x02)
	#define Cx_STW_PTR(i)    (SHADOW_BASE + (3*i) + 0x10)
	#define Cx_STR_PTR(i)    (SHADOW_BASE + (3*i) + 0x11)


	// Settings for REG_Cx_SOPR
	#define SOCKOPT_BROADCAST    0x80
	#define SOCKOPT_NDTIMEOUT    0x40
	#define SOCKOPT_NDACK        0x20
	#define SOCKOPT_SWS          0x10

	#define MACLOPT_RXERR        0x80
	#define MACLOPT_BROADCAST    0x40
	#define MACLOPT_PROMISC      0x10

	#define SOCK_MACL_RAW        0x04
	#define SOCK_IPL_RAW         0x03
	#define SOCK_DGRAM           0x02
	#define SOCK_STREAM          0x01
	#define SOCK_CLOSEDM         0x00


	// Settings for IP PROTOCOL
	//#define IPPROTO_IP              0               // dummy for IP
	//#define IPPROTO_ICMP            1               // control message protocol
	//#define IPPROTO_IGMP            2               // internet group management protocol
	//#define IPPROTO_GGP             3               // gateway^2 (deprecated)
	//#define IPPROTO_TCP             6               // tcp
	//#define IPPROTO_PUP             12              // pup
	//#define IPPROTO_UDP             17              // user datagram protocol
	//#define IPPROTO_IDP             22              // xns idp
	//#define IPPROTO_ND              77              // UNOFFICIAL net disk proto
	//#define IPPROTO_RAW             255             // raw IP packet


	// Parameters
	//#define SEL_CONTROL     0    // Confirm socket status
	//#define SEL_SEND        1    // Confirm Tx free buffer size
	//#define SEL_RECV        2    // Confirm Rx data size


	// Settings for REG_Cx_CR
	#define CSYS_INIT    0x01
	#define CSOCK_INIT   0x02
	#define CCONNECT     0x04
	#define CLISTEN      0x08
	#define CCLOSE       0x10
	#define CSEND        0x20
	#define CRECV        0x40
	#define CSW_RESET    0x80


	// Settings for REG_Cx_ISR
	#define SSYS_INIT_OK    0x01
	#define SSOCK_INIT_OK   0x02
	#define SESTABLISHED    0x04
	#define SCLOSED         0x08
	#define STIMEOUT        0x10
	#define SSEND_OK        0x20
	#define SRECV_OK        0x40


	// Socket Status Variables
	#define SOCK_CLOSED         0x00
	#define SOCK_ARP            0x01
	#define SOCK_LISTEN         0x02
	#define SOCK_SYNSENT        0x03
	#define SOCK_SYNSENT_ACK    0x04
	#define SOCK_SYNRECV        0x05
	#define SOCK_ESTABLISHED    0x06
	#define SOCK_CLOSE_WAIT     0x07
	#define SOCK_LAST_ACK       0x08
	#define SOCK_FIN_WAIT1      0x09
	#define SOCK_FIN_WAIT2      0x0A
	#define SOCK_CLOSING        0x0B
	#define SOCK_TIME_WAIT      0x0C
	#define SOCK_RESET          0x0D
	#define SOCK_INIT           0x0E
	#define SOCK_UDP            0x0F
	#define SOCK_RAW            0x10
	#define SOCK_UDP_ARP        0x11
	#define SOCK_UDP_DATA       0x12
	#define SOCK_RAW_INIT       0x13

	//4 channels... each channel to have 2k each
	#define RMSR_VAL            0x55
	#define TMSR_VAL            0x55

	//these are the high offsets (low byte = 0x00) and masks (low byte = 0xFF)
	#define MEM_TX_OFFSET(i)    (0x40 + (i*8))
	#define MEM_TX_MASK(i)      (0x07)
	#define MEM_RX_OFFSET(i)    (0x60 + (i*8))
	#define MEM_RX_MASK(i)      (0x07)

	#define MAC_RD_ADDR    (MAC_WR_ADDR+1)

#elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)


	//200ms retry time, count = 25
	#define IRTR_HI        0x07
	#define IRTR_LO        0xD0
	#define RCR_VAL        25

	// Wiznet W5100 Register Addresses
	#define W5100_MR			0x0000   // Mode Register
	#define W5100_GAR			0x0001   // Gateway Address: 0x0001 to 0x0004
	#define W5100_SUBR			0x0005   // Subnet mask Address: 0x0005 to 0x0008
	#define W5100_SHAR  		0x0009   // Source Hardware Address (MAC): 0x0009 to 0x000E
	#define W5100_SIPR			0x000F   // Source IP Address: 0x000F to 0x0012
	#define W5100_IR			0x0015   // Interrupt
	#define W5100_IMR			0x0016   // Interrupt mask
	#define W5100_RTR			0x0017   // Retry time
	#define W5100_RCR			0x0019   // Retry count
	#define W5100_RMSR			0x001A   // RX Memory Size Register
	#define W5100_TMSR			0x001B   // TX Memory Size Register
	#define W5100_PATR			0x001C   // Authentication type in PPPoE
	#define W5100_PTIMER		0x0028   // PPP LCP Request Timer
	#define W5100_PMAGIC		0x0029   // PPP LCP Magic Number
	#define W5100_UIPR			0x002A   // Unreachable IP Address
	#define W5100_UPORT			0x002E   // Unreachable Port

	#define W5100_CH_BASE		0x0400   // Socket base
	#define W5100_CH_SIZE		0x0100   // Size of each channel

	#define W5100_Sn_MR(ch)		(W5100_CH_BASE + (ch*W5100_CH_SIZE) + 0x00)   // Socket Mode
	#define W5100_Sn_CR(ch)		(W5100_CH_BASE + (ch*W5100_CH_SIZE) + 0x01)   // Socket Command
	#define W5100_Sn_IR(ch)		(W5100_CH_BASE + (ch*W5100_CH_SIZE) + 0x02)   // Socket Interrupt
	#define W5100_Sn_SR(ch)		(W5100_CH_BASE + (ch*W5100_CH_SIZE) + 0x03)   // Socket Status
	#define W5100_Sn_PORT(ch)	(W5100_CH_BASE + (ch*W5100_CH_SIZE) + 0x04)   // Socket Source Port
	#define W5100_Sn_DHAR(ch)	(W5100_CH_BASE + (ch*W5100_CH_SIZE) + 0x06)   // Socket Dest Hardware Addr
	#define W5100_Sn_DIPR(ch)	(W5100_CH_BASE + (ch*W5100_CH_SIZE) + 0x0C)   // Socket Dest IP Addr
	#define W5100_Sn_DPORT(ch)	(W5100_CH_BASE + (ch*W5100_CH_SIZE) + 0x10)   // Socket Dest Port
	#define W5100_Sn_MSSR(ch)	(W5100_CH_BASE + (ch*W5100_CH_SIZE) + 0x12)   // Socket Max Segment Size
	#define W5100_Sn_PROTO(ch)	(W5100_CH_BASE + (ch*W5100_CH_SIZE) + 0x14)   // Socket Protocol in IP Raw Mode
	#define W5100_Sn_TOS(ch)	(W5100_CH_BASE + (ch*W5100_CH_SIZE) + 0x15)   // Socket Type of Service
	#define W5100_Sn_TTL(ch)	(W5100_CH_BASE + (ch*W5100_CH_SIZE) + 0x16)   // Socket Time To Live

	#define W5100_Sn_TX_FSR(ch)	(W5100_CH_BASE + (ch*W5100_CH_SIZE) + 0x20)   // Socket TX Free Size
	#define W5100_Sn_TX_RD(ch)	(W5100_CH_BASE + (ch*W5100_CH_SIZE) + 0x22)   // Socket TX Read Pointer
	#define W5100_Sn_TX_WR(ch)	(W5100_CH_BASE + (ch*W5100_CH_SIZE) + 0x24)   // Socket TX Write Pointer

	#define W5100_Sn_RX_RSR(ch)	(W5100_CH_BASE + (ch*W5100_CH_SIZE) + 0x26)   // Socket RX Received Size
	#define W5100_Sn_RX_RD(ch)	(W5100_CH_BASE + (ch*W5100_CH_SIZE) + 0x28)   // Socket RX Read Pointer

	//4 channels... each channel to have 2k each
	#define RMSR_VAL            0x55
	#define TMSR_VAL            0x55

	//TX and RX memory
	#define W5100_TXMEM_BASE	0x4000
	#define W5100_TXMEM_SIZE	0x0800 // 2kb
	#define W5100_TXMEM_MASK	0x07FF
	#define W5100_TXMEM(ch)		(W5100_TXMEM_BASE + (ch*W5100_TXMEM_SIZE))	// start of socket tx data

	#define W5100_RXMEM_BASE	0x6000
	#define W5100_RXMEM_SIZE	0x0800 // 2kb
	#define W5100_RXMEM_MASK	0x07FF
	#define W5100_RXMEM(ch)		(W5100_RXMEM_BASE + (ch*W5100_RXMEM_SIZE))	// start of socket rx data

	// MODE register values
	#define W5100_MR_RST				0x80			// reset
	#define W5100_MR_PB					0x10			// ping block
	#define W5100_MR_PPPOE				0x08			// enable pppoe
	//#define W5100_MR_LB  				0x04			// little or big endian selector in indirect mode
	#define W5100_MR_AI					0x02			// auto-increment in indirect mode
	#define W5100_MR_IND				0x01			// enable indirect mode

	// IR/IRM register values
	#define W5100_IR_CONFLICT			0x80			// check ip confict
	#define W5100_IR_UNREACH			0x40			// get the destination unreachable message in UDP sending
	#define W5100_IR_PPPoE				0x20			// get the PPPoE close message
	#define W5100_IR_SOCK(ch)			(0x01 << ch)	// check socket interrupt

	// Sn_MR values
	#define W5100_Sn_MR_CLOSE			0x00			// unused socket
	#define W5100_Sn_MR_TCP				0x01			// TCP
	#define W5100_Sn_MR_UDP				0x02			// UDP
	#define W5100_Sn_MR_IPRAW			0x03			// IP LAYER RAW SOCK
	#define W5100_Sn_MR_MACRAW			0x04			// MAC LAYER RAW SOCK
	#define W5100_Sn_MR_PPPOE			0x05			// PPPoE
	#define W5100_Sn_MR_ND				0x20			// No Delayed Ack(TCP) flag
	#define W5100_Sn_MR_MULTI			0x80			// support multicating


	// Sn_CR values
	#define W5100_Sn_CR_OPEN			0x01			// initialize or open socket
	#define W5100_Sn_CR_LISTEN			0x02			// wait connection request in tcp mode(Server mode)
	#define W5100_Sn_CR_CONNECT			0x04			// send connection request in tcp mode(Client mode)
	#define W5100_Sn_CR_DISCON			0x08			// send closing reqeuset in tcp mode
	#define W5100_Sn_CR_CLOSE			0x10			// close socket
	#define W5100_Sn_CR_SEND			0x20			// updata txbuf pointer, send data
	#define W5100_Sn_CR_SEND_MAC		0x21			// send data with MAC address, so without ARP process
	#define W5100_Sn_CR_SEND_KEEP		0x22			//  send keep alive message
	#define W5100_Sn_CR_RECV			0x40			// update rxbuf pointer, recv data

	//#ifdef __DEF_IINCHIP_PPP__
	//	#define W5100_Sn_CR_PCON		0x23
	//	#define W5100_Sn_CR_PDISCON		0x24
	//	#define W5100_Sn_CR_PCR			0x25
	//	#define W5100_Sn_CR_PCN			0x26
	//	#define W5100_Sn_CR_PCJ			0x27
	//#endif

	// Sn_IR values
	//#ifdef __DEF_IINCHIP_PPP__
	//	#define W5100_Sn_IR_PRECV		0x80
	//	#define W5100_Sn_IR_PFAIL		0x40
	//	#define W5100_Sn_IR_PNEXT		0x20
	//#endif
	#define W5100_Sn_IR_SEND_OK			0x10			// complete sending
	#define W5100_Sn_IR_TIMEOUT			0x08			// assert timeout
	#define W5100_Sn_IR_RECV			0x04			// receiving data
	#define W5100_Sn_IR_DISCON			0x02			// closed socket
	#define W5100_Sn_IR_CON				0x01			// established connection

	// Sn_SR values
	#define W5100_SOCK_CLOSED			0x00			// closed
	#define W5100_SOCK_ARP	 			0x01			// ARP request sent
	#define W5100_SOCK_INIT 			0x13			// init state
	#define W5100_SOCK_LISTEN			0x14			// listen state
	#define W5100_SOCK_SYNSENT	   		0x15			// connection state
	#define W5100_SOCK_SYNRECV			0x16			// connection state
	#define W5100_SOCK_ESTABLISHED		0x17			// success to connect
	#define W5100_SOCK_FIN_WAIT			0x18			// closing state
	#define W5100_SOCK_CLOSING			0x1A			// closing state
	#define W5100_SOCK_TIME_WAIT		0x1B			// closing state
	#define W5100_SOCK_CLOSE_WAIT		0x1C			// closing state
	#define W5100_SOCK_LAST_ACK			0x1D			// closing state
	#define W5100_SOCK_UDP				0x22			// udp socket
	#define W5100_SOCK_IPRAW			0x32			// ip raw mode socket
	#define W5100_SOCK_MACRAW			0x42			// mac raw mode socket
	#define W5100_SOCK_PPPOE			0x5F			// pppoe socket

	// IP PROTOCOL
	#define W5100_IPPROTO_IP			0       	    // Dummy for IP (v6)
	#define W5100_IPPROTO_ICMP			1       	    // Control message protocol
	#define W5100_IPPROTO_IGMP			2       	    // Internet group management protocol
	#define W5100_IPPROTO_GGP			3       	    // Gateway^2 (deprecated)
	#define W5100_IPPROTO_TCP			6       	    // TCP
	#define W5100_IPPROTO_PUP			12      	    // PUP
	#define W5100_IPPROTO_UDP			17      	    // UDP
	#define W5100_IPPROTO_IDP			22      	    // XNS idp
	#define W5100_IPPROTO_ND			77      	    // UNOFFICIAL net disk protocol
	#define W5100_IPPROTO_RAW			255     	    // Raw IP packet

#elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

	#error "TODO: MIAC stuff here"

#else

	#error "Error - no Internet bus type defined."

#endif

/*
  JW: For debug purposes - send value to LEDs on port D
*/
#define MX_TCPIP_DEBUG(data, wait) trisd = 0x00;portd = (data);delay_s(wait);
#define MX_TCPIP_DEBUG_FLICKER()   trisd = 0x00;portd = 0xFF;delay_ms(250);portd = 0x00;delay_ms(250);portd = 0xFF;delay_ms(250);portd = 0x0;delay_ms(250);


//Internal prototypes
char MAC_MaskFromBit(char which_bit);
void MAC_Reset_Pointers(char channel);

#if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)

	//common globals
	char MAC_PTR_VAL[4];
	char MAC_WR_ADDR;

	void MAC_SendI2CByte(char val);
	void MAC_SendI2CStart();
	void MAC_SendI2CStop();
	char MAC_ReadI2CByte(char lastbyte);

	void MAC_WriteReg(char reg_h, char reg_l, char val);
	char MAC_ReadReg(char reg_h, char reg_l);
	void MAC_WritePtr(char reg_h, char reg_l);
	void MAC_ReadPtr(char reg_h, char reg_l);
	void MAC_ReadPtrWithShadow(char reg_shadow, char reg_pointer);

#elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)

	//common globals
	unsigned short gTxPtr[4];
	unsigned short gTxSize[4];
	unsigned short gRxPtr[4];
	unsigned short gRxSize[4];
	char gHrdSize[4];

	//unsigned short MAC_RX_PTR;
	char MAC_WR_ADDR;

	char MAC_SPI_Byte(char val);

	void MAC_WriteByte(unsigned short addr, char data);
	char MAC_ReadByte(unsigned short addr);
	void MAC_WriteWord(unsigned short addr, unsigned short data);
	unsigned short MAC_ReadWord(unsigned short addr);
    // Helper for blocking write
    #define MAC_WriteCmd(reg, data, dowhile) do { MAC_WriteByte(W5100_Sn_##reg(0), (data)) while(MAC_ReadByte(W5100_Sn_##reg(0)) dowhile); } while (0)

#endif

//common implementations
char MAC_MaskFromBit(char which_bit)
{
   char retVal = 0;
   switch (which_bit)
   {
      case 0:
         retVal = 0x01;
         break;

      case 1:
         retVal = 0x02;
         break;

      case 2:
         retVal = 0x04;
         break;

      case 3:
         retVal = 0x08;
         break;

      case 4:
         retVal = 0x10;
         break;

      case 5:
         retVal = 0x20;
         break;

      case 6:
         retVal = 0x40;
         break;

      case 7:
         retVal = 0x80;
         break;
   }
   return (retVal);
}


//Low level comms
#if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)
	char MAC_ReadI2CByte(char lastbyte)
	{
		char retVal = 0;
		char mask;

	  #ifndef MX_MI2C
			#error "Chip does not have master I2C capability"
	  #else

		//wait until the bus is idle...
		//mask = MAC_MaskFromBit(R_W);
		mask = 0x04;
		while (sspstat & mask);     // tx in progress?

		//mask = MAC_MaskFromBit(ACKEN);
		//mask = mask | MAC_MaskFromBit(RCEN);
		//mask = mask | MAC_MaskFromBit(PEN);
		//mask = mask | MAC_MaskFromBit(RSEN);
		//mask = mask | MAC_MaskFromBit(SEN);
		mask = 0x1F;
		while (sspcon2 & mask);     // bus busy?

		//set RCEN and wait for it to clear
		//mask = MAC_MaskFromBit(RCEN);
		mask = 0x08;
		sspcon2 = sspcon2 | mask;
		while ((sspcon2 & mask) == mask);

		if (lastbyte == 0)
		{
			cr_bit(sspcon2, ACKDT);
		} else {
			st_bit(sspcon2, ACKDT);
		}

		//set ACKEN and wait for it to clear
		//mask = MAC_MaskFromBit(ACKEN);
		mask = 0x10;
		sspcon2 = sspcon2 | mask;
		while ((sspcon2 & mask) == mask);

		retVal = sspbuf;

	  #endif

		return (retVal);
	}


	void MAC_SendI2CByte(char val)
	{
		char rw_mask;
		char mask;

	  #ifndef MX_MI2C
			#error "Chip does not have master I2C capability"
	  #else

		//rw_mask = MAC_MaskFromBit(R_W);
		rw_mask = 0x04;

		//wait until the bus is idle...
		while (sspstat & rw_mask);     // tx in progress?

		//mask = MAC_MaskFromBit(ACKEN);
		//mask = mask | MAC_MaskFromBit(RCEN);
		//mask = mask | MAC_MaskFromBit(PEN);
		//mask = mask | MAC_MaskFromBit(RSEN);
		//mask = mask | MAC_MaskFromBit(SEN);
		mask = 0x1F;
		while (sspcon2 & mask);        // bus busy?

		//mask = MAC_MaskFromBit(BF);
		mask = 0x01;
		while (sspstat & mask);        // buffer still full?

		sspbuf = val;

		while (sspstat & rw_mask);     // tx in progress?

		//mask = MAC_MaskFromBit(ACKSTAT);
		//while (sspcon2 & mask);        // ack?

	  #endif

		return;
	}


	void MAC_SendI2CStart()
	{

	  #ifndef MX_MI2C
			#error "Chip does not have master I2C capability"
	  #else

		//set SEN and wait for it to clear
		char mask;
		//mask = MAC_MaskFromBit(SEN);
		mask = 0x01;
		sspcon2 = sspcon2 | mask;
		while ((sspcon2 & mask) == mask);

	  #endif

		return;
	}


	void MAC_SendI2CStop()
	{

	  #ifndef MX_MI2C
			#error "Chip does not have master I2C capability"
	  #else

		//set PEN and wait for it to clear
		char mask;
		//mask = MAC_MaskFromBit(PEN);
		mask = 0x04;
		sspcon2 = sspcon2 | mask;
		while ((sspcon2 & mask) == mask);

	  #endif

		return;
	}

#elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)

	char MAC_SPI_Byte(char val)
	{
		char Retval = 0;

		#ifndef MX_SPI
			#error "Chip does not have SPI capability"
		#else
			sspbuf = val;					//send a Char
			while ((sspstat & 0x01) == 0);	//wait until sspbuf is empty
			Retval = sspbuf;				//receive the Char
		#endif

		return Retval;
	}

#elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

	#error "TODO: MIAC stuff here"

#else

	#error "Error - no Internet bus type defined."

#endif



////////////////////////////////////////////////////
// Misc functions
////////////////////////////////////////////////////

#if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)

	void MAC_WriteReg(char reg_h, char reg_l, char val)
	{
		MAC_SendI2CStart();

		MAC_SendI2CByte(MAC_WR_ADDR);    //write to W3100A
		MAC_SendI2CByte(reg_h);            //addr (hi byte)
		MAC_SendI2CByte(reg_l);            //addr (low byte)
		MAC_SendI2CByte(val);            //data

		MAC_SendI2CStop();
	}

	char MAC_ReadReg(char reg_h, char reg_l)
	{
		char retVal;

		MAC_SendI2CStart();
		MAC_SendI2CByte(MAC_WR_ADDR);    //write to W3100A
		MAC_SendI2CByte(reg_h);            //addr (hi byte)
		MAC_SendI2CByte(reg_l);            //addr (low byte)
		MAC_SendI2CStop();

		MAC_SendI2CStart();
		MAC_SendI2CByte(MAC_RD_ADDR);    //read from W3100A
		retVal = MAC_ReadI2CByte(1);    //read data
		MAC_SendI2CStop();

		return (retVal);
	}

	void MAC_WritePtr(char reg_h, char reg_l)
	{
		MAC_SendI2CStart();
		MAC_SendI2CByte(MAC_WR_ADDR);		//write to W3100A
		MAC_SendI2CByte(reg_h);				//addr (hi byte)
		MAC_SendI2CByte(reg_l);				//addr (low byte)
		MAC_SendI2CByte(MAC_PTR_VAL[3]);	//data
		MAC_SendI2CByte(MAC_PTR_VAL[2]);
		MAC_SendI2CByte(MAC_PTR_VAL[1]);
		MAC_SendI2CByte(MAC_PTR_VAL[0]);
		MAC_SendI2CStop();

		MX_MAC_SHORTDELAY
	}

	void MAC_ReadPtr(char reg_h, char reg_l)
	{
		MAC_SendI2CStart();
		MAC_SendI2CByte(MAC_WR_ADDR);			//write to W3100A
		MAC_SendI2CByte(reg_h);					//addr (hi byte)
		MAC_SendI2CByte(reg_l);					//addr (low byte)
		MAC_SendI2CStop();

		MAC_SendI2CStart();
		MAC_SendI2CByte(MAC_RD_ADDR);			//read from W3100A
		MAC_PTR_VAL[3] = MAC_ReadI2CByte(0);	//read data
		MAC_PTR_VAL[2] = MAC_ReadI2CByte(0);
		MAC_PTR_VAL[1] = MAC_ReadI2CByte(0);
		MAC_PTR_VAL[0] = MAC_ReadI2CByte(1);
		MAC_SendI2CStop();
	}

	void MAC_ReadPtrWithShadow(char reg_shadow, char reg_pointer)
	{
		//read the shadow reg first
		MAC_ReadReg(SHADOW_REG_HI, reg_shadow);
		MX_MAC_SHORTDELAY

		//now read the actual register
		MAC_ReadPtr(0, reg_pointer);
		MX_MAC_SHORTDELAY
	}

#elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)

	void MAC_WriteByte(unsigned short addr, char data)
	{
		//TODO: disable interrupts?
		clear_bit(%a_CS_PORT, %a_CS_PIN);	//enable SPI device
		//TODO: may need a small delay here (fast chips only??) - and also in other similar places.
		MAC_SPI_Byte(0xF0);									//Write Op Code
		MAC_SPI_Byte((addr & 0xFF00) >> 8);
		MAC_SPI_Byte((addr & 0xFF));
		MAC_SPI_Byte(data);
		set_bit(%a_CS_PORT, %a_CS_PIN);		//disable SPI device
		//TODO: re-enable interrupts?
	}

	char MAC_ReadByte(unsigned short addr)
	{
		char retVal;

		//TODO: disable interrupts
		clear_bit(%a_CS_PORT, %a_CS_PIN);	//enable SPI device
		MAC_SPI_Byte(0x0F);									//Read Op Code
		MAC_SPI_Byte((addr & 0xFF00) >> 8);
		MAC_SPI_Byte((addr & 0xFF));
		retVal = MAC_SPI_Byte(0xAA);
		set_bit(%a_CS_PORT, %a_CS_PIN);		//disable SPI device
		//TODO: re-enable interrupts

		return (retVal);
	}

	void MAC_WriteWord(unsigned short addr, unsigned short data)
	{
		//TODO: disable interrupts
		char data_lo = (data & 0xFF);
		char data_hi = (data & 0xFF00) >> 8;
		char addr_lo = (addr & 0xFF);
		char addr_hi = (addr & 0xFF00) >> 8;

		//write hi byte
		clear_bit(%a_CS_PORT, %a_CS_PIN);	//enable SPI device
		MAC_SPI_Byte(0xF0);									//Write Op Code
		MAC_SPI_Byte(addr_hi);
		MAC_SPI_Byte(addr_lo);
		MAC_SPI_Byte(data_hi);
		set_bit(%a_CS_PORT, %a_CS_PIN);		//disable SPI device
		MX_MAC_SHORTDELAY									//need a delay of at least 32 ns here

		//write lo byte
		clear_bit(%a_CS_PORT, %a_CS_PIN);	//enable SPI device
		MAC_SPI_Byte(0xF0);									//Write Op Code
		MAC_SPI_Byte(addr_hi);
		MAC_SPI_Byte(addr_lo + 1);
		MAC_SPI_Byte(data_lo);
		set_bit(%a_CS_PORT, %a_CS_PIN);		//disable SPI device
		MX_MAC_SHORTDELAY									//need a delay of at least 32 ns here

		//TODO: re-enable interrupts
	}

	unsigned short MAC_ReadWord(unsigned short addr)
	{
		unsigned short retVal;
		char addr_lo = (addr & 0xFF);
		char addr_hi = (addr & 0xFF00) >> 8;

		//TODO: disable interrupts

		//read hi byte
		clear_bit(%a_CS_PORT, %a_CS_PIN);	//enable SPI device
		MAC_SPI_Byte(0x0F);									//Write Op Code
		MAC_SPI_Byte(addr_hi);
		MAC_SPI_Byte(addr_lo);
		retVal = MAC_SPI_Byte(0xaa);
		set_bit(%a_CS_PORT, %a_CS_PIN);		//disable SPI device
		MX_MAC_SHORTDELAY									//need a delay of at least 32 ns here

		retVal = retVal << 8;

		//read lo byte
		clear_bit(%a_CS_PORT, %a_CS_PIN);	//enable SPI device
		MAC_SPI_Byte(0x0F);									//Write Op Code
		MAC_SPI_Byte(addr_hi);
		MAC_SPI_Byte(addr_lo+1);
		retVal = retVal | MAC_SPI_Byte(0xaa);
		set_bit(%a_CS_PORT, %a_CS_PIN);		//disable SPI device
		MX_MAC_SHORTDELAY									//need a delay of at least 32 ns here
		//TODO: re-enable interrupts

		return (retVal);
	}

#elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

	#error "TODO: MIAC stuff here"

#else

	#error "Error - no Internet bus type defined."

#endif




void MAC_Reset_Pointers(char channel)
{
    char ssr;

  #if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)

    //make all pointers equal zero
    MAC_PTR_VAL[0] = 0;
    MAC_PTR_VAL[1] = 0;
    MAC_PTR_VAL[2] = 0;
    MAC_PTR_VAL[3] = 0;

    //Update RR_PR
    MAC_WritePtr(0, REG_Cx_RR_PR(channel));

    //Update RW_PR
    MAC_WritePtr(0, REG_Cx_RW_PR(channel));

    //read SSR to see which mode we are in...
    ssr = MAC_ReadReg(0, REG_Cx_SSR(channel));

    //read W3100A's TR (or TA) pointer
    if (ssr <= 0x0E)
    {
        //TCP mode, so need "random" pointer value
        MAC_PTR_VAL[3] = 0x12;
        MAC_PTR_VAL[2] = 0x34;
        MAC_PTR_VAL[1] = tmr0;
        MAC_PTR_VAL[0] = ~tmr0;
    }

    //Update TW_PR
    MAC_WritePtr(0, REG_Cx_TW_PR(channel));

    //Update TR_PR
    MAC_WritePtr(0, REG_Cx_TR_PR(channel));

    //Update TA_PR
    MAC_WritePtr(0, REG_Cx_TA_PR(channel));

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)

    //make all pointers equal zero
	//MAC_WriteWord(W5100_Sn_TX_RD(channel), 0);
	//MAC_WriteWord(W5100_Sn_TX_WR(channel), 0);
	//MAC_WriteWord(W5100_Sn_RX_RD(channel), 0);

	//gTxSize[channel] = 0;

/*
	#ifdef MAC_DEBUG
		unsigned short word;

		MXDBG_Clear();

		word = MAC_ReadWord(W5100_Sn_TX_RD(channel));
		MXDBG_PrintNumber(word);
		MXDBG_PrintASCII(' ');
		word = MAC_ReadWord(W5100_Sn_TX_WR(channel));
		MXDBG_PrintNumber(word);

		MXDBG_Cursor(0, 1);

		MAC_WriteWord(W5100_Sn_TX_RD(channel), 0);
		MAC_WriteWord(W5100_Sn_TX_WR(channel), 0);

		word = MAC_ReadWord(W5100_Sn_TX_RD(channel));
		MXDBG_PrintNumber(word);
		MXDBG_PrintASCII(' ');
		word = MAC_ReadWord(W5100_Sn_TX_WR(channel));
		MXDBG_PrintNumber(word);

		FCM_WAIT_FOR_KEY(1,0);
		MXDBG_Clear();
	#endif
*/

	#warning "TODO? MAC_Reset_Pointers: do we need a random seed for TCP comms??"
	#warning "TODO? MAC_Reset_Pointers: do we need even need to reset these pointers??"

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

	#error "TODO: MIAC stuff here"

  #else

	#error "Error - no Internet bus type defined."

  #endif
}

#endif   //MX_TCPIP_STATIC_DEFINES
//////////////////////////////////////////////////////////////////////////////////////////


#define %a_W3100A_WR     %b
#define %a_W3100A_RD     %b+1
//#define %a_I2C_INT       %c
//#define %a_I2C_INTMASK   %d
#define %a_INT_PIN       %c
#define %a_INT_MASK      %d
#define %a_I2C_BAUD      %e
#define %a_INT_PORT      port%l
#define %a_INT_TRIS      tris%l

/*e.g.
#define TCPIP_1509580_W3100A_WR     14
#define TCPIP_1509580_W3100A_RD     14+1
#define TCPIP_1509580_I2C_INT       0
#define TCPIP_1509580_I2C_INTMASK   1
#define TCPIP_1509580_I2C_BAUD      48
//Real baud = 100310.204081633
//Error = 0.31%
*/

//GAR
%f
/*e.g.
#define TCPIP_1509580_GAR_0         192
#define TCPIP_1509580_GAR_1         168
#define TCPIP_1509580_GAR_2         0
#define TCPIP_1509580_GAR_3         10
*/

//SMR
%g
/*e.g.
#define TCPIP_1509580_SMR_0         255
#define TCPIP_1509580_SMR_1         255
#define TCPIP_1509580_SMR_2         255
#define TCPIP_1509580_SMR_3         0
*/

//SHAR
%h
/*e.g.
#define TCPIP_1509580_SHAR_0         0
#define TCPIP_1509580_SHAR_1         8
#define TCPIP_1509580_SHAR_2         220
#define TCPIP_1509580_SHAR_3         0
#define TCPIP_1509580_SHAR_4         0
#define TCPIP_1509580_SHAR_5         0
*/

//SIPR
%i
/*e.g.
#define TCPIP_1509580_SIPR_0         192
#define TCPIP_1509580_SIPR_1         168
#define TCPIP_1509580_SIPR_2         0
#define TCPIP_1509580_SIPR_3         2
*/

//#define %a_DEBUG

#ifdef %a_DEBUG
  #define %a_DBG_IDX_MAX 40
  char %a_DBG_VALS[%a_DBG_IDX_MAX];
  char %a_DBG_IDX;
#endif

#define %a_MAX_CHANNELS        4
#define %a_CHANNEL_SIZE        4
#define %a_ARRAY_SIZE          16

//prototypes
char %a_Check_Int();

//globals
char %a_MY_IP[4];
char %a_HEADER[4*8];

#if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)
	char %a_ARR_IDX;
    char %a_STATUS[%a_MAX_CHANNELS];
	char %a_TX_PTR[%a_ARRAY_SIZE];
	char %a_RX_PTR[%a_ARRAY_SIZE];
	char %a_RX_END_PTR[%a_ARRAY_SIZE];

	void %a_Inc_RX_Ptr(char channel);
#endif

char %a_Check_Int()
{
    //periodically query this and populate %a_STATUS[]
    char retVal = 0;
    char in_val;
    char status;
    char isr_val;

    //make sure we are addressing this device
    MAC_WR_ADDR = %a_W3100A_WR;

  #if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)

    //read the appropriate port
    #ifdef MX_I2C_B
      in_val = %a_INT_PORT;
    #endif

    #ifdef MX_I2C_C
      in_val = %a_INT_PORT;
    #endif

    if ((in_val & %a_INT_MASK) == 0)
    {
        //the interrupt pin is set
        status = MAC_ReadReg(0, REG_IR);
        retVal = status;

        while (status)
        {

            if (status & 0x01)
            {
                //channel 0 interrupt
                isr_val = MAC_ReadReg(0, REG_Cx_ISR(0));
                MAC_WriteReg(0, REG_IR, 0x01);
                %a_STATUS[0] = isr_val;
            }
            if (status & 0x02)
            {
                //channel 1 interrupt
                isr_val = MAC_ReadReg(0, REG_Cx_ISR(1));
                MAC_WriteReg(0, REG_IR, 0x02);
                %a_STATUS[1] = isr_val;
            }
            if (status & 0x04)
            {
                //channel 2 interrupt
                isr_val = MAC_ReadReg(0, REG_Cx_ISR(2));
                MAC_WriteReg(0, REG_IR, 0x04);
                %a_STATUS[2] = isr_val;
            }
            if (status & 0x08)
            {
                //channel 3 interrupt
                isr_val = MAC_ReadReg(0, REG_Cx_ISR(3));
                MAC_WriteReg(0, REG_IR, 0x08);
                %a_STATUS[3] = isr_val;
            }

            if (status & 0x10)
            {
                //channel 0 receive interrupt
                MAC_WriteReg(0, REG_IR, 0x10);
            }
            if (status & 0x20)
            {
                //channel 1 receive interrupt
                MAC_WriteReg(0, REG_IR, 0x20);
            }
            if (status & 0x40)
            {
                //channel 2 receive interrupt
                MAC_WriteReg(0, REG_IR, 0x40);
            }
            if (status & 0x80)
            {
                //channel 3 receive interrupt
                MAC_WriteReg(0, REG_IR, 0x80);
            }

            status = MAC_ReadReg(0, REG_IR);
        }
        MAC_WriteReg(0, REG_IR, 0xFF);
    }
  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)

	#warning "TODO? a_Check_Int"

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

	#error "TODO: MIAC stuff here"

  #else

	#error "Error - no Internet bus type defined."

  #endif

    return (retVal);
}


#if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)

	void %a_Inc_RX_Ptr(char channel)
	{
		%a_ARR_IDX = channel*%a_CHANNEL_SIZE;

		%a_RX_PTR[%a_ARR_IDX+0] = %a_RX_PTR[%a_ARR_IDX+0] + 1;
		if (%a_RX_PTR[%a_ARR_IDX+0] == 0)
		{
			%a_RX_PTR[%a_ARR_IDX+1] = %a_RX_PTR[%a_ARR_IDX+1] + 1;
			if (%a_RX_PTR[%a_ARR_IDX+1] == 0)
			{
				%a_RX_PTR[%a_ARR_IDX+2] = %a_RX_PTR[%a_ARR_IDX+2] + 1;
				if (%a_RX_PTR[%a_ARR_IDX+2] == 0)
				{
					%a_RX_PTR[%a_ARR_IDX+3] = %a_RX_PTR[%a_ARR_IDX+3] + 1;
				}
			}
		}
	}

#endif

/*DefinesCode_End*/

/*InitialisationCode_Start*/
/*InitialisationCode_End*/

/*InterruptCode_Start*/
/*InterruptCode_End*/

/********************************************************************
 * FUNCTIONS
 ********************************************************************/

void Initialize()
{
/*Macro_Initialize_Start*/
    //set up the default IP and initialise the internet device
    %a_MY_IP[0] = %a_SIPR_0;
    %a_MY_IP[1] = %a_SIPR_1;
    %a_MY_IP[2] = %a_SIPR_2;
    %a_MY_IP[3] = %a_SIPR_3;

    //make sure we are addressing this device
    MAC_WR_ADDR = %a_W3100A_WR;

  #if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)

    //initialise i2c
    #ifndef MX_MI2C
		#error "Chip does not have master I2C capability"
    #endif

    #ifdef MX_I2C_B
        set_bit(trisb, MX_I2C_SDA);
        set_bit(trisb, MX_I2C_SCL);
        set_bit(%a_INT_TRIS, %a_INT_PIN);
    #endif

    #ifdef MX_I2C_C
        set_bit(trisc, MX_I2C_SDA);
        set_bit(trisc, MX_I2C_SCL);
        set_bit(%a_INT_TRIS, %a_INT_PIN);
    #endif

    //sspadd=48;      //48 = 100kHz, 11 = 400kHz
    sspadd = %a_I2C_BAUD;

    //sspstat = 0x80;
    sspstat = 0;
    st_bit(sspstat,SMP);

    sspcon2 = 0;

    #ifdef SSPCON
        //sspcon=0x38;    //i2c master mode, SPEN
        sspcon = 0;
        st_bit(sspcon,SSPM3);
        st_bit(sspcon,CKP);
        st_bit(sspcon,SSPEN);
    #endif
    #ifdef SSPCON1
        //sspcon1=0x38;    //i2c master mode, SPEN
        sspcon1 = 0;
        st_bit(sspcon1,SSPM3);
        st_bit(sspcon1,CKP);
        st_bit(sspcon1,SSPEN);
    #endif

    //InitW3100A();
    MAC_WriteReg(0, REG_Cx_CR(0), CSW_RESET);  //send a software reset

    //InitNetConfig();
    //Set the gateway IP address
    MAC_WriteReg(0, REG_GAR+0, %a_GAR_0);
    MAC_WriteReg(0, REG_GAR+1, %a_GAR_1);
    MAC_WriteReg(0, REG_GAR+2, %a_GAR_2);
    MAC_WriteReg(0, REG_GAR+3, %a_GAR_3);

    //Set the subnet mask
    MAC_WriteReg(0, REG_SMR+0, %a_SMR_0);
    MAC_WriteReg(0, REG_SMR+1, %a_SMR_1);
    MAC_WriteReg(0, REG_SMR+2, %a_SMR_2);
    MAC_WriteReg(0, REG_SMR+3, %a_SMR_3);

    //Set the MAC address
    MAC_WriteReg(0, REG_SHAR+0, %a_SHAR_0);
    MAC_WriteReg(0, REG_SHAR+1, %a_SHAR_1);
    MAC_WriteReg(0, REG_SHAR+2, %a_SHAR_2);
    MAC_WriteReg(0, REG_SHAR+3, %a_SHAR_3);
    MAC_WriteReg(0, REG_SHAR+4, %a_SHAR_4);
    MAC_WriteReg(0, REG_SHAR+5, %a_SHAR_5);

    //Set the source IP address
    MAC_WriteReg(0, REG_SIPR+0, %a_MY_IP[0]);
    MAC_WriteReg(0, REG_SIPR+1, %a_MY_IP[1]);
    MAC_WriteReg(0, REG_SIPR+2, %a_MY_IP[2]);
    MAC_WriteReg(0, REG_SIPR+3, %a_MY_IP[3]);

    //Set the memory size registers
    MAC_WriteReg(0, REG_RMSR, RMSR_VAL);
    MAC_WriteReg(0, REG_TMSR, TMSR_VAL);

    //Set the TCP timeout values
    //Internet_WriteReg(0, REG_IRTR, IRTR_HI);
    //Internet_WriteReg(0, REG_IRTR+1, IRTR_LO);
    //Internet_WriteReg(0, REG_RCR, RCR_VAL);

    //send a sys init
    %a_STATUS[0] = 0;
    MAC_WriteReg(0, REG_Cx_CR(0), CSYS_INIT);

    //wait until a SSYS_INIT_OK interrupt
    while (!(%a_STATUS[0] & SSYS_INIT_OK))
    {
        %a_Check_Int();
    }

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)

    //initialise spi
	#ifndef MX_SPI
		#error "Chip does not have SPI capability"
	#endif

	//initialise spi
	#ifdef MX_SPI_B
		set_bit(trisb,MX_SPI_SDI);
		clear_bit(trisb,MX_SPI_SDO);
		clear_bit(trisb,MX_SPI_SCK);
	#endif

	#ifdef MX_SPI_C
		set_bit(trisc,MX_SPI_SDI);
		clear_bit(trisc,MX_SPI_SDO);
		clear_bit(trisc,MX_SPI_SCK);
	#endif

	#ifdef MX_SPI_BCB
		set_bit(trisb,MX_SPI_SDI);
		clear_bit(trisc,MX_SPI_SDO);
		clear_bit(trisb,MX_SPI_SCK);
	#endif

	clear_bit(%a_CS_TRIS,%a_CS_PIN);		//chip select is an output
	set_bit(%a_CS_PORT, %a_CS_PIN);		//disable SPI device

	sspstat = 64;

	#ifdef _BOOSTC
		#ifdef SSPCON							// SPI Speed Initialise
			sspcon = 1;
			st_bit(sspcon, SSPEN);
		#endif
		#ifdef SSPCON1
			sspcon1 = 1;
			st_bit(sspcon1, SSPEN);
		#endif
	#endif

	#ifdef HI_TECH_C
		#ifdef sspcon							// SPI Speed Initialise
			sspcon = 1;
			st_bit(sspcon, SSPEN);
		#endif
		#ifdef sspcon1
			sspcon1 = 1;
			st_bit(sspcon1, SSPEN);
		#endif
	#endif


    //Init W5100
    MAC_WriteByte(W5100_MR, W5100_MR_RST);  //send a software reset

/*
	#ifdef MAC_DEBUG
		char x,y;
		unsigned short i, j, start, end;
		start = 0x0500;
		end = 0x0529;

		MXDBG_Clear();
		for (i=start; i<=end; i=i+4)
		{
			for (j=0; j<4; j++)
			{
				if ((j==2) || (j==3))
				{
					y = 1;
				} else {
					y = 0;
				}
				if ((j==0) || (j==2))
				{
					x = 0;
				} else {
					x = 8;
				}
				MXDBG_Cursor(x, y);
				MXDBG_PrintNumber(i+j);
				MXDBG_PrintASCII(':');
				cDebug = MAC_ReadByte(i+j);
				MXDBG_PrintNumber(cDebug);
				MXDBG_PrintASCII(' ');
			}
			FCM_WAIT_FOR_KEY(1,0);
			MXDBG_Clear();
		}
	#endif
*/

    //TODO: Set the interrupt mask
    MAC_WriteByte(W5100_IMR+0, 0);

    //Set the gateway IP address
    MAC_WriteByte(W5100_GAR+0, %a_GAR_0);
    MAC_WriteByte(W5100_GAR+1, %a_GAR_1);
    MAC_WriteByte(W5100_GAR+2, %a_GAR_2);
    MAC_WriteByte(W5100_GAR+3, %a_GAR_3);

    //Set the subnet mask
    MAC_WriteByte(W5100_SUBR+0, %a_SMR_0);
    MAC_WriteByte(W5100_SUBR+1, %a_SMR_1);
    MAC_WriteByte(W5100_SUBR+2, %a_SMR_2);
    MAC_WriteByte(W5100_SUBR+3, %a_SMR_3);

    //Set the MAC address
    MAC_WriteByte(W5100_SHAR+0, %a_SHAR_0);
    MAC_WriteByte(W5100_SHAR+1, %a_SHAR_1);
    MAC_WriteByte(W5100_SHAR+2, %a_SHAR_2);
    MAC_WriteByte(W5100_SHAR+3, %a_SHAR_3);
    MAC_WriteByte(W5100_SHAR+4, %a_SHAR_4);
    MAC_WriteByte(W5100_SHAR+5, %a_SHAR_5);

    //Set the source IP address
    MAC_WriteByte(W5100_SIPR+0, %a_MY_IP[0]);
    MAC_WriteByte(W5100_SIPR+1, %a_MY_IP[1]);
    MAC_WriteByte(W5100_SIPR+2, %a_MY_IP[2]);
    MAC_WriteByte(W5100_SIPR+3, %a_MY_IP[3]);

    //Set the memory size registers
    MAC_WriteByte(W5100_RMSR, RMSR_VAL);
    MAC_WriteByte(W5100_TMSR, TMSR_VAL);

    //Set the TCP timeout values
    MAC_WriteByte(W5100_RTR+0, IRTR_HI);
    MAC_WriteByte(W5100_RTR+1, IRTR_LO);
    MAC_WriteByte(W5100_RCR, RCR_VAL);


  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

	#error "TODO: MIAC stuff here"

  #else

	#error "Error - no Internet bus type defined."

  #endif


/*Macro_Initialize_End*/
}



char Create_MAC_Socket(char promiscuous, char broadcast, char error)
{
/*Macro_Create_MAC_Socket_Start*/
	//NOTE: this is always channel 0

	//make sure we are addressing this device
	MAC_WR_ADDR = %a_W3100A_WR;

  #if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)

	//Create the MAC layer socket
	char regval = SOCK_MACL_RAW;
	if (promiscuous)
	{
		regval = regval | MACLOPT_PROMISC;
	}
	if (broadcast)
	{
		regval = regval | MACLOPT_BROADCAST;
	}
	if (error)
	{
		regval = regval | MACLOPT_RXERR;
	}

	//set up the socket protocol and option flags
	MAC_WriteReg(0, REG_Cx_SOPR(0), regval);

	//initialise the socket
	%a_STATUS[0] = 0;
	MAC_WriteReg(0, REG_Cx_CR(0), CSOCK_INIT);

	//check for result
	while (%a_STATUS[0] == 0)
	{
		%a_Check_Int();
	}

	//check the result
	if ((%a_STATUS[0] & SSOCK_INIT_OK) == 0)
	{
		return (0);  //failed to open the socket
	}

	MAC_Reset_Pointers(0);

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100) // ST: Done

	// Make sure we close the socket first
	if (MAC_ReadByte(W5100_Sn_SR(0)) != W5100_SOCK_CLOSED)
	{
		MAC_WriteByte(W5100_Sn_CR(0), W5100_Sn_CR_CLOSE);

		// Waiting until the S0_CR is clear
		while(MAC_ReadByte(W5100_Sn_CR(0)));
	}

	#warning "TODO? Create_MAC_Socket - promiscuous; broadcast; error all must be zero???"

	MAC_Reset_Pointers(0);

    //make all pointers equal zero
	//MAC_WriteWord(W5100_Sn_TX_RD(channel), 0);
	//MAC_WriteWord(W5100_Sn_TX_WR(channel), 0);
	//MAC_WriteWord(W5100_Sn_RX_RD(channel), 0);

	//gTxSize[channel] = 0;


	while(MAC_ReadByte(W5100_Sn_SR(0)) != W5100_SOCK_MACRAW)
	{
		//Create the MAC layer socket
		char regval = W5100_Sn_MR_MACRAW;
		//if (promiscuous)
		//{
		//	regval = regval | MACLOPT_PROMISC;
		//}
		//if (broadcast)
		//{
		//	regval = regval | MACLOPT_BROADCAST;
		//}
		//if (error)
		//{
		//	regval = regval | MACLOPT_RXERR;
		//}

		//set up the socket protocol and option flags
		MAC_WriteByte(W5100_Sn_MR(0), regval);

		//initialise the socket
		MAC_WriteByte(W5100_Sn_CR(0), W5100_Sn_CR_OPEN);

		// Waiting until the S0_CR is clear
		while(MAC_ReadByte(W5100_Sn_CR(0)));
	}

/*
	#ifdef MAC_DEBUG
		char x,y;
		unsigned short i, j, start, end;
		start = 0x0400;
		end = 0x0429;

		MXDBG_Clear();
		for (i=start; i<=end; i=i+4)
		{
			for (j=0; j<4; j++)
			{
				if ((j==2) || (j==3))
				{
					y = 1;
				} else {
					y = 0;
				}
				if ((j==0) || (j==2))
				{
					x = 0;
				} else {
					x = 8;
				}
				MXDBG_Cursor(x, y);
				MXDBG_PrintNumber(i+j);
				MXDBG_PrintASCII(':');
				cDebug = MAC_ReadByte(i+j);
				MXDBG_PrintNumber(cDebug);
				MXDBG_PrintASCII(' ');
			}
			FCM_WAIT_FOR_KEY(1,0);
			MXDBG_Clear();
		}
	#endif
*/
  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

	#error "TODO: MIAC stuff here"

  #else

	#error "Error - no Internet bus type defined."

  #endif


	//reset globals
	%a_HEADER[0] = 0;
	%a_HEADER[1] = 0;
	%a_HEADER[2] = 0;
	%a_HEADER[3] = 0;
	%a_HEADER[4] = 0;
	%a_HEADER[5] = 0;
	%a_HEADER[6] = 0;
	%a_HEADER[7] = 0;


    return (1);
/*Macro_Create_MAC_Socket_End*/
}

char Create_IP_Socket(char channel, char protocol, char broadcast)
{
/*Macro_Create_IP_Socket_Start*/
	//make sure we are addressing this device
	MAC_WR_ADDR = %a_W3100A_WR;

  #if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)

	char hdr_idx_base = channel*8;

	if (channel > %a_MAX_CHANNELS)
	{
		return (0);  //error trying to open a non-existent channel
	}

	//set up the socket protocol and option flags
	if (broadcast)
	{
		MAC_WriteReg(0, REG_Cx_SOPR(channel), SOCK_IPL_RAW | SOCKOPT_BROADCAST);
	} else {
		MAC_WriteReg(0, REG_Cx_SOPR(channel), SOCK_IPL_RAW);
	}

	//set up the protocol
	MAC_WriteReg(0, REG_Cx_IPR(channel), protocol);

	//initialise the socket
	%a_STATUS[channel] = 0;
	MAC_WriteReg(0, REG_Cx_CR(channel), CSOCK_INIT);

	//check for result
	while (%a_STATUS[channel] == 0)
	{
		%a_Check_Int();
	}

	//check the result
	if ((%a_STATUS[channel] & SSOCK_INIT_OK) == 0)
	{
		return (0);  //failed to open the socket
	}

	MAC_Reset_Pointers(channel);

	//reset globals
	%a_HEADER[hdr_idx_base+0] = 0;
	%a_HEADER[hdr_idx_base+1] = 0;
	%a_HEADER[hdr_idx_base+2] = 0;
	%a_HEADER[hdr_idx_base+3] = 0;
	%a_HEADER[hdr_idx_base+4] = 0;
	%a_HEADER[hdr_idx_base+5] = 0;
	%a_HEADER[hdr_idx_base+6] = 0;
	%a_HEADER[hdr_idx_base+7] = 0;

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100) // JW: Done

	// Make sure we close the socket first
	if (MAC_ReadByte(W5100_Sn_SR(channel)) != W5100_SOCK_CLOSED)
	{
		MAC_WriteByte(W5100_Sn_CR(channel), W5100_Sn_CR_CLOSE);
		// Waiting until the S0_CR is clear
		while(MAC_ReadByte(W5100_Sn_CR(channel)));
	}
	#warning "TODO? Create_IP_Socket - broadcast must be zero???"
	MAC_Reset_Pointers(channel);

	while(MAC_ReadByte(W5100_Sn_SR(channel)) != W5100_SOCK_IPRAW)
	{
		//Create the IP (raw) layer socket
		char regval = W5100_Sn_MR_IPRAW;
		//if (broadcast)
		//{
		//	regval = regval | MACLOPT_BROADCAST;
		//}
		//set up the socket protocol and option flags
		MAC_WriteByte(W5100_Sn_MR(channel), regval);
        // Set the protocol (eg W5100_IPPROTO_UDP)
        MAC_WriteByte(W5100_Sn_PROTO(channel), protocol);
		//initialise the socket
		MAC_WriteByte(W5100_Sn_CR(channel), W5100_Sn_CR_OPEN);
		// Waiting until the S0_CR is clear
		while(MAC_ReadByte(W5100_Sn_CR(channel)));
	}

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

	#error "TODO: MIAC stuff here"

  #else

	#error "Error - no Internet bus type defined."

  #endif

	return (1);
/*Macro_Create_IP_Socket_End*/
}

char Create_UDP_Socket(char channel, char src_port_hi, char src_port_lo)
{
/*Macro_Create_UDP_Socket_Start*/
	//make sure we are addressing this device
	MAC_WR_ADDR = %a_W3100A_WR;

  #if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)

	char hdr_idx_base = channel*8;

	if (channel > %a_MAX_CHANNELS)
	{
		return (0);  //error trying to open a non-existent channel
	}

	//set up the socket protocol and option flags
	MAC_WriteReg(0, REG_Cx_SOPR(channel), SOCK_DGRAM);

	//set up the source port
	MAC_WriteReg(0, REG_Cx_SPR(channel),   src_port_hi);
	MAC_WriteReg(0, REG_Cx_SPR(channel)+1, src_port_lo);

	//initialise the socket
	%a_STATUS[channel] = 0;
	MAC_WriteReg(0, REG_Cx_CR(channel), CSOCK_INIT);

	//check for result
	while (%a_STATUS[channel] == 0)
	{
		%a_Check_Int();
	}

	//check the result
	if ((%a_STATUS[channel] & SSOCK_INIT_OK) == 0)
	{
		return (0);  //failed to open the socket
	}

	MAC_Reset_Pointers(channel);

	//reset globals
	%a_HEADER[hdr_idx_base+0] = 0;
	%a_HEADER[hdr_idx_base+1] = 0;
	%a_HEADER[hdr_idx_base+2] = 0;
	%a_HEADER[hdr_idx_base+3] = 0;
	%a_HEADER[hdr_idx_base+4] = 0;
	%a_HEADER[hdr_idx_base+5] = 0;
	%a_HEADER[hdr_idx_base+6] = 0;
	%a_HEADER[hdr_idx_base+7] = 0;

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)

	// Make sure we close the socket first
	if (MAC_ReadByte(W5100_Sn_SR(channel)) != W5100_SOCK_CLOSED)
	{
		MAC_WriteByte(W5100_Sn_CR(channel), W5100_Sn_CR_CLOSE);
		// Waiting until the S0_CR is clear
		while(MAC_ReadByte(W5100_Sn_CR(channel)));
	}
	#warning "TODO? Create_UDP_Socket - check order of port high/low???"
	MAC_Reset_Pointers(channel);
	// Open the socket
	while(MAC_ReadByte(W5100_Sn_SR(channel)) != W5100_SOCK_UDP)
	{
		//Create the UDP layer socket
		char regval = W5100_Sn_MR_UDP;
		//set up the socket protocol and option flags
		MAC_WriteByte(W5100_Sn_MR(channel), regval);
        // Set the protocol (eg W5100_IPPROTO_UDP)
        MAC_WriteByte(W5100_Sn_PORT(channel) + 0, src_port_hi);
        MAC_WriteByte(W5100_Sn_PORT(channel) + 1, src_port_lo);
		//initialise the socket
		MAC_WriteByte(W5100_Sn_CR(channel), W5100_Sn_CR_OPEN);
		// Waiting until the S0_CR is clear
		while(MAC_ReadByte(W5100_Sn_CR(channel)));
	}

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

	#error "TODO: MIAC stuff here"

  #else

	#error "Error - no Internet bus type defined."

  #endif

    return (1);
/*Macro_Create_UDP_Socket_End*/
}

char Create_TCP_Socket(char channel, char src_port_hi, char src_port_lo)
{
/*Macro_Create_TCP_Socket_Start*/

	//make sure we are addressing this device
	MAC_WR_ADDR = %a_W3100A_WR;

  #if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)

	char hdr_idx_base = channel*8;

	if (channel > %a_MAX_CHANNELS)
	{
		return (0);  //error trying to open a non-existent channel
	}

	//set up the socket protocol and option flags
	MAC_WriteReg(0, REG_Cx_SOPR(channel), SOCK_STREAM);

	//set up the source port
	MAC_WriteReg(0, REG_Cx_SPR(channel),   src_port_hi);
	MAC_WriteReg(0, REG_Cx_SPR(channel)+1, src_port_lo);

	//initialise the socket
	%a_STATUS[channel] = 0;
	MAC_WriteReg(0, REG_Cx_CR(channel), CSOCK_INIT);

	//check for result
	while (%a_STATUS[channel] == 0)
	{
		%a_Check_Int();
	}

	//check the result
	if ((%a_STATUS[channel] & SSOCK_INIT_OK) == 0)
	{
		return (0);  //failed to open the socket
	}

	MAC_Reset_Pointers(channel);

	//reset globals
	%a_HEADER[hdr_idx_base+0] = 0;
	%a_HEADER[hdr_idx_base+1] = 0;
	%a_HEADER[hdr_idx_base+2] = 0;
	%a_HEADER[hdr_idx_base+3] = 0;
	%a_HEADER[hdr_idx_base+4] = 0;
	%a_HEADER[hdr_idx_base+5] = 0;
	%a_HEADER[hdr_idx_base+6] = 0;
	%a_HEADER[hdr_idx_base+7] = 0;

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)

	// Make sure we close the socket first
	if (MAC_ReadByte(W5100_Sn_SR(channel)) != W5100_SOCK_CLOSED)
	{
		MAC_WriteByte(W5100_Sn_CR(channel), W5100_Sn_CR_CLOSE);
		// Waiting until the S0_CR is clear
		while(MAC_ReadByte(W5100_Sn_CR(channel)));
	}
	#warning "TODO? Create_TCP_Socket - check order of port high/low???"
	MAC_Reset_Pointers(channel);
	// Open the socket
	{
	    char state = MAC_ReadByte(W5100_Sn_SR(channel));
	    while(state != W5100_SOCK_INIT)
	    {
		    //Create the IP (raw) layer socket
		    char regval = W5100_Sn_MR_TCP;
		    //set up the socket protocol and option flags
		    MAC_WriteByte(W5100_Sn_MR(channel), regval);
            // Set the protocol (eg W5100_IPPROTO_UDP)
            MAC_WriteByte(W5100_Sn_PORT(channel) + 0, src_port_hi);
            MAC_WriteByte(W5100_Sn_PORT(channel) + 1, src_port_lo);
		    //initialise the socket
		    MAC_WriteByte(W5100_Sn_CR(channel), W5100_Sn_CR_OPEN);
		    // Waiting until the S0_CR is clear
		    while(MAC_ReadByte(W5100_Sn_CR(channel)));
		    state = MAC_ReadByte(W5100_Sn_SR(channel));
	    }
	}

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

	#error "TODO: MIAC stuff here"

  #else

	#error "Error - no Internet bus type defined."

  #endif

    return (1);
/*Macro_Create_TCP_Socket_End*/
}

char Set_Destination(char channel, char dst_ip0, char dst_ip1, char dst_ip2, char dst_ip3, char dst_port_hi, char dst_port_lo)
{
/*Macro_Set_Destination_Start*/

  #if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)

	if (channel > %a_MAX_CHANNELS)
	{
		return (0);  //error trying to open a non-existent channel
	}

	//set up the destination port
	MAC_WriteReg(0, REG_Cx_DPR(channel),   dst_port_hi);
	MAC_WriteReg(0, REG_Cx_DPR(channel)+1, dst_port_lo);

	//set up the destination IP address
	MAC_WriteReg(0, REG_Cx_DIR(channel),   dst_ip0);
	MAC_WriteReg(0, REG_Cx_DIR(channel)+1, dst_ip1);
	MAC_WriteReg(0, REG_Cx_DIR(channel)+2, dst_ip2);
	MAC_WriteReg(0, REG_Cx_DIR(channel)+3, dst_ip3);

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)

	#warning "TODO? Set_Destination - check order of port/address high/low, and do we need to wait???"

    MAC_WriteByte(W5100_Sn_DIPR(channel) + 0, dst_ip0);
    MAC_WriteByte(W5100_Sn_DIPR(channel) + 1, dst_ip1);
    MAC_WriteByte(W5100_Sn_DIPR(channel) + 2, dst_ip2);
    MAC_WriteByte(W5100_Sn_DIPR(channel) + 3, dst_ip3);

    MAC_WriteByte(W5100_Sn_DPORT(channel) + 0, dst_port_hi);
    MAC_WriteByte(W5100_Sn_DPORT(channel) + 1, dst_port_lo);

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

	#error "TODO: MIAC stuff here"

  #else

	#error "Error - no Internet bus type defined."

  #endif

	return (1);
/*Macro_Set_Destination_End*/
}

void Set_My_IP(char ip0, char ip1, char ip2, char ip3)
{
/*Macro_Set_My_IP_Start*/
	%a_MY_IP[0] = ip0;
	%a_MY_IP[1] = ip1;
	%a_MY_IP[2] = ip2;
	%a_MY_IP[3] = ip3;
/*Macro_Set_My_IP_End*/
}

char GetSocketStatus(char channel)
{
/*Macro_GetSocketStatus_Start*/
	char ssr;

	//make sure we are addressing this device
	MAC_WR_ADDR = %a_W3100A_WR;

  #if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)

	if (channel > %a_MAX_CHANNELS)
	{
		return (255);  //error trying to open a non-existent channel
	}

	ssr = MAC_ReadReg(0, REG_Cx_SSR(channel));

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)

	if (channel > %a_MAX_CHANNELS)
	{
		return (255);  //error trying to open a non-existent channel
	}
	ssr = MAC_ReadByte(W5100_Sn_SR(channel));

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

	#error "TODO: MIAC stuff here"

  #else

	#error "Error - no Internet bus type defined."

  #endif

	return (ssr);
/*Macro_GetSocketStatus_End*/
}

void TCP_Listen(char channel)
{
/*Macro_TCP_Listen_Start*/

	//make sure we are addressing this device
	MAC_WR_ADDR = %a_W3100A_WR;

  #if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)

	%a_STATUS[channel] = 0;
	MAC_WriteReg(0, REG_Cx_CR(channel), CLISTEN);

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)

    MAC_WriteByte(W5100_Sn_CR(channel), W5100_Sn_CR_LISTEN); // Send command 'listen'

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

	#error "TODO: MIAC stuff here"

  #else

	#error "Error - no Internet bus type defined."

  #endif

/*Macro_TCP_Listen_End*/
}

char TCP_Connect(char channel, char dst_ip0, char dst_ip1, char dst_ip2, char dst_ip3, char dst_port_hi, char dst_port_lo)
{
/*Macro_TCP_Connect_Start*/
	char ssr;

	//make sure we are addressing this device
	MAC_WR_ADDR = %a_W3100A_WR;
	if (channel > %a_MAX_CHANNELS)
	{
		return (0);  //error trying to open a non-existent channel
	}

  #if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)

	//we can only "connect" when the socket status is initialising...
	ssr = MAC_ReadReg(0, REG_Cx_SSR(channel));
	if (ssr != 0x0E)
	{
		return (0);
	}

	//set up the destination port
	MAC_WriteReg(0, REG_Cx_DPR(channel),   dst_port_hi);
	MAC_WriteReg(0, REG_Cx_DPR(channel)+1, dst_port_lo);

	//set up the destination IP address
	MAC_WriteReg(0, REG_Cx_DIR(channel),   dst_ip0);
	MAC_WriteReg(0, REG_Cx_DIR(channel)+1, dst_ip1);
	MAC_WriteReg(0, REG_Cx_DIR(channel)+2, dst_ip2);
	MAC_WriteReg(0, REG_Cx_DIR(channel)+3, dst_ip3);

	//connect
	%a_STATUS[channel] = 0;
	MAC_WriteReg(0, REG_Cx_CR(channel), CCONNECT);

	//check for result
	while (%a_STATUS[channel] == 0)
	{
		%a_Check_Int();
	}

	//wait for connection to be established...
	if ((%a_STATUS[channel] & SESTABLISHED) == 0)
	{
		return (1);
	} else {
		return (0);  //failed to open the socket
	}

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)

	//we can only "connect" when the socket status is initialising...

	#warning "TODO? TCP_Connect - check order of port/address high/low, and do we need to wait???"

	//set up the destination port
    MAC_WriteByte(W5100_Sn_DPORT(channel) + 0, dst_port_hi);
    MAC_WriteByte(W5100_Sn_DPORT(channel) + 1, dst_port_lo);

	//set up the destination IP address
    MAC_WriteByte(W5100_Sn_DIPR(channel) + 0, dst_ip0);
    MAC_WriteByte(W5100_Sn_DIPR(channel) + 1, dst_ip1);
    MAC_WriteByte(W5100_Sn_DIPR(channel) + 2, dst_ip2);
    MAC_WriteByte(W5100_Sn_DIPR(channel) + 3, dst_ip3);

	//connect and wait for result to be clear
	MAC_WriteByte(W5100_Sn_CR(channel), W5100_Sn_CR_CONNECT);
	while (MAC_ReadByte(W5100_Sn_CR(channel)));

	//wait for connection to be established...
	// JW: Does interrupt flag(s) need to be cleared first?
	char status;
	do
	{
      // Spin idle until interrupt triggered
	  status = MAC_ReadByte(W5100_Sn_IR(channel));
	} while (status == 0);

	if ((status & W5100_Sn_IR_CON) != 0)
	{
		return (1);
	} else {
		return (0);  //failed to open the socket
	}

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

	#error "TODO: MIAC stuff here"

  #else

	#error "Error - no Internet bus type defined."

  #endif

/*Macro_TCP_Connect_End*/
}

void TCP_Close(char channel)
{
/*Macro_TCP_Close_Start*/

	char ta0, ta1;
	//char ta2, ta3;
	char is_equal;
	char timeout;

	//make sure we are addressing this device
	MAC_WR_ADDR = %a_W3100A_WR;

  #if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)

	//make sure Tx buffer is empty...
	is_equal = 0;
	timeout = 100;
	while (is_equal == 0)
	{
		MAC_ReadPtrWithShadow(Cx_STA_PTR(channel), REG_Cx_TA_PR(channel));
		ta0 = MAC_PTR_VAL[0];
		ta1 = MAC_PTR_VAL[1];
		//ta2 = MAC_PTR_VAL[2];
		//ta3 = MAC_PTR_VAL[3];

		MAC_ReadPtrWithShadow(Cx_STW_PTR(channel), REG_Cx_TW_PR(channel));
		if ((ta0 == MAC_PTR_VAL[0]) && (ta1 == MAC_PTR_VAL[1]))
		{
			//if ((ta2 == MAC_PTR_VAL[2]) && (ta3 == MAC_PTR_VAL[3]))
			//{
				is_equal = 1;
			} else {
				delay_ms(1);
				timeout--;
				if (timeout == 0)
				{
					is_equal = 1;
				}
			//}
		}
	}

	if (timeout != 0)
	{
		//no timeout, so ok to close
		%a_STATUS[channel] = 0;
		MAC_WriteReg(0, REG_Cx_CR(channel), CCLOSE);
	}

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)

	//make sure Tx buffer is empty...
	is_equal = 0;
	timeout = 100;
	while (is_equal == 0)
	{
	    //unsigned short rpos;
	    //unsigned short tpos;
	    //rpos = MAC_ReadWord(W5100_Sn_TX_RD(channel)); // Pos at send
	    //tpos = MAC_ReadWord(W5100_Sn_TX_WR(channel)); // Pos at buffer

		//if (rpos == tpos)
		if((MAC_ReadByte(W5100_Sn_IR(channel)) & W5100_Sn_IR_SEND_OK) != 0)
		{
			is_equal = 1;
		} else {
			delay_ms(1);
			timeout--;
			if (timeout == 0)
			{
				is_equal = 1;
			}
		}
	}

	if (timeout != 0)
	{
		// Not timed out, so ok to close
		MAC_WriteByte(W5100_Sn_CR(0), W5100_Sn_CR_DISCON); // W5100_Sn_CR_CLOSE);
        while(MAC_ReadByte(W5100_Sn_CR(channel)));
        // Disconnected, now close
		MAC_WriteByte(W5100_Sn_CR(0), W5100_Sn_CR_CLOSE);
        while(MAC_ReadByte(W5100_Sn_CR(channel)));
	}

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

	#error "TODO: MIAC stuff here"

  #else

	#error "Error - no Internet bus type defined."

  #endif

/*Macro_TCP_Close_End*/
}

void Tx_start(char channel)
{
/*Macro_Tx_start_Start*/

	char ssr;

	//make sure we are addressing this device
	MAC_WR_ADDR = %a_W3100A_WR;

  #if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)

	%a_ARR_IDX = channel*%a_CHANNEL_SIZE;

	//read SSR to see which mode we are in...
	ssr = MAC_ReadReg(0, REG_Cx_SSR(channel));

	//read W3100A's TR (or TA) pointer
	if (ssr > 0x0E)
	{
		//not TCP mode, so use TR_PR
		MAC_ReadPtrWithShadow(Cx_STR_PTR(channel), REG_Cx_TR_PR(channel));
	} else {
		//TCP mode, so use TA_PR
		MAC_ReadPtrWithShadow(Cx_STA_PTR(channel), REG_Cx_TA_PR(channel));
	}

	//set our local TX_PTR to it
	%a_TX_PTR[%a_ARR_IDX+0] = MAC_PTR_VAL[0];
	%a_TX_PTR[%a_ARR_IDX+1] = MAC_PTR_VAL[1];
	%a_TX_PTR[%a_ARR_IDX+2] = MAC_PTR_VAL[2];
	%a_TX_PTR[%a_ARR_IDX+3] = MAC_PTR_VAL[3];

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)

    /*
      Assuming the correct mode and setup, nothing needs to be done here
      The HW is ready to send bytes with Tx_sendbyte()
    */
	// Read the Tx Write Pointer
	unsigned short tx_ptr_val;
	tx_ptr_val = MAC_ReadWord(W5100_Sn_TX_WR(channel));
	gTxPtr[channel] = W5100_TXMEM(channel) + (tx_ptr_val & W5100_TXMEM_MASK);
	gTxSize[channel] = 0;

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

	#error "TODO: MIAC stuff here"

  #else

	#error "Error - no Internet bus type defined."

  #endif

/*Macro_Tx_start_End*/
}

void Tx_sendbyte(char channel, char data)
{
/*Macro_Tx_sendbyte_Start*/

	char addr_l;
	char addr_h;

	//make sure we are addressing this device
	MAC_WR_ADDR = %a_W3100A_WR;

  #if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)

	%a_ARR_IDX = channel*%a_CHANNEL_SIZE;

	addr_l = %a_TX_PTR[%a_ARR_IDX+0];
	addr_h = (%a_TX_PTR[%a_ARR_IDX+1] & MEM_TX_MASK(channel)) + MEM_TX_OFFSET(channel);

	//send the i2c start bit and the address
	MAC_SendI2CStart();
	MAC_SendI2CByte(%a_W3100A_WR);    //write to W3100A
	MAC_SendI2CByte(addr_h);            //addr (hi byte)
	MAC_SendI2CByte(addr_l);            //addr (low byte)
	MAC_SendI2CByte(data);
	MAC_SendI2CStop();

	%a_TX_PTR[%a_ARR_IDX+0] = %a_TX_PTR[%a_ARR_IDX+0] + 1;
	if (%a_TX_PTR[%a_ARR_IDX+0] == 0)
	{
		%a_TX_PTR[%a_ARR_IDX+1] = %a_TX_PTR[%a_ARR_IDX+1] + 1;
		if (%a_TX_PTR[%a_ARR_IDX+1] == 0)
		{
			%a_TX_PTR[%a_ARR_IDX+2] = %a_TX_PTR[%a_ARR_IDX+2] + 1;
			if (%a_TX_PTR[%a_ARR_IDX+2] == 0)
			{
				%a_TX_PTR[%a_ARR_IDX+3] = %a_TX_PTR[%a_ARR_IDX+3] + 1;
			}
		}
	}

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)

	//add the data to the appropriate place
	MAC_WriteByte(gTxPtr[channel], data);

	//increment the pointer and wrap it around
	gTxPtr[channel]++;
	gTxSize[channel]++;
	if (gTxSize[channel] >= W5100_TXMEM_MASK)
	{
	  // Buffer will overflow, loosing data
	  Tx_end(channel);
	}
	if (gTxPtr[channel] > W5100_TXMEM(channel) + W5100_TXMEM_MASK)
	{
		gTxPtr[channel] = W5100_TXMEM(channel); // Wrap pointer
	}

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

	#error "TODO: MIAC stuff here"

  #else

	#error "Error - no Internet bus type defined."

  #endif

/*Macro_Tx_sendbyte_End*/
}

void Tx_sendmymac(char channel)
{
/*Macro_Tx_sendmymac_Start*/

	char addr_l;
	char addr_h;

	//make sure we are addressing this device
	MAC_WR_ADDR = %a_W3100A_WR;

  #if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)

	%a_ARR_IDX = channel*%a_CHANNEL_SIZE;

	addr_l = %a_TX_PTR[%a_ARR_IDX+0];
	addr_h = (%a_TX_PTR[%a_ARR_IDX+1] & MEM_TX_MASK(channel)) + MEM_TX_OFFSET(channel);

	//send the i2c start bit and the address
	MAC_SendI2CStart();
	MAC_SendI2CByte(%a_W3100A_WR);    //write to W3100A
	MAC_SendI2CByte(addr_h);            //addr (hi byte)
	MAC_SendI2CByte(addr_l);            //addr (low byte)
	MAC_SendI2CByte(%a_SHAR_0);
	MAC_SendI2CByte(%a_SHAR_1);
	MAC_SendI2CByte(%a_SHAR_2);
	MAC_SendI2CByte(%a_SHAR_3);
	MAC_SendI2CByte(%a_SHAR_4);
	MAC_SendI2CByte(%a_SHAR_5);
	MAC_SendI2CStop();

	%a_TX_PTR[%a_ARR_IDX+0] = %a_TX_PTR[%a_ARR_IDX+0] + 6;
	if (%a_TX_PTR[%a_ARR_IDX+0] < 6)
	{
		%a_TX_PTR[%a_ARR_IDX+1] = %a_TX_PTR[%a_ARR_IDX+1] + 1;
		if (%a_TX_PTR[%a_ARR_IDX+1] == 0)
		{
			%a_TX_PTR[%a_ARR_IDX+2] = %a_TX_PTR[%a_ARR_IDX+2] + 1;
			if (%a_TX_PTR[%a_ARR_IDX+2] == 0)
			{
				%a_TX_PTR[%a_ARR_IDX+3] = %a_TX_PTR[%a_ARR_IDX+3] + 1;
			}
		}
	}

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)

	Tx_sendbyte(channel, %a_SHAR_0);
	Tx_sendbyte(channel, %a_SHAR_1);
	Tx_sendbyte(channel, %a_SHAR_2);
	Tx_sendbyte(channel, %a_SHAR_3);
	Tx_sendbyte(channel, %a_SHAR_4);
	Tx_sendbyte(channel, %a_SHAR_5);

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

	#error "TODO: MIAC stuff here"

  #else

	#error "Error - no Internet bus type defined."

  #endif

/*Macro_Tx_sendmymac_End*/
}

void Tx_sendmyip(char channel)
{
/*Macro_Tx_sendmyip_Start*/

	char addr_l;
	char addr_h;

	//make sure we are addressing this device
	MAC_WR_ADDR = %a_W3100A_WR;

  #if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)

	%a_ARR_IDX = channel*%a_CHANNEL_SIZE;

	addr_l = %a_TX_PTR[%a_ARR_IDX+0];
	addr_h = (%a_TX_PTR[%a_ARR_IDX+1] & MEM_TX_MASK(channel)) + MEM_TX_OFFSET(channel);

	//send the i2c start bit and the address
	MAC_SendI2CStart();
	MAC_SendI2CByte(%a_W3100A_WR);    //write to W3100A
	MAC_SendI2CByte(addr_h);            //addr (hi byte)
	MAC_SendI2CByte(addr_l);            //addr (low byte)
	MAC_SendI2CByte(%a_MY_IP[0]);
	MAC_SendI2CByte(%a_MY_IP[1]);
	MAC_SendI2CByte(%a_MY_IP[2]);
	MAC_SendI2CByte(%a_MY_IP[3]);
	MAC_SendI2CStop();

	%a_TX_PTR[%a_ARR_IDX+0] = %a_TX_PTR[%a_ARR_IDX+0] + 4;
	if (%a_TX_PTR[%a_ARR_IDX+0] < 4)
	{
		%a_TX_PTR[%a_ARR_IDX+1] = %a_TX_PTR[%a_ARR_IDX+1] + 1;
		if (%a_TX_PTR[%a_ARR_IDX+1] == 0)
		{
			%a_TX_PTR[%a_ARR_IDX+2] = %a_TX_PTR[%a_ARR_IDX+2] + 1;
			if (%a_TX_PTR[%a_ARR_IDX+2] == 0)
			{
				%a_TX_PTR[%a_ARR_IDX+3] = %a_TX_PTR[%a_ARR_IDX+3] + 1;
			}
		}
	}

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)

	for (char i=0; i<4; i++)
	{
		Tx_sendbyte(channel, %a_MY_IP[i]);
	}

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

	#error "TODO: MIAC stuff here"

  #else

	#error "Error - no Internet bus type defined."

  #endif

/*Macro_Tx_sendmyip_End*/
}

void Tx_end(char channel)
{
/*Macro_Tx_end_Start*/

	//make sure we are addressing this device
	MAC_WR_ADDR = %a_W3100A_WR;

  #if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)

	%a_ARR_IDX = channel*%a_CHANNEL_SIZE;

	//FlushBuffer
	MAC_PTR_VAL[0] = %a_TX_PTR[%a_ARR_IDX+0];
	MAC_PTR_VAL[1] = %a_TX_PTR[%a_ARR_IDX+1];
	MAC_PTR_VAL[2] = %a_TX_PTR[%a_ARR_IDX+2];
	MAC_PTR_VAL[3] = %a_TX_PTR[%a_ARR_IDX+3];

	//Update TW_PR
	MAC_WritePtr(0, REG_Cx_TW_PR(channel));

	//send the "send" command
	%a_STATUS[channel] = 0;
	MAC_WriteReg(0, REG_Cx_CR(channel), CSEND);

	//wait until a SSYS_SSEND_OK interrupt
	while (!(%a_STATUS[channel] & SSEND_OK))
	{
		%a_Check_Int();
	}

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)

	unsigned short tx_ptr_cur;

    // Set the position to write as the position to read from, marking the end of the data
	tx_ptr_cur = MAC_ReadWord(W5100_Sn_TX_WR(channel)); // Current pointer
	tx_ptr_cur += gTxSize[channel];                     // Add buffered size
	//tx_ptr_cur = tx_ptr_cur & W5100_TXMEM_MASK;       // Dont wrap buffer pointer, only wrapped when calculating address
	MAC_WriteWord(W5100_Sn_TX_WR(channel), tx_ptr_cur);

    // Now Send the SEND command
    MAC_WriteByte(W5100_Sn_CR(channel), W5100_Sn_CR_SEND);

    // Wait for Sending Process
    while(MAC_ReadByte(W5100_Sn_CR(channel)));
    // Reset sent size
    gTxSize[channel] = 0;

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

	#error "TODO: MIAC stuff here"

  #else

	#error "Error - no Internet bus type defined."

  #endif

/*Macro_Tx_end_End*/
}

char Rx_data_available(char channel)
{
/*Macro_Rx_data_available_Start*/

	//returns 0 for false, <packet_size> for true (255 means >= 255)

	char retval = 1;

	//make sure we are addressing this device
	MAC_WR_ADDR = %a_W3100A_WR;

  #if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)

	char addr_l;
	char addr_h;
	char tlen0 = 0;
	char tlen1 = 0;
	char rw_ptr[4];
	char ssr;
	char hdr_diff;
	char idx = 0;
	char old_hrd;
	char hdr_size;
	char all_rx;
	char hdr_idx_base;
	char is_tcp = 0;

	//debug:
	char init_rx0;
	char init_rx1;
	char init_hdr0;
	char init_hdr1;

	//send the "receive" command
	%a_STATUS[channel] = 0;

	%a_ARR_IDX = channel*%a_CHANNEL_SIZE;
	hdr_idx_base = channel*8;

	//wait until a SSYS_SRECV_OK interrupt
	while (!(%a_STATUS[channel] & SRECV_OK))
	{
		MAC_WriteReg(0, REG_Cx_CR(channel), CRECV);
		%a_Check_Int();
	}

	//get the RW pointer
	MAC_ReadPtrWithShadow(Cx_SRW_PTR(channel), REG_Cx_RW_PR(channel));
	rw_ptr[0] = MAC_PTR_VAL[0];
	rw_ptr[1] = MAC_PTR_VAL[1];
	rw_ptr[2] = MAC_PTR_VAL[2];
	rw_ptr[3] = MAC_PTR_VAL[3];

	//get the RR pointer
	MAC_ReadPtrWithShadow(Cx_SRR_PTR(channel), REG_Cx_RR_PR(channel));
	%a_RX_PTR[%a_ARR_IDX+0] = MAC_PTR_VAL[0];
	%a_RX_PTR[%a_ARR_IDX+1] = MAC_PTR_VAL[1];
	%a_RX_PTR[%a_ARR_IDX+2] = MAC_PTR_VAL[2];
	%a_RX_PTR[%a_ARR_IDX+3] = MAC_PTR_VAL[3];

	//calculate pointer differences
	if (rw_ptr[0] == %a_RX_PTR[%a_ARR_IDX+0])
	{
		if (rw_ptr[1] == %a_RX_PTR[%a_ARR_IDX+1])
		{
			if (rw_ptr[2] == %a_RX_PTR[%a_ARR_IDX+2])
			{
				if (rw_ptr[3] == %a_RX_PTR[%a_ARR_IDX+3])
				{
					//same, so no data received
					retval = 0;
				}
			}
		}
	}

	//read SSR to see which mode we are in...
	ssr = MAC_ReadReg(0, REG_Cx_SSR(channel));

	//set the header size and additional tlen:
	//   UDP: header = tlen(2) + SIP(4) + SPORT(2) = 8
	//   IP:  header = tlen(2) + SIP(4)            = 6
	//   MAC: header = tlen(2) + STT(1)            = 3
	//
	//NOTE: RX_END is bigger than (RX+tlen) by the following
	//      factors (and there seems to be no logic to the values
	//      and no documentation to confirm them)
	//    UDP: TX_END = TX + tlen + 0
	//    IP:  TX_END = TX + tlen + 2
	//    MAC: TX_END = TX + tlen + 3

	switch (ssr)
	{
		case SOCK_UDP:            //UDP
		case SOCK_UDP_ARP:
		case SOCK_UDP_DATA:
			hdr_size = 8;
			tlen0 = 0;
			hdr_diff = 8;
			break;

		case SOCK_RAW:            //IP
			hdr_size = 6;
			tlen0 = 2;
			hdr_diff = 0;
			break;

		case SOCK_RAW_INIT:        //MAC
			hdr_size = 3;
			tlen0 = 3;
			hdr_diff = 4;
			break;

		default:                //assume TCP
			//retval = 0;
			is_tcp = 1;
	}

	if (retval != 0)
	{
		if (is_tcp != 0)
		{
			//TCP
			%a_RX_END_PTR[%a_ARR_IDX+0] = rw_ptr[0];
			%a_RX_END_PTR[%a_ARR_IDX+1] = rw_ptr[1];
			%a_RX_END_PTR[%a_ARR_IDX+2] = rw_ptr[2];
			%a_RX_END_PTR[%a_ARR_IDX+3] = rw_ptr[3];

		} else {

			//debug:
			init_rx0 = %a_RX_PTR[%a_ARR_IDX+0];
			init_rx1 = %a_RX_PTR[%a_ARR_IDX+1];

			//set baseline value for RX_END_PTR
			%a_RX_END_PTR[%a_ARR_IDX+0] = %a_RX_PTR[%a_ARR_IDX+0];
			%a_RX_END_PTR[%a_ARR_IDX+1] = %a_RX_PTR[%a_ARR_IDX+1];
			%a_RX_END_PTR[%a_ARR_IDX+2] = %a_RX_PTR[%a_ARR_IDX+2];
			%a_RX_END_PTR[%a_ARR_IDX+3] = %a_RX_PTR[%a_ARR_IDX+3];

			addr_l = %a_RX_PTR[%a_ARR_IDX+0];
			addr_h = (%a_RX_PTR[%a_ARR_IDX+1] & MEM_RX_MASK(channel)) + MEM_RX_OFFSET(channel);

			//send the i2c start bit and the address
			MAC_SendI2CStart();
			MAC_SendI2CByte(%a_W3100A_WR);    //write to W3100A
			MAC_SendI2CByte(addr_h);            //addr (hi byte)
			MAC_SendI2CByte(addr_l);            //addr (low byte)
			MAC_SendI2CStop();

			MAC_SendI2CStart();
			MAC_SendI2CByte(%a_W3100A_RD);    //read from W3100A

			idx = 0;
			while (idx < (hdr_size-1))
			{
				%a_HEADER[hdr_idx_base+idx] = MAC_ReadI2CByte(0);    //read header data
				%a_Inc_RX_Ptr(channel);
				idx++;
			}

			%a_HEADER[hdr_idx_base+idx] = MAC_ReadI2CByte(1);        //read header data (last byte)
			MAC_SendI2CStop();
			%a_Inc_RX_Ptr(channel);

			if (%a_HEADER[hdr_idx_base+0] > 10)
			{

				//something's gone wrong, so we should reset the socket
				//NOTE: this does not work (CCLOSE and CSOCK_INIT are never positively interrupted)

				//close the socket
				%a_STATUS[channel] = 0;
				MAC_WriteReg(0, REG_Cx_CR(channel), CCLOSE);

				//check for result
				/*
				while (%a_STATUS[channel] == 0)
				{
					ssr = MAC_ReadReg(0, REG_Cx_SSR(channel));   //debug
					%a_Check_Int();
				}
				*/
				while (ssr != SOCK_CLOSED)
				{
					ssr = MAC_ReadReg(0, REG_Cx_SSR(channel));
				}


				//reset the buffer pointers
				MAC_PTR_VAL[0] = 0;
				MAC_PTR_VAL[1] = 0;
				MAC_PTR_VAL[2] = 0;
				MAC_PTR_VAL[3] = 0;

				//Update RR_PR
				MAC_WritePtr(0, REG_Cx_RR_PR(channel));

				//Update RW_PR
				MAC_WritePtr(0, REG_Cx_RW_PR(channel));


				//re-initialise the socket
				%a_STATUS[channel] = 0;
				MAC_WriteReg(0, REG_Cx_CR(channel), CSOCK_INIT);

				//check for result
				/*
				while (%a_STATUS[channel] == 0)
				{
					ssr = MAC_ReadReg(0, REG_Cx_SSR(channel));   //debug
					%a_Check_Int();
				}
				*/
				while (ssr == SOCK_CLOSED)
				{
					ssr = MAC_ReadReg(0, REG_Cx_SSR(channel));
				}

				//also, return false
				retval = 0;

			} else {

				tlen0 = tlen0 + %a_HEADER[hdr_idx_base+1];
				if (tlen0 < %a_HEADER[hdr_idx_base+1])
				{
					tlen1++;
				}
				//tlen1 = %a_HEADER[hdr_idx_base+0];		//TODO: I think this was a bug
				tlen1 = tlen1 + %a_HEADER[hdr_idx_base+0];	//TODO: (replaced with this line)


				//debug:
				init_hdr0 = %a_HEADER[hdr_idx_base+0];
				init_hdr1 = %a_HEADER[hdr_idx_base+1];

				//now modify the tlen in the header:
				old_hrd = %a_HEADER[hdr_idx_base+1];
				%a_HEADER[hdr_idx_base+1] = %a_HEADER[hdr_idx_base+1] - hdr_diff;
				if (%a_HEADER[hdr_idx_base+1] > old_hrd)
				{
					//wrapped
					%a_HEADER[hdr_idx_base+0] = %a_HEADER[hdr_idx_base+0] - 1;
					//sanity check...
					//if (%a_HEADER[hdr_idx_base+0] > 100)
					//{
					//    %a_HEADER[hdr_idx_base+0] = 0;
					//}
				}

			  #ifdef %a_DEBUG
				%a_DBG_IDX = 0;
			  #endif

				//calculate the end of packet pointer
				while (tlen0 > 0)
				{
					tlen0--;
					%a_RX_END_PTR[%a_ARR_IDX+0] = %a_RX_END_PTR[%a_ARR_IDX+0] + 1;
					if (%a_RX_END_PTR[%a_ARR_IDX+0] == 0)
					{
						%a_RX_END_PTR[%a_ARR_IDX+1] = %a_RX_END_PTR[%a_ARR_IDX+1] + 1;
						if (%a_RX_END_PTR[%a_ARR_IDX+1] == 0)
						{
							%a_RX_END_PTR[%a_ARR_IDX+2] = %a_RX_END_PTR[%a_ARR_IDX+2] + 1;
							if (%a_RX_END_PTR[%a_ARR_IDX+2] == 0)
							{
								%a_RX_END_PTR[%a_ARR_IDX+3] = %a_RX_END_PTR[%a_ARR_IDX+3] + 1;
							}
						}
					}
				}

				while (tlen1 > 0)
				{
					tlen1--;
					%a_RX_END_PTR[%a_ARR_IDX+1] = %a_RX_END_PTR[%a_ARR_IDX+1] + 1;
					if (%a_RX_END_PTR[%a_ARR_IDX+1] == 0)
					{
						%a_RX_END_PTR[%a_ARR_IDX+2] = %a_RX_END_PTR[%a_ARR_IDX+2] + 1;
						if (%a_RX_END_PTR[%a_ARR_IDX+2] == 0)
						{
							%a_RX_END_PTR[%a_ARR_IDX+3] = %a_RX_END_PTR[%a_ARR_IDX+3] + 1;
						}
					}
				}

				//check to see if all data has been received
				all_rx = 0;

				if (rw_ptr[3] >= %a_RX_END_PTR[%a_ARR_IDX+1])
				{
					if (rw_ptr[2] >= %a_RX_END_PTR[%a_ARR_IDX+2])
					{
						if (rw_ptr[1] >= %a_RX_END_PTR[%a_ARR_IDX+1])
						{
							if (rw_ptr[0] >= %a_RX_END_PTR[%a_ARR_IDX+0])
							{
								all_rx = 1;
							}
						}
					}
				}

				//only returns true when whole packet has been received
				retval = 0;
				if (all_rx == 1)
				{
					if (%a_HEADER[hdr_idx_base+0] > 0)
					{
						retval = 255;
					} else {
						retval = %a_HEADER[hdr_idx_base+1];
					}
				}
			}
		}
	}

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)

	//read SSR to see which mode we are in...
	char ssr;
	unsigned short tlen;

	ssr = MAC_ReadByte(W5100_Sn_SR(channel));
	switch (ssr)
	{
		//set the header size and additional tlen:
		//   TCP: no header
		//   UDP: header = dest_IP(4) + dest_PORT(2) + tlen(2)   = 8
		//   IP:  header = dest_IP(4) + tlen(2)                  = 6
		//   MAC: header = tlen(2)                               = 2

		case W5100_SOCK_UDP:
			gHrdSize[channel] = 8;
			break;

		case W5100_SOCK_IPRAW:
			gHrdSize[channel] = 6;
			break;

		case W5100_SOCK_MACRAW:
			gHrdSize[channel] = 2;
			break;

		//case W5100_SOCK_PPPOE:
		//	gHrdSize[channel] = ???;
		//	break;

		default:
			//TCP or closed
			gHrdSize[channel] = 0;
			break;
	}

	//get the size of received data
	unsigned short rx_rsr_val;
	rx_rsr_val = MAC_ReadWord(W5100_Sn_RX_RSR(channel));

	retval = 0;
	if (rx_rsr_val >= gHrdSize[channel])
	{
	    if (gHrdSize[channel] > 0)
	    {
		    unsigned short offset;
		    offset = MAC_ReadWord(W5100_Sn_RX_RD(channel));
		    offset = offset & W5100_RXMEM_MASK;
		    gRxPtr[channel] = W5100_RXMEM(channel) + offset;

		    //read in the header
		    for (char i=0; i<gHrdSize[channel]; i++)
		    {
			    %a_HEADER[(channel*8)+i] = MAC_ReadByte(gRxPtr[channel]++);
			    //check for rollover
			    if (gRxPtr[channel] >= W5100_RXMEM(channel) + W5100_RXMEM_MASK)
			    {
				    gRxPtr[channel] = W5100_RXMEM(channel);
			    }
		    }

		    gRxSize[channel] = %a_HEADER[(channel*8) + gHrdSize[channel] - 2];
		    gRxSize[channel] = gRxSize[channel] << 8;
		    gRxSize[channel] = gRxSize[channel] | %a_HEADER[(channel*8) + gHrdSize[channel] - 1];

		    //now rearrange the header so it looks like the old W3200 header info
		    //i.e. tlen[2] + SIP[4] + SPORT[2]
		    for (i=7; i>1; i--)
		    {
			    %a_HEADER[(channel*8)+i] = %a_HEADER[(channel*8)+i-2];
		    }
		    %a_HEADER[(channel*8)+1] = gRxSize[channel] & 0xff;
		    %a_HEADER[(channel*8)+0] = (gRxSize[channel] & 0xff00) >> 8;

		    if (gRxSize[channel] >= 255)
		    {
			    retval = 255;
		    } else {
			    retval = gRxSize[channel];
		    }
	    }
	    else
	    {
	        // TCP
		    unsigned short offset;
		    offset = MAC_ReadWord(W5100_Sn_RX_RD(channel));
		    offset = offset & W5100_RXMEM_MASK;
		    gRxPtr[channel] = W5100_RXMEM(channel) + offset;

		    //read in the header - zero size so there is none
		    // Write the data size to the storage
		    gRxSize[channel] = rx_rsr_val;

		    //now rearrange the header so it looks like the old W3200 header info
		    //i.e. tlen[2] + SIP[4] + SPORT[2]
		    for (char i=7; i>1; i--)
		    {
			    %a_HEADER[(channel*8)+i] = 0; // %a_HEADER[(channel*8)+i-2];
		    }
		    %a_HEADER[(channel*8)+1] = gRxSize[channel] & 0xff;
		    %a_HEADER[(channel*8)+0] = (gRxSize[channel] & 0xff00) >> 8;

		    if (gRxSize[channel] >= 255)
		    {
			    retval = 255;
		    } else {
			    retval = gRxSize[channel];
		    }
	    }
	}

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

	#error "TODO: MIAC stuff here"

  #else

	#error "Error - no Internet bus type defined."

  #endif

        return (retval);
/*Macro_Rx_data_available_End*/
}


signed short Rx_data_size(char channel)
{
/*Macro_Rx_data_size_Start*/

	short size;

  #if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)

	%a_ARR_IDX = channel * %a_CHANNEL_SIZE;
	size = (%a_RX_END_PTR[%a_ARR_IDX+1] - %a_RX_PTR[%a_ARR_IDX+1]) & (MEM_RX_MASK(channel));
	size = (size << 8) + %a_RX_END_PTR[%a_ARR_IDX] - %a_RX_PTR[%a_ARR_IDX];

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)

	//not as simple as this: 	size = MAC_ReadWord(W5100_Sn_RX_RSR(channel));
	//need to take into account the header size (i.e. subtract it)
	size = gRxSize[channel];
	#warning "TODO? Rx_data_size"

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

	#error "TODO: MIAC stuff here"

  #else

	#error "Error - no Internet bus type defined."

  #endif

    return (size);
/*Macro_Rx_data_size_End*/
}


char Rx_readheader(char channel, char idx)
{
/*Macro_Rx_readheader_Start*/

	return (%a_HEADER[(channel*8)+idx]);

/*Macro_Rx_readheader_End*/
}

char Rx_readbyte(char channel)
{
/*Macro_Rx_readbyte_Start*/

	char retval;
	char addr_l;
	char addr_h;

	//make sure we are addressing this device
	MAC_WR_ADDR = %a_W3100A_WR;

  #if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)

	%a_ARR_IDX = channel*%a_CHANNEL_SIZE;

	addr_l = %a_RX_PTR[%a_ARR_IDX+0];
	addr_h = (%a_RX_PTR[%a_ARR_IDX+1] & MEM_RX_MASK(channel)) + MEM_RX_OFFSET(channel);

	//send the i2c start bit and the address
	MAC_SendI2CStart();
	MAC_SendI2CByte(%a_W3100A_WR);    //write to W3100A
	MAC_SendI2CByte(addr_h);            //addr (hi byte)
	MAC_SendI2CByte(addr_l);            //addr (low byte)
	MAC_SendI2CStop();

	MAC_SendI2CStart();
	MAC_SendI2CByte(%a_W3100A_RD);    //read from W3100A
	retval = MAC_ReadI2CByte(1);        //read data
	MAC_SendI2CStop();

	//increment the RR pointer
	%a_Inc_RX_Ptr(channel);

	#ifdef %a_DEBUG
		if (channel == 0) && (%a_DBG_IDX < %a_DBG_IDX_MAX))
		{
			%a_DBG_VALS[%a_DBG_IDX] = retval;
			%a_DBG_IDX++;
		}
	#endif

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)

    //debug read the byte 14 spaces previous
	retval = MAC_ReadByte(gRxPtr[channel]++);

	//check for rollover
	if (gRxPtr[channel] >= W5100_RXMEM(channel) + W5100_RXMEM_SIZE)
	{
		gRxPtr[channel] = W5100_RXMEM(channel);
	}
	//#warning "TODO? Rx_readbyte"

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

	#error "TODO: MIAC stuff here"

  #else

	#error "Error - no Internet bus type defined."

  #endif

    return (retval);
/*Macro_Rx_readbyte_End*/
}

void Rx_skipbytes(char channel, char count)
{
/*Macro_Rx_skipbytes_Start*/
	char i = 0;

	//make sure we are addressing this device
	MAC_WR_ADDR = %a_W3100A_WR;

	while (i < count)
	{
		Rx_readbyte(channel);
		i++;
	}
/*Macro_Rx_skipbytes_End*/
}

char Rx_match2bytes(char channel, char byte1, char byte2)
{
/*Macro_Rx_match2bytes_Start*/
	//returns 1 for a match, 0 for no match
	char retval = 1;
	char rx_byte;

	//make sure we are addressing this device
	MAC_WR_ADDR = %a_W3100A_WR;

	rx_byte = Rx_readbyte(channel);
	if (rx_byte != byte1)
	{
		retval = 0;
	}
	rx_byte = Rx_readbyte(channel);
	if (rx_byte != byte2)
	{
		retval = 0;
	}

	return (retval);
/*Macro_Rx_match2bytes_End*/
}

char Rx_match4bytes(char channel, char byte1, char byte2, char byte3, char byte4)
{
/*Macro_Rx_match4bytes_Start*/
	//returns 1 for a match, 0 for no match
	char retval = 1;
	char rx_byte;

	//make sure we are addressing this device
	MAC_WR_ADDR = %a_W3100A_WR;

	rx_byte = Rx_readbyte(channel);
	if (rx_byte != byte1)
	{
		retval = 0;
	}
	rx_byte = Rx_readbyte(channel);
	if (rx_byte != byte2)
	{
		retval = 0;
	}
	rx_byte = Rx_readbyte(channel);
	if (rx_byte != byte3)
	{
		retval = 0;
	}
	rx_byte = Rx_readbyte(channel);
	if (rx_byte != byte4)
	{
		retval = 0;
	}

	return (retval);
/*Macro_Rx_match4bytes_End*/
}

char Rx_match6bytes(char channel, char byte1, char byte2, char byte3, char byte4, char byte5, char byte6)
{
/*Macro_Rx_match6bytes_Start*/
	//returns 1 for a match, 0 for no match
	char retval = 1;
	char rx_byte;

	//make sure we are addressing this device
	MAC_WR_ADDR = %a_W3100A_WR;

	rx_byte = Rx_readbyte(channel);
	if (rx_byte != byte1)
	{
		retval = 0;
	}
	rx_byte = Rx_readbyte(channel);
	if (rx_byte != byte2)
	{
		retval = 0;
	}
	rx_byte = Rx_readbyte(channel);
	if (rx_byte != byte3)
	{
		retval = 0;
	}
	rx_byte = Rx_readbyte(channel);
	if (rx_byte != byte4)
	{
		retval = 0;
	}
	rx_byte = Rx_readbyte(channel);
	if (rx_byte != byte5)
	{
		retval = 0;
	}
	rx_byte = Rx_readbyte(channel);
	if (rx_byte != byte6)
	{
		retval = 0;
	}

	return (retval);
/*Macro_Rx_match6bytes_End*/
}

char Rx_match_mymac(char channel)
{
/*Macro_Rx_match_mymac_Start*/
	//returns 1 for a match, 0 for no match
	char retval = 1;
	char rx_byte;

	//make sure we are addressing this device
	MAC_WR_ADDR = %a_W3100A_WR;

	//NOTE: flowcode does not like things like:
	//      "if (Rx_xxreadbyte(channel) != %a_SIPR_0)"
	//      (it falls over during its internal substitution)
	rx_byte = Rx_readbyte(channel);
	if (rx_byte != %a_SHAR_0)
	{
		retval = 0;
	}
	rx_byte = Rx_readbyte(channel);
	if (rx_byte != %a_SHAR_1)
	{
		retval = 0;
	}
	rx_byte = Rx_readbyte(channel);
	if (rx_byte != %a_SHAR_2)
	{
		retval = 0;
	}
	rx_byte = Rx_readbyte(channel);
	if (rx_byte != %a_SHAR_3)
	{
		retval = 0;
	}
	rx_byte = Rx_readbyte(channel);
	if (rx_byte != %a_SHAR_4)
	{
		retval = 0;
	}
	rx_byte = Rx_readbyte(channel);
	if (rx_byte != %a_SHAR_5)
	{
		retval = 0;
	}

	return (retval);
/*Macro_Rx_match_mymac_End*/
}

char Rx_match_myip(char channel)
{
/*Macro_Rx_match_myip_Start*/
	//returns 1 for a match, 0 for no match
	char retval = 1;
	char rx_byte;

	//make sure we are addressing this device
	MAC_WR_ADDR = %a_W3100A_WR;

	rx_byte = Rx_readbyte(channel);
	if (rx_byte != %a_MY_IP[0])
	{
		retval = 0;
	}
	rx_byte = Rx_readbyte(channel);
	if (rx_byte != %a_MY_IP[1])
	{
		retval = 0;
	}
	rx_byte = Rx_readbyte(channel);
	if (rx_byte != %a_MY_IP[2])
	{
		retval = 0;
	}
	rx_byte = Rx_readbyte(channel);
	if (rx_byte != %a_MY_IP[3])
	{
		retval = 0;
	}

	return (retval);
/*Macro_Rx_match_myip_End*/
}

void Rx_flush_data(char channel)
{
/*Macro_Rx_flush_data_Start*/
	//make sure we are addressing this device
	MAC_WR_ADDR = %a_W3100A_WR;

  #if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)

	%a_ARR_IDX = channel*%a_CHANNEL_SIZE;

	//Update RR_PR
	MAC_PTR_VAL[0] = %a_RX_END_PTR[%a_ARR_IDX+0];
	MAC_PTR_VAL[1] = %a_RX_END_PTR[%a_ARR_IDX+1];
	MAC_PTR_VAL[2] = %a_RX_END_PTR[%a_ARR_IDX+2];
	MAC_PTR_VAL[3] = %a_RX_END_PTR[%a_ARR_IDX+3];

	MAC_WritePtr(0, REG_Cx_RR_PR(channel));

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)

	//update the read pointer
	unsigned short rx_sz = MAC_ReadWord(W5100_Sn_RX_RSR(channel));
	unsigned short rx_ptr = MAC_ReadWord(W5100_Sn_RX_RD(channel));

	rx_ptr = (rx_ptr + rx_sz);
	// Dont do rx_ptr &= W5100_RXMEM_MASK; - only for calculating addreses
	MAC_WriteWord(W5100_Sn_RX_RD(channel), rx_ptr);

	//send the RECV command
	MAC_WriteByte(W5100_Sn_CR(channel), W5100_Sn_CR_RECV);

	//#warning "TODO? Rx_flush_data"

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

	#error "TODO: MIAC stuff here"

  #else

	#error "Error - no Internet bus type defined."

  #endif

/*Macro_Rx_flush_data_End*/
}

