/*********************************************************************
 *                  Flowcode Webserver Component Code
 *
 * File: PIC_WebServer.c
 *
 * (c) 2009 Matrix Multimedia Ltd.
 * http://www.matrixmultimedia.com
 *
 * Software License Agreement
 *
 * The software supplied herewith by Matrix Multimedia Ltd (the
 * Company) for its Flowcode graphical programming language is
 * intended and supplied to you, the Companys customer, for use
 * solely and exclusively on the Company's products. The software
 * is owned by the Company, and is protected under applicable
 * copyright laws. All rights are reserved. Any use in violation
 * of the foregoing restrictions may subject the user to criminal
 * sanctions under applicable laws, as well as to civil liability
 * for the breach of the terms and conditions of this licence.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * Changelog:
 *
 *  date  | by | description
 * -------+----+-----------------------------------------------------
 * 240407 | ST | Created
 * 250607 | BR | Externalised
 * 120707 | ST | Made to work with the CTL file
 * 180309 | BR | Changed delays to be WDT compatible
 * 260309 | BR | Moved private functions from AddDefines section
 * 130509 | BR | Added % definitions to DefinesCode section
 * 100909 | BR | Added HiTECH compatible definitions
 * 131109 | BR | Fixed #pragma error HITECH problem
 * 190412 | ST | Added substitutions from the component (hw type, cs pins)
 * 190412 | ST | INT pin now has its own port/tris vals
 ********************************************************************/

/*********************************************************************

 Return & parameter types:
   void
   char
   short
   char*

 Pin directions:
   0 = OUTPUT
   2 = INPUT
   3 = BIDIRECTIONAL

**********************************************************************

[Settings]
CLSID={3696AC8D-FF7A-4431-B39C-0415077C7B8E}
IsAnalogue=0
MultipleAllowed=1
Description=Webserver Flowcode Component

**********************************************************************

[Port]
DesiredPort=0
PortMandatory=0
SamePortAsID=0

[Pins]
Count=2
1=Interrupt
2=CS

[PinPort]
# PORTA = 0
# PORTB = 1
# PORTC = 2
# PORTD = 3
# PORTE = 4
1=0
2=0

[PinDesiredBit]
1=0
2=3

[PinDirection]
# DIR_INPUT = 2
# DIR_OUTPUT = 0
# DIR_BIDIRECTIONAL = 3
1=2
2=3

[PinMustUsePort]
# boolean
1=1
2=1

[SamePortAsPinX]
# -1 = any pin
# otherwise, the pin number
1=-1
2=-1

[PinMustUsePin]
# boolean
1=1
2=1

[PinValue]
# boolean (0 = off, 1 = on)
1=0
2=0


**********************************************************************

[MacroNames]
Count=3
1=Initialize
2=CreateServerSocket
3=CheckSocketActivity

[MacroReturns]
1=void
2=char
3=char

[MacroIsPrivate]
1=0
2=0
3=0

[MacroParameters_Initialize]
Count=0

[MacroParamTypes_Initialize]


[MacroParameters_CreateServerSocket]
Count=3
1=channel
2=port_hi
3=port_lo

[MacroParamTypes_CreateServerSocket]
1=char
2=char
3=char

[MacroParameters_CheckSocketActivity]
Count=0

[MacroParamTypes_CheckSocketActivity]

 ********************************************************************/

/********************************************************************
 * ADDITIONAL CODE
 ********************************************************************/

/*DefinesCode_Start*/

/**** Macro Substitutions ****
a = COMP_X
b = W3100A_WR
c = I2C_INT
d = I2C_INTMASK
e = I2C_BAUD
f = GAR
g = SMR
h = SHAR
i = SIPR
j = MAX_PAGES
k = PAGE1_NAME
l = PAGE2_NAME
m = PAGE3_NAME
n = PAGE4_NAME
o = PAGE1_HTML
p = PAGE2_HTML
q = PAGE3_HTML
r = PAGE4_HTML
s = CS pin
t = CS port letter
u = INT port letter
v = HW type (0 = W3100, 1 = W5100)
******************************/

//////////////////////////////////////////////////////////////////////////////////////////
//This section details the static defines, prototypes
//and functions.

#ifndef MX_WEBSVR_STATIC_DEFINES
  #define MX_WEBSVR_STATIC_DEFINES

//I'll start with the defines that are common to all component instances:
#ifdef _BOOSTC
	#define MX_INTERNET_SHORTDELAY nop();nop();nop();nop();nop();nop();nop();nop();nop();nop();
#endif

#ifdef HI_TECH_C
	#define MX_INTERNET_SHORTDELAY asm("nop");asm("nop");asm("nop");asm("nop");asm("nop");asm("nop");asm("nop");asm("nop");asm("nop");asm("nop");
#endif


#define MX_TCPIP_HWTYPE_W3100	0		//uses I2C
#define MX_TCPIP_HWTYPE_W5100	1		//uses SPI
#define MX_TCPIP_HWTYPE_MIAC	2		//uses CAN

//these will come from the component
#define MX_TCPIP_HWTYPE		%v
#define %a_CS_PIN			%s
#define %a_CS_PORT			port%t
#define %a_CS_TRIS			tris%t

#if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)

	#define IRTR_HI        0x03
	#define IRTR_LO        0xE8
	#define RCR_VAL        100

	// Register definitions
	#define REG_Cx_CR(i)    (i + 0x00)
	#define REG_Cx_ISR(i)   (i + 0x04)
	#define REG_IR          0x08
	#define REG_IMR         0x09

	#define REG_IDM_OR      0x0C
	#define REG_IDM_AR0     0x0D
	#define REG_IDM_AR1     0x0E
	#define REG_IDM_DR      0x0F

	#define RX_PTR_BASE     0x10
	#define RX_PTR_SIZE     0x0C
	#define REG_Cx_RW_PR(i) (RX_PTR_BASE + (RX_PTR_SIZE*i))
	#define REG_Cx_RR_PR(i) (RX_PTR_BASE + (RX_PTR_SIZE*i) + 0x04)
	#define REG_Cx_TA_PR(i) (RX_PTR_BASE + (RX_PTR_SIZE*i) + 0x08)

	#define TX_PTR_BASE     0x40
	#define TX_PTR_SIZE     0x0C
	#define REG_Cx_TW_PR(i) (TX_PTR_BASE + (TX_PTR_SIZE*i))
	#define REG_Cx_TR_PR(i) (TX_PTR_BASE + (TX_PTR_SIZE*i) + 0x04)

	#define REG_GAR         0x80
	#define REG_SMR         0x84
	#define REG_SHAR        0x88
	#define REG_SIPR        0x8E
	#define REG_IRTR        0x92
	#define REG_RCR         0x94
	#define REG_RMSR        0x95
	#define REG_TMSR        0x96

	#define SOCK_BASE       0xA0
	#define SOCK_SIZE       0x18
	#define REG_Cx_SSR(i)   (SOCK_BASE + (SOCK_SIZE*i))
	#define REG_Cx_SOPR(i)  (SOCK_BASE + (SOCK_SIZE*i) + 0x01)
	#define REG_Cx_DIR(i)   (SOCK_BASE + (SOCK_SIZE*i) + 0x08)
	#define REG_Cx_DPR(i)   (SOCK_BASE + (SOCK_SIZE*i) + 0x0C)
	#define REG_Cx_SPR(i)   (SOCK_BASE + (SOCK_SIZE*i) + 0x0E)
	#define REG_Cx_IPR(i)   (SOCK_BASE + (SOCK_SIZE*i) + 0x10)
	#define REG_Cx_TOSR(i)  (SOCK_BASE + (SOCK_SIZE*i) + 0x11)
	#define REG_Cx_MSSR(i)  (SOCK_BASE + (SOCK_SIZE*i) + 0x12)

	//shadow registers
	#define SHADOW_REG_HI    0x01
	#define SHADOW_BASE      0xE0
	#define Cx_SRW_PTR(i)    (SHADOW_BASE + (3*i))
	#define Cx_SRR_PTR(i)    (SHADOW_BASE + (3*i) + 0x01)
	#define Cx_STA_PTR(i)    (SHADOW_BASE + (3*i) + 0x02)
	#define Cx_STW_PTR(i)    (SHADOW_BASE + (3*i) + 0x10)
	#define Cx_STR_PTR(i)    (SHADOW_BASE + (3*i) + 0x11)


	// Settings for REG_Cx_SOPR
	#define SOCKOPT_BROADCAST    0x80
	#define SOCKOPT_NDTIMEOUT    0x40
	#define SOCKOPT_NDACK        0x20
	#define SOCKOPT_SWS          0x10

	#define MACLOPT_RXERR        0x80
	#define MACLOPT_BROADCAST    0x40
	#define MACLOPT_PROMISC      0x10

	#define SOCK_MACL_RAW        0x04
	#define SOCK_IPL_RAW         0x03
	#define SOCK_DGRAM           0x02
	#define SOCK_STREAM          0x01
	#define SOCK_CLOSEDM         0x00


	// Settings for IP PROTOCOL
	//#define IPPROTO_IP              0               // dummy for IP
	//#define IPPROTO_ICMP            1               // control message protocol
	//#define IPPROTO_IGMP            2               // internet group management protocol
	//#define IPPROTO_GGP             3               // gateway^2 (deprecated)
	//#define IPPROTO_TCP             6               // tcp
	//#define IPPROTO_PUP             12              // pup
	//#define IPPROTO_UDP             17              // user datagram protocol
	//#define IPPROTO_IDP             22              // xns idp
	//#define IPPROTO_ND              77              // UNOFFICIAL net disk proto
	//#define IPPROTO_RAW             255             // raw IP packet


	// Parameters
	//#define SEL_CONTROL     0    // Confirm socket status
	//#define SEL_SEND        1    // Confirm Tx free buffer size
	//#define SEL_RECV        2    // Confirm Rx data size


	// Settings for REG_Cx_CR
	#define CSYS_INIT    0x01
	#define CSOCK_INIT   0x02
	#define CCONNECT     0x04
	#define CLISTEN      0x08
	#define CCLOSE       0x10
	#define CSEND        0x20
	#define CRECV        0x40
	#define CSW_RESET    0x80


	// Settings for REG_Cx_ISR
	#define SSYS_INIT_OK    0x01
	#define SSOCK_INIT_OK   0x02
	#define SESTABLISHED    0x04
	#define SCLOSED         0x08
	#define STIMEOUT        0x10
	#define SSEND_OK        0x20
	#define SRECV_OK        0x40


	// Socket Status Variables
	#define SOCK_CLOSED         0x00
	#define SOCK_ARP            0x01
	#define SOCK_LISTEN         0x02
	#define SOCK_SYNSENT        0x03
	#define SOCK_SYNSENT_ACK    0x04
	#define SOCK_SYNRECV        0x05
	#define SOCK_ESTABLISHED    0x06
	#define SOCK_CLOSE_WAIT     0x07
	#define SOCK_LAST_ACK       0x08
	#define SOCK_FIN_WAIT1      0x09
	#define SOCK_FIN_WAIT2      0x0A
	#define SOCK_CLOSING        0x0B
	#define SOCK_TIME_WAIT      0x0C
	#define SOCK_RESET          0x0D
	#define SOCK_INIT           0x0E
	#define SOCK_UDP            0x0F
	#define SOCK_RAW            0x10
	#define SOCK_UDP_ARP        0x11
	#define SOCK_UDP_DATA       0x12
	#define SOCK_RAW_INIT       0x13

	#define CHANNEL_FLAG_0 0x01
	#define CHANNEL_FLAG_1 0x02
	#define CHANNEL_FLAG_2 0x04
	#define CHANNEL_FLAG_3 0x08

	#define Internet_RD_ADDR    (Internet_WR_ADDR+1)


#elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)


	//200ms retry time, count = 25
	#define IRTR_HI        0x07
	#define IRTR_LO        0xD0
	#define RCR_VAL        25

	// Wiznet W5100 Register Addresses
	#define W5100_MR			0x0000   // Mode Register
	#define W5100_GAR			0x0001   // Gateway Address: 0x0001 to 0x0004
	#define W5100_SUBR			0x0005   // Subnet mask Address: 0x0005 to 0x0008
	#define W5100_SHAR  		0x0009   // Source Hardware Address (MAC): 0x0009 to 0x000E
	#define W5100_SIPR			0x000F   // Source IP Address: 0x000F to 0x0012
	#define W5100_IR			0x0015   // Interrupt
	#define W5100_IMR			0x0016   // Interrupt mask
	#define W5100_RTR			0x0017   // Retry time
	#define W5100_RCR			0x0019   // Retry count
	#define W5100_RMSR			0x001A   // RX Memory Size Register
	#define W5100_TMSR			0x001B   // TX Memory Size Register
	#define W5100_PATR			0x001C   // Authentication type in PPPoE
	#define W5100_PTIMER		0x0028   // PPP LCP Request Timer
	#define W5100_PMAGIC		0x0029   // PPP LCP Magic Number
	#define W5100_UIPR			0x002A   // Unreachable IP Address
	#define W5100_UPORT			0x002E   // Unreachable Port

	#define W5100_CH_BASE		0x0400   // Socket base
	#define W5100_CH_SIZE		0x0100   // Size of each channel

	#define W5100_Sn_MR(ch)		(W5100_CH_BASE + (ch*W5100_CH_SIZE) + 0x00)   // Socket Mode
	#define W5100_Sn_CR(ch)		(W5100_CH_BASE + (ch*W5100_CH_SIZE) + 0x01)   // Socket Command
	#define W5100_Sn_IR(ch)		(W5100_CH_BASE + (ch*W5100_CH_SIZE) + 0x02)   // Socket Interrupt
	#define W5100_Sn_SR(ch)		(W5100_CH_BASE + (ch*W5100_CH_SIZE) + 0x03)   // Socket Status
	#define W5100_Sn_PORT(ch)	(W5100_CH_BASE + (ch*W5100_CH_SIZE) + 0x04)   // Socket Source Port
	#define W5100_Sn_DHAR(ch)	(W5100_CH_BASE + (ch*W5100_CH_SIZE) + 0x06)   // Socket Dest Hardware Addr
	#define W5100_Sn_DIPR(ch)	(W5100_CH_BASE + (ch*W5100_CH_SIZE) + 0x0C)   // Socket Dest IP Addr
	#define W5100_Sn_DPORT(ch)	(W5100_CH_BASE + (ch*W5100_CH_SIZE) + 0x10)   // Socket Dest Port
	#define W5100_Sn_MSSR(ch)	(W5100_CH_BASE + (ch*W5100_CH_SIZE) + 0x12)   // Socket Max Segment Size
	#define W5100_Sn_PROTO(ch)	(W5100_CH_BASE + (ch*W5100_CH_SIZE) + 0x14)   // Socket Protocol in IP Raw Mode
	#define W5100_Sn_TOS(ch)	(W5100_CH_BASE + (ch*W5100_CH_SIZE) + 0x15)   // Socket Type of Service
	#define W5100_Sn_TTL(ch)	(W5100_CH_BASE + (ch*W5100_CH_SIZE) + 0x16)   // Socket Time To Live

	#define W5100_Sn_TX_FSR(ch)	(W5100_CH_BASE + (ch*W5100_CH_SIZE) + 0x20)   // Socket TX Free Size
	#define W5100_Sn_TX_RD(ch)	(W5100_CH_BASE + (ch*W5100_CH_SIZE) + 0x22)   // Socket TX Read Pointer
	#define W5100_Sn_TX_WR(ch)	(W5100_CH_BASE + (ch*W5100_CH_SIZE) + 0x24)   // Socket TX Write Pointer

	#define W5100_Sn_RX_RSR(ch)	(W5100_CH_BASE + (ch*W5100_CH_SIZE) + 0x26)   // Socket RX Received Size
	#define W5100_Sn_RX_RD(ch)	(W5100_CH_BASE + (ch*W5100_CH_SIZE) + 0x28)   // Socket RX Read Pointer

	//4 channels... each channel to have 2k each
	#define RMSR_VAL            0x55
	#define TMSR_VAL            0x55

	//TX and RX memory
	#define W5100_TXMEM_BASE	0x4000
	#define W5100_TXMEM_SIZE	0x0800 // 2kb
	#define W5100_TXMEM_MASK	0x07FF
	#define W5100_TXMEM(ch)		(W5100_TXMEM_BASE + (ch*W5100_TXMEM_SIZE))	// start of socket tx data

	#define W5100_RXMEM_BASE	0x6000
	#define W5100_RXMEM_SIZE	0x0800 // 2kb
	#define W5100_RXMEM_MASK	0x07FF
	#define W5100_RXMEM(ch)		(W5100_RXMEM_BASE + (ch*W5100_RXMEM_SIZE))	// start of socket rx data

	// MODE register values
	#define W5100_MR_RST				0x80			// reset
	#define W5100_MR_PB					0x10			// ping block
	#define W5100_MR_PPPOE				0x08			// enable pppoe
	//#define W5100_MR_LB  				0x04			// little or big endian selector in indirect mode
	#define W5100_MR_AI					0x02			// auto-increment in indirect mode
	#define W5100_MR_IND				0x01			// enable indirect mode

	// IR/IRM register values
	#define W5100_IR_CONFLICT			0x80			// check ip confict
	#define W5100_IR_UNREACH			0x40			// get the destination unreachable message in UDP sending
	#define W5100_IR_PPPoE				0x20			// get the PPPoE close message
	#define W5100_IR_SOCK(ch)			(0x01 << ch)	// check socket interrupt

	// Sn_MR values
	#define W5100_Sn_MR_CLOSE			0x00			// unused socket
	#define W5100_Sn_MR_TCP				0x01			// TCP
	#define W5100_Sn_MR_UDP				0x02			// UDP
	#define W5100_Sn_MR_IPRAW			0x03			// IP LAYER RAW SOCK
	#define W5100_Sn_MR_MACRAW			0x04			// MAC LAYER RAW SOCK
	#define W5100_Sn_MR_PPPOE			0x05			// PPPoE
	#define W5100_Sn_MR_ND				0x20			// No Delayed Ack(TCP) flag
	#define W5100_Sn_MR_MULTI			0x80			// support multicating


	// Sn_CR values
	#define W5100_Sn_CR_OPEN			0x01			// initialize or open socket
	#define W5100_Sn_CR_LISTEN			0x02			// wait connection request in tcp mode(Server mode)
	#define W5100_Sn_CR_CONNECT			0x04			// send connection request in tcp mode(Client mode)
	#define W5100_Sn_CR_DISCON			0x08			// send closing reqeuset in tcp mode
	#define W5100_Sn_CR_CLOSE			0x10			// close socket
	#define W5100_Sn_CR_SEND			0x20			// updata txbuf pointer, send data
	#define W5100_Sn_CR_SEND_MAC		0x21			// send data with MAC address, so without ARP process
	#define W5100_Sn_CR_SEND_KEEP		0x22			//  send keep alive message
	#define W5100_Sn_CR_RECV			0x40			// update rxbuf pointer, recv data

	//#ifdef __DEF_IINCHIP_PPP__
	//	#define W5100_Sn_CR_PCON		0x23
	//	#define W5100_Sn_CR_PDISCON		0x24
	//	#define W5100_Sn_CR_PCR			0x25
	//	#define W5100_Sn_CR_PCN			0x26
	//	#define W5100_Sn_CR_PCJ			0x27
	//#endif

	// Sn_IR values
	//#ifdef __DEF_IINCHIP_PPP__
	//	#define W5100_Sn_IR_PRECV		0x80
	//	#define W5100_Sn_IR_PFAIL		0x40
	//	#define W5100_Sn_IR_PNEXT		0x20
	//#endif
	#define W5100_Sn_IR_SEND_OK			0x10			// complete sending
	#define W5100_Sn_IR_TIMEOUT			0x08			// assert timeout
	#define W5100_Sn_IR_RECV			0x04			// receiving data
	#define W5100_Sn_IR_DISCON			0x02			// closed socket
	#define W5100_Sn_IR_CON				0x01			// established connection

	// Sn_SR values
	#define W5100_SOCK_CLOSED			0x00			// closed
	#define W5100_SOCK_ARP	 			0x01			// ARP request sent
	#define W5100_SOCK_INIT 			0x13			// init state
	#define W5100_SOCK_LISTEN			0x14			// listen state
	#define W5100_SOCK_SYNSENT	   		0x15			// connection state
	#define W5100_SOCK_SYNRECV			0x16			// connection state
	#define W5100_SOCK_ESTABLISHED		0x17			// success to connect
	#define W5100_SOCK_FIN_WAIT			0x18			// closing state
	#define W5100_SOCK_CLOSING			0x1A			// closing state
	#define W5100_SOCK_TIME_WAIT		0x1B			// closing state
	#define W5100_SOCK_CLOSE_WAIT		0x1C			// closing state
	#define W5100_SOCK_LAST_ACK			0x1D			// closing state
	#define W5100_SOCK_UDP				0x22			// udp socket
	#define W5100_SOCK_IPRAW			0x32			// ip raw mode socket
	#define W5100_SOCK_MACRAW			0x42			// mac raw mode socket
	#define W5100_SOCK_PPPOE			0x5F			// pppoe socket

	// IP PROTOCOL
	#define W5100_IPPROTO_IP			0       	    // Dummy for IP (v6)
	#define W5100_IPPROTO_ICMP			1       	    // Control message protocol
	#define W5100_IPPROTO_IGMP			2       	    // Internet group management protocol
	#define W5100_IPPROTO_GGP			3       	    // Gateway^2 (deprecated)
	#define W5100_IPPROTO_TCP			6       	    // TCP
	#define W5100_IPPROTO_PUP			12      	    // PUP
	#define W5100_IPPROTO_UDP			17      	    // UDP
	#define W5100_IPPROTO_IDP			22      	    // XNS idp
	#define W5100_IPPROTO_ND			77      	    // UNOFFICIAL net disk protocol
	#define W5100_IPPROTO_RAW			255     	    // Raw IP packet

#elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

	#error "TODO: MIAC stuff here"

#else

	#error "Error - no Internet bus type defined."

#endif

#define CHANNEL_FLAG_0 0x01
#define CHANNEL_FLAG_1 0x02
#define CHANNEL_FLAG_2 0x04
#define CHANNEL_FLAG_3 0x08

#define Internet_RD_ADDR    (Internet_WR_ADDR+1)

//common globals
char Internet_PTR_VAL[4];
char Internet_WR_ADDR;

#if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)

    //common prototypes
    char Internet_MaskFromBit(char which_bit);
    void Internet_SendI2CByte(char val);
    void Internet_SendI2CStart();
    void Internet_SendI2CStop();
    char Internet_ReadI2CByte(char lastbyte);
    char Internet_SPI_Byte(char val);
    void Internet_WriteReg(char reg_h, char reg_l, char val);
    char Internet_ReadReg(char reg_h, char reg_l);
    void Internet_WritePtr(char reg_h, char reg_l);
    void Internet_ReadPtr(char reg_h, char reg_l);
    void Internet_ReadPtrWithShadow(char reg_shadow, char reg_pointer);
    void Internet_DisplayDebugInfo(char ch, char pos, char dispval, char erase);


#elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)

	//common globals
	unsigned short Internet_gTxPtr[4];
	unsigned short Internet_gTxSize[4];

	char Internet_SPI_Byte(char val);

	void Internet_WriteByte(unsigned short addr, char data);
	char Internet_ReadByte(unsigned short addr);
	void Internet_WriteWord(unsigned short addr, unsigned short data);
	unsigned short Internet_ReadWord(unsigned short addr);

#endif

//common implementations
char Internet_MaskFromBit(char which_bit)
{
   char retVal = 0;
   switch (which_bit)
   {
      case 0:
         retVal = 0x01;
         break;

      case 1:
         retVal = 0x02;
         break;

      case 2:
         retVal = 0x04;
         break;

      case 3:
         retVal = 0x08;
         break;

      case 4:
         retVal = 0x10;
         break;

      case 5:
         retVal = 0x20;
         break;

      case 6:
         retVal = 0x40;
         break;

      case 7:
         retVal = 0x80;
         break;
   }
   return (retVal);
}

// I2C functions
#if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)
  char Internet_ReadI2CByte(char lastbyte)
  {
      char retVal = 0;
      char mask;

  	#ifndef MX_MI2C
		#error "Chip does not have master I2C capability"
  	#else

      //wait until the bus is idle...
      //mask = Internet_MaskFromBit(R_W);
      mask = 0x04;
      while (sspstat & mask);     // tx in progress?

      //mask = Internet_MaskFromBit(ACKEN);
      //mask = mask | Internet_MaskFromBit(RCEN);
      //mask = mask | Internet_MaskFromBit(PEN);
      //mask = mask | Internet_MaskFromBit(RSEN);
      //mask = mask | Internet_MaskFromBit(SEN);
      mask = 0x1F;
      while (sspcon2 & mask);     // bus busy?

      //set RCEN and wait for it to clear
      //mask = Internet_MaskFromBit(RCEN);
      mask = 0x08;
      sspcon2 = sspcon2 | mask;
      while ((sspcon2 & mask) == mask);

      if (lastbyte == 0)
      {
          cr_bit(sspcon2, ACKDT);
      } else {
          st_bit(sspcon2, ACKDT);
      }

      //set ACKEN and wait for it to clear
      //mask = Internet_MaskFromBit(ACKEN);
      mask = 0x10;
      sspcon2 = sspcon2 | mask;
      while ((sspcon2 & mask) == mask);

      retVal = sspbuf;
	#endif
      return (retVal);
  }

  void Internet_SendI2CByte(char val)
  {
      char rw_mask;
      char mask;

    #ifndef MX_MI2C
		#error "Chip does not have master I2C capability"
  	#else

	  //rw_mask = Internet_MaskFromBit(R_W);
	  rw_mask = 0x04;

      //wait until the bus is idle...
      while (sspstat & rw_mask);     // tx in progress?

      //mask = Internet_MaskFromBit(ACKEN);
      //mask = mask | Internet_MaskFromBit(RCEN);
      //mask = mask | Internet_MaskFromBit(PEN);
      //mask = mask | Internet_MaskFromBit(RSEN);
      //mask = mask | Internet_MaskFromBit(SEN);
      mask = 0x1F;
      while (sspcon2 & mask);        // bus busy?

      //mask = Internet_MaskFromBit(BF);
      mask = 0x01;
      while (sspstat & mask);        // buffer still full?

      sspbuf = val;

      while (sspstat & rw_mask);     // tx in progress?

      //mask = Internet_MaskFromBit(ACKSTAT);
      //while (sspcon2 & mask);        // ack?
	#endif
      return;
  }

  void Internet_SendI2CStart()
  {
    #ifndef MX_MI2C
		#error "Chip does not have master I2C capability"
  	#else

      //set SEN and wait for it to clear
      char mask;
      //mask = Internet_MaskFromBit(SEN);
      mask = 0x01;
      sspcon2 = sspcon2 | mask;
      while ((sspcon2 & mask) == mask);
	#endif
      return;
  }

  void Internet_SendI2CStop()
  {
    #ifndef MX_MI2C
		#error "Chip does not have master I2C capability"
  	#else
      //set PEN and wait for it to clear
      char mask;
      //mask = Internet_MaskFromBit(PEN);
      mask = 0x04;
      sspcon2 = sspcon2 | mask;
      while ((sspcon2 & mask) == mask);
	#endif
      return;
  }

#elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)

	char Internet_SPI_Byte(char val)
	{
		char Retval = 0;

		#ifndef MX_SPI
			#error "Chip does not have SPI capability"
		#else
			sspbuf = val;					//send a Char
			while ((sspstat & 0x01) == 0);	//wait until sspbuf is empty
			Retval = sspbuf;				//receive the Char
		#endif

		return Retval;
	}

#elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

	#error "TODO: MIAC stuff here"

#else

	#error "Error - no Internet bus type defined."

#endif

////////////////////////////////////////////////////
// Misc functions
////////////////////////////////////////////////////

#if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)

#define Internet_SendHtmlChar(c)    Internet_SendI2CByte((c))
#define Internet_ReadHtmlChar(ch)   Internet_ReadI2CByte((ch))
#define Internet_ReadHtmlCharNull() Internet_SendI2CStop(); \
                                    /*finally, set RR pointer to RW value (to make W3100A think all data has been read)*/ \
                                    Internet_PTR_VAL[0] = rw_ptr[0]; \
                                    Internet_PTR_VAL[1] = rw_ptr[1]; \
                                    Internet_PTR_VAL[2] = rw_ptr[2]; \
                                    Internet_PTR_VAL[3] = rw_ptr[3]; \
                                    /*Update RR_PR*/ \
                                    Internet_WritePtr(0, REG_Cx_RR_PR(ch));

void Internet_WriteReg(char reg_h, char reg_l, char val)
{
	Internet_SendI2CStart();
	Internet_SendI2CByte(Internet_WR_ADDR);     //write to W3100A
	Internet_SendI2CByte(reg_h);                //addr (hi byte)
	Internet_SendI2CByte(reg_l);                //addr (low byte)
	Internet_SendI2CByte(val);                  //data
	Internet_SendI2CStop();
}

char Internet_ReadReg(char reg_h, char reg_l)
{
    char retVal;

	Internet_SendI2CStart();
	Internet_SendI2CByte(Internet_WR_ADDR);     //write to W3100A
	Internet_SendI2CByte(reg_h);                //addr (hi byte)
	Internet_SendI2CByte(reg_l);                //addr (low byte)
	Internet_SendI2CStop();
	Internet_SendI2CStart();
	Internet_SendI2CByte(Internet_RD_ADDR);      //read from W3100A
	retVal = Internet_ReadI2CByte(1);            //read data
	Internet_SendI2CStop();

    return (retVal);
}

void Internet_WritePtr(char reg_h, char reg_l)
{
	Internet_SendI2CStart();
	Internet_SendI2CByte(Internet_WR_ADDR);        //write to W3100A
	Internet_SendI2CByte(reg_h);                //addr (hi byte)
	Internet_SendI2CByte(reg_l);                //addr (low byte)
	Internet_SendI2CByte(Internet_PTR_VAL[3]);    //data
	Internet_SendI2CByte(Internet_PTR_VAL[2]);
	Internet_SendI2CByte(Internet_PTR_VAL[1]);
	Internet_SendI2CByte(Internet_PTR_VAL[0]);
	Internet_SendI2CStop();

    MX_INTERNET_SHORTDELAY
}

void Internet_ReadPtr(char reg_h, char reg_l)
{
	Internet_SendI2CStart();
	Internet_SendI2CByte(Internet_WR_ADDR);      //write to W3100A
	Internet_SendI2CByte(reg_h);          //addr (hi byte)
	Internet_SendI2CByte(reg_l);          //addr (low byte)
	Internet_SendI2CStop();
	Internet_SendI2CStart();
	Internet_SendI2CByte(Internet_RD_ADDR);      //read from W3100A
	Internet_PTR_VAL[3] = Internet_ReadI2CByte(0); //read data
	Internet_PTR_VAL[2] = Internet_ReadI2CByte(0);
	Internet_PTR_VAL[1] = Internet_ReadI2CByte(0);
	Internet_PTR_VAL[0] = Internet_ReadI2CByte(1);
	Internet_SendI2CStop();
}

void Internet_ReadPtrWithShadow(char reg_shadow, char reg_pointer)
{
    //read the shadow reg first
    Internet_ReadReg(SHADOW_REG_HI, reg_shadow);
    MX_INTERNET_SHORTDELAY
    //now read the actual register
    Internet_ReadPtr(0, reg_pointer);
    MX_INTERNET_SHORTDELAY
}


#elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)

    // Sends a byte into the HTML buffer for the assigned channel
    #define Internet_SendHtmlChar(c)      Internet_WriteByte(Internet_gTxPtr[%a_CHANNEL], (c))
    #define Internet_ReadHtmlChar(ch)     ((size-- > 0) ? Internet_ReadByte(address < (W5100_RXMEM(ch) + W5100_RXMEM_MASK) ? (address + 1) : W5100_RXMEM(ch)) : 0)
    #define Internet_ReadHtmlCharNull()

	void Internet_WriteByte(unsigned short addr, char data)
	{
		//TODO: disable interrupts?
		clear_bit(%a_CS_PORT, %a_CS_PIN);	//enable SPI device
		//TODO: may need a small delay here (fast chips only??) - and also in other similar places.
		Internet_SPI_Byte(0xF0);									//Write Op Code
		Internet_SPI_Byte((addr & 0xFF00) >> 8);
		Internet_SPI_Byte((addr & 0xFF));
		Internet_SPI_Byte(data);
		set_bit(%a_CS_PORT, %a_CS_PIN);		//disable SPI device
		//TODO: re-enable interrupts?
	}

	char Internet_ReadByte(unsigned short addr)
	{
		char retVal;

		//TODO: disable interrupts
		clear_bit(%a_CS_PORT, %a_CS_PIN);	//enable SPI device
		Internet_SPI_Byte(0x0F);									//Read Op Code
		Internet_SPI_Byte((addr & 0xFF00) >> 8);
		Internet_SPI_Byte((addr & 0xFF));
		retVal = Internet_SPI_Byte(0xAA);
		set_bit(%a_CS_PORT, %a_CS_PIN);		//disable SPI device
		//TODO: re-enable interrupts

		return (retVal);
	}

	void Internet_WriteWord(unsigned short addr, unsigned short data)
	{
		//TODO: disable interrupts
		char data_lo = (data & 0xFF);
		char data_hi = (data & 0xFF00) >> 8;
		char addr_lo = (addr & 0xFF);
		char addr_hi = (addr & 0xFF00) >> 8;

		//write hi byte
		clear_bit(%a_CS_PORT, %a_CS_PIN);	//enable SPI device
		Internet_SPI_Byte(0xF0);									//Write Op Code
		Internet_SPI_Byte(addr_hi);
		Internet_SPI_Byte(addr_lo);
		Internet_SPI_Byte(data_hi);
		set_bit(%a_CS_PORT, %a_CS_PIN);		//disable SPI device
		MX_INTERNET_SHORTDELAY									//need a delay of at least 32 ns here

		//write lo byte
		clear_bit(%a_CS_PORT, %a_CS_PIN);	//enable SPI device
		Internet_SPI_Byte(0xF0);									//Write Op Code
		Internet_SPI_Byte(addr_hi);
		Internet_SPI_Byte(addr_lo + 1);
		Internet_SPI_Byte(data_lo);
		set_bit(%a_CS_PORT, %a_CS_PIN);		//disable SPI device
		MX_INTERNET_SHORTDELAY									//need a delay of at least 32 ns here

		//TODO: re-enable interrupts
	}

	unsigned short Internet_ReadWord(unsigned short addr)
	{
		unsigned short retVal;
		char addr_lo = (addr & 0xFF);
		char addr_hi = (addr & 0xFF00) >> 8;

		//TODO: disable interrupts

		//read hi byte
		clear_bit(%a_CS_PORT, %a_CS_PIN);	//enable SPI device
		Internet_SPI_Byte(0x0F);									//Write Op Code
		Internet_SPI_Byte(addr_hi);
		Internet_SPI_Byte(addr_lo);
		retVal = Internet_SPI_Byte(0xaa);
		set_bit(%a_CS_PORT, %a_CS_PIN);		//disable SPI device
		MX_INTERNET_SHORTDELAY									//need a delay of at least 32 ns here

		retVal = retVal << 8;

		//read lo byte
		clear_bit(%a_CS_PORT, %a_CS_PIN);	//enable SPI device
		Internet_SPI_Byte(0x0F);									//Write Op Code
		Internet_SPI_Byte(addr_hi);
		Internet_SPI_Byte(addr_lo+1);
		retVal = retVal | Internet_SPI_Byte(0xaa);
		set_bit(%a_CS_PORT, %a_CS_PIN);		//disable SPI device
		MX_INTERNET_SHORTDELAY									//need a delay of at least 32 ns here
		//TODO: re-enable interrupts

		return (retVal);
	}

#elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

	#error "TODO: MIAC stuff here"

#else

	#error "Error - no Internet bus type defined."

#endif




void Internet_DisplayDebugInfo(char ch, char pos, char dispval, char erase)
{
  #ifdef MX_INTERNET_DEBUG
    char curs_x;
    char curs_y;
    char blank_loop;
    if (ch == 0)
    {
        curs_x = 0;
        curs_y = 0;
    }
    if (ch == 1)
    {
        curs_x = 8;
        curs_y = 0;
    }
    if (ch == 2)
    {
        curs_x = 0;
        curs_y = 1;
    }
    if (ch == 3)
    {
        curs_x = 8;
        curs_y = 1;
    }
    if (erase != 0)
    {
        FCD_LCDDisplay0_Cursor(curs_x,curs_y);
        for (blank_loop=0; blank_loop<8; blank_loop++)
        {
            FCD_LCDDisplay0_PrintASCII(' ');
        }
    }
    curs_x = curs_x + pos;
    FCD_LCDDisplay0_Cursor(curs_x,curs_y);
    FCD_LCDDisplay0_PrintASCII(dispval);
  #endif
}

#endif
//////////////////////////////////////////////////////////////////////////////////////////


#define %a_W3100A_WR     %b
#define %a_W3100A_RD     %b+1
#define %a_INT_PIN       %c
#define %a_INT_MASK      %d
#define %a_I2C_BAUD      %e
#define %a_INT_PORT      port%u
#define %a_INT_TRIS      tris%u

//GAR
%f

//SMR
%g

//SHAR
%h

//SIPR
%i


//the following sets the number of consecutive sockets, and the memory allocation for them
//#define %a_MAX_CH0
//#define %a_MAX_CH1
//#define %a_MAX_CH2
#define %a_MAX_CH3

//the following defines the number of HTML pages and the names of these pages
#define %a_MAX_PAGES%j

//Defines to automatically code the appropriate pages...
#ifdef %a_MAX_PAGES1
  #define %a_MAX_PAGES    1
  #define %a_USE_PAGE1
#endif
#ifdef %a_MAX_PAGES2
  #define %a_MAX_PAGES    2
  #define %a_USE_PAGE1
  #define %a_USE_PAGE2
#endif
#ifdef %a_MAX_PAGES3
  #define %a_MAX_PAGES    3
  #define %a_USE_PAGE1
  #define %a_USE_PAGE2
  #define %a_USE_PAGE3
#endif
#ifdef %a_MAX_PAGES4
  #define %a_MAX_PAGES    4
  #define %a_USE_PAGE1
  #define %a_USE_PAGE2
  #define %a_USE_PAGE3
  #define %a_USE_PAGE4
#endif

#ifdef %a_USE_PAGE1
  const char* %a_HTML_PAGE1 = "%k.htm";
  void %a_SendHTMLPage1();
#endif
#ifdef %a_USE_PAGE2
  const char* %a_HTML_PAGE2 = "%l.htm";
  void %a_SendHTMLPage2();
#endif
#ifdef %a_USE_PAGE3
  const char* %a_HTML_PAGE3 = "%m.htm";
  void %a_SendHTMLPage3();
#endif
#ifdef %a_USE_PAGE4
  const char* %a_HTML_PAGE4 = "%n.htm";
  void %a_SendHTMLPage4();
#endif


#ifdef %a_MAX_CH0
  //only channel 0, so give it all 8k
  #define %a_RMSR_VAL            0x03
  #define %a_TMSR_VAL            0x03

  //these are the high offsets (low byte = 0x00) and masks (low byte = 0xFF)
  //note that TX3 (etc) need to be defined even if they are not used
  #define %a_MEM_TX0_OFFSET    0x40
  #define %a_MEM_TX1_OFFSET    0x40
  #define %a_MEM_TX2_OFFSET    0x40
  #define %a_MEM_TX3_OFFSET    0x40
  #define %a_MEM_TX0_MASK      0x1F
  #define %a_MEM_TX1_MASK      0x1F
  #define %a_MEM_TX2_MASK      0x1F
  #define %a_MEM_TX3_MASK      0x1F

  #define %a_MEM_RX0_OFFSET    0x60
  #define %a_MEM_RX1_OFFSET    0x60
  #define %a_MEM_RX2_OFFSET    0x60
  #define %a_MEM_RX3_OFFSET    0x60
  #define %a_MEM_RX0_MASK      0x1F
  #define %a_MEM_RX1_MASK      0x1F
  #define %a_MEM_RX2_MASK      0x1F
  #define %a_MEM_RX3_MASK      0x1F
#endif

#ifdef %a_MAX_CH1
  //2 channels with 4k each
  #define %a_RMSR_VAL            0x0A
  #define %a_TMSR_VAL            0x0A

  //these are the high offsets (low byte = 0x00) and masks (low byte = 0xFF)
  //note that TX3 (etc) need to be defined even if they are not used
  #define %a_MEM_TX0_OFFSET    0x40
  #define %a_MEM_TX1_OFFSET    0x50
  #define %a_MEM_TX2_OFFSET    0x40
  #define %a_MEM_TX3_OFFSET    0x50
  #define %a_MEM_TX0_MASK      0x0F
  #define %a_MEM_TX1_MASK      0x0F
  #define %a_MEM_TX2_MASK      0x0F
  #define %a_MEM_TX3_MASK      0x0F

  #define %a_MEM_RX0_OFFSET    0x60
  #define %a_MEM_RX1_OFFSET    0x70
  #define %a_MEM_RX2_OFFSET    0x60
  #define %a_MEM_RX3_OFFSET    0x70
  #define %a_MEM_RX0_MASK      0x0F
  #define %a_MEM_RX1_MASK      0x0F
  #define %a_MEM_RX2_MASK      0x0F
  #define %a_MEM_RX3_MASK      0x0F
#endif

#ifdef %a_MAX_CH2
  //3 channels - 4k in channel 0 and 2k in channels 1 and 2
  #define %a_RMSR_VAL            0x16
  #define %a_TMSR_VAL            0x16

  //these are the high offsets (low byte = 0x00) and masks (low byte = 0xFF)
  //note that TX3 (etc) need to be defined even if they are not used
  #define %a_MEM_TX0_OFFSET    0x40
  #define %a_MEM_TX1_OFFSET    0x50
  #define %a_MEM_TX2_OFFSET    0x58
  #define %a_MEM_TX3_OFFSET    0x58
  #define %a_MEM_TX0_MASK      0x0F
  #define %a_MEM_TX1_MASK      0x07
  #define %a_MEM_TX2_MASK      0x07
  #define %a_MEM_TX3_MASK      0x07

  #define %a_MEM_RX0_OFFSET    0x60
  #define %a_MEM_RX1_OFFSET    0x70
  #define %a_MEM_RX2_OFFSET    0x78
  #define %a_MEM_RX3_OFFSET    0x78
  #define %a_MEM_RX0_MASK      0x0F
  #define %a_MEM_RX1_MASK      0x07
  #define %a_MEM_RX2_MASK      0x07
  #define %a_MEM_RX3_MASK      0x07
#endif

#ifdef %a_MAX_CH3
  //each channel to have 2k each
  #define %a_RMSR_VAL          0x55
  #define %a_TMSR_VAL          0x55

  //these are the high offsets (low byte = 0x00) and masks (low byte = 0xFF)
  #define %a_MEM_TX0_OFFSET    0x40
  #define %a_MEM_TX1_OFFSET    0x48
  #define %a_MEM_TX2_OFFSET    0x50
  #define %a_MEM_TX3_OFFSET    0x58
  #define %a_MEM_TX0_MASK      0x07
  #define %a_MEM_TX1_MASK      0x07
  #define %a_MEM_TX2_MASK      0x07
  #define %a_MEM_TX3_MASK      0x07

  #define %a_MEM_RX0_OFFSET    0x60
  #define %a_MEM_RX1_OFFSET    0x68
  #define %a_MEM_RX2_OFFSET    0x70
  #define %a_MEM_RX3_OFFSET    0x78
  #define %a_MEM_RX0_MASK      0x07
  #define %a_MEM_RX1_MASK      0x07
  #define %a_MEM_RX2_MASK      0x07
  #define %a_MEM_RX3_MASK      0x07
#endif



//the following calculates MAX_CHANNEL and USE_CHx defines from appropriate MAX_CHx setting:
//NOTE: we always use ch0
#define %a_USE_CH0
#ifdef %a_MAX_CH0
  #define %a_MAX_CHANNEL       0
  #define %a_ARRAY_SIZE        1
  #define %a_PTR_REG_SIZE      4
#endif
#ifdef %a_MAX_CH1
  #define %a_MAX_CHANNEL       1
  #define %a_ARRAY_SIZE        2
  #define %a_PTR_REG_SIZE      8
  #define %a_USE_CH1
#endif
#ifdef %a_MAX_CH2
  #define %a_MAX_CHANNEL       2
  #define %a_ARRAY_SIZE        3
  #define %a_PTR_REG_SIZE      12
  #define %a_USE_CH1
  #define %a_USE_CH2
#endif
#ifdef %a_MAX_CH3
  #define %a_MAX_CHANNEL       3
  #define %a_ARRAY_SIZE        4
  #define %a_PTR_REG_SIZE      16
  #define %a_USE_CH1
  #define %a_USE_CH2
  #define %a_USE_CH3
#endif


//misc defines for our code
#define %a_MAX_BUFFER_CHECKS    (20)
#define %a_MAX_NON_LISTEN       (20)
#define %a_MAX_SUBSTITUTIONS    4
#define %a_MAX_PAGENAME_LENGTH  16



// globals
char %a_CHANNEL;
char %a_CHANNEL_FLAG;
char %a_STATUS[%a_ARRAY_SIZE];
char %a_DATA_SENT_CNT[%a_ARRAY_SIZE];
char %a_SAME_STATE_CNT[%a_ARRAY_SIZE];
char %a_LAST_STATE[%a_ARRAY_SIZE];
char %a_SEQ_NUM[4];
char %a_ACK_PTR[%a_PTR_REG_SIZE];
char %a_TX_PTR[%a_PTR_REG_SIZE];
char %a_SUB_VAL[%a_MAX_SUBSTITUTIONS];

//prototypes
void %a_UpdateSeqnum(char ch);
char %a_Check_Int();
char %a_Listen(char ch);
void %a_InitialiseGlobals(char ch);
char %a_CheckForProbs(char ch);
char %a_ProcessSocket(char ch);
void %a_ReadTWPtrReg(char ch);
void %a_ReadTAPtrReg(char ch);
char %a_TW_TA_Comp(char ch);
void %a_WriteString(const char* string);
void %a_IncTxPointer(char idxBase);
void %a_FlushBuffer(char ch);
char %a_RecreateSocket(char ch);
char %a_GetHTMLReq(char ch);

////////////////////////////////////////////////////
// Misc functions
////////////////////////////////////////////////////

//socket functions
/*
  - Not done
  + Done, no code
  * Done, code added
  = Ignored, code the same
  ? Come back to this
  X Unused, no code

  X a_Check_Int
  * a_RecreateSocket
  * a_UpdateSeqnum
  * a_Listen
  = a_InitialiseGlobals
  + a_CheckForProbs
  * a_ProcessSocket
  X a_TW_TA_Comp
  X a_ReadTWPtrReg
  X a_ReadTAPtrReg
  - a_WriteString
  * a_IncTxPointer
  * a_FlushBuffer
  * a_GetHTMLReq

  * Initialize
  * CreateServerSocket
  * CheckSocketActivity
*/
char %a_Check_Int()
{
    //periodically query this and populate the %a_STATUS[] array
    char retVal = 0;
    char in_val;
    char status;
    char isr_val;

    //make sure we are addressing this device
    Internet_WR_ADDR = %a_W3100A_WR;

  #if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)

    //read the appropriate port
  #ifdef MX_I2C_B
    in_val = %a_INT_PORT;
  #endif

  #ifdef MX_I2C_C
    in_val = %a_INT_PORT;
  #endif

    if ((in_val & %a_INT_MASK) == 0)
    {
        //the interrupt pin is set
        status = Internet_ReadReg(0, REG_IR);
        retVal = status;

        while (status)
        {

            if (status & 0x01)
            {
                //channel 0 interrupt
                isr_val = Internet_ReadReg(0, REG_Cx_ISR(0));
                Internet_WriteReg(0, REG_IR, 0x01);
                #ifdef %a_USE_CH0
                  %a_STATUS[0] = isr_val;
                #endif
            }
            if (status & 0x02)
            {
                //channel 1 interrupt
                isr_val = Internet_ReadReg(0, REG_Cx_ISR(1));
                Internet_WriteReg(0, REG_IR, 0x02);
                #ifdef %a_USE_CH1
                  %a_STATUS[1] = isr_val;
                #endif
            }
            if (status & 0x04)
            {
                //channel 2 interrupt
                isr_val = Internet_ReadReg(0, REG_Cx_ISR(2));
                Internet_WriteReg(0, REG_IR, 0x04);
                #ifdef %a_USE_CH2
                  %a_STATUS[2] = isr_val;
                #endif
            }
            if (status & 0x08)
            {
                //channel 3 interrupt
                isr_val = Internet_ReadReg(0, REG_Cx_ISR(3));
                Internet_WriteReg(0, REG_IR, 0x08);
                #ifdef %a_USE_CH3
                  %a_STATUS[3] = isr_val;
                #endif
            }

            if (status & 0x10)
            {
                //channel 0 receive interrupt
                Internet_WriteReg(0, REG_IR, 0x10);
            }
            if (status & 0x20)
            {
                //channel 1 receive interrupt
                Internet_WriteReg(0, REG_IR, 0x20);
            }
            if (status & 0x40)
            {
                //channel 2 receive interrupt
                Internet_WriteReg(0, REG_IR, 0x40);
            }
            if (status & 0x80)
            {
                //channel 3 receive interrupt
                Internet_WriteReg(0, REG_IR, 0x80);
            }

            status = Internet_ReadReg(0, REG_IR);
        }
        Internet_WriteReg(0, REG_IR, 0xFF);

    }
  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)

	// Unused, no code here

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

	#error "TODO: MIAC stuff here"

  #else

	#error "Error - no Internet bus type defined."

  #endif


    return (retVal);
}

char %a_RecreateSocket(char ch)
{
    if (ch > %a_MAX_CHANNEL)
    {
        return (255);  //error trying to open a non-existent channel
    }
  #if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)

    //make sure we are addressing this device
    Internet_WR_ADDR = %a_W3100A_WR;

    //initialise the socket
    %a_STATUS[ch] = 0;
    Internet_WriteReg(0, REG_Cx_CR(ch), CSOCK_INIT);

    //check for result
    while (%a_STATUS[ch] == 0)
    {
        %a_Check_Int();
    }

    //check the result
    if ((%a_STATUS[ch] & SSOCK_INIT_OK) == 0)
    {
        return (%a_STATUS[ch]);  //failed to open the socket
    }

    %a_UpdateSeqnum(ch);

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)

	Internet_gTxSize[%a_CHANNEL] = 0;
	Internet_gTxPtr[%a_CHANNEL] = 0;
    // Not timed out, so ok to close
    Internet_WriteByte(W5100_Sn_CR(0), W5100_Sn_CR_DISCON); // W5100_Sn_CR_CLOSE);
    while(Internet_ReadByte(W5100_Sn_CR(ch)));
    // Disconnected, now close
    Internet_WriteByte(W5100_Sn_CR(0), W5100_Sn_CR_CLOSE);
    while(Internet_ReadByte(W5100_Sn_CR(ch)));
    // Delay to allow states to settle
    delay_ms(5);
	// Open the socket
  	{
	    char state = Internet_ReadByte(W5100_Sn_SR(ch));
	    while(state != W5100_SOCK_INIT)
	    {
		    //Create the TCP layer socket
		    //set up the socket protocol and option flags
		    Internet_WriteByte(W5100_Sn_MR(ch), W5100_Sn_MR_TCP);
		    //initialise the socket
		    Internet_WriteByte(W5100_Sn_CR(ch), W5100_Sn_CR_OPEN);
		    // Waiting until the S0_CR is clear
		    while(Internet_ReadByte(W5100_Sn_CR(ch)));
		    state = Internet_ReadByte(W5100_Sn_SR(ch));
	    }
	}
  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

    #error "TODO: MIAC stuff here"

  #else

    #error "Error - no Internet bus type defined."

  #endif
    return (0);
}

void %a_UpdateSeqnum(char ch)
{

  #if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)
    //make sure we are addressing this device
    Internet_WR_ADDR = %a_W3100A_WR;

    //increase the 4-byte seq_num value
    %a_SEQ_NUM[3] = %a_SEQ_NUM[3] + 1;
    if (%a_SEQ_NUM[3] == 0)
    {
        %a_SEQ_NUM[2] = %a_SEQ_NUM[2] + 1;
        if (%a_SEQ_NUM[2] == 0)
        {
            %a_SEQ_NUM[1] = %a_SEQ_NUM[1] + 1;
            if (%a_SEQ_NUM[1] == 0)
            {
                %a_SEQ_NUM[0] = %a_SEQ_NUM[0] + 1;
            }
        }
    }

    //save the value
    Internet_PTR_VAL[0] = %a_SEQ_NUM[0];
    Internet_PTR_VAL[1] = %a_SEQ_NUM[1];
    Internet_PTR_VAL[2] = %a_SEQ_NUM[2];
    Internet_PTR_VAL[3] = %a_SEQ_NUM[3];

    //Update TW_PR
    Internet_WritePtr(0, REG_Cx_TW_PR(ch));

    //Update TR_PR
    Internet_WritePtr(0, REG_Cx_TR_PR(ch));

    //Update TA_PR
    Internet_WritePtr(0, REG_Cx_TA_PR(ch));
  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)

    // Nothing else to do here

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

    #error "TODO: MIAC stuff here"

  #else

    #error "Error - no Internet bus type defined."

  #endif
}

char %a_Listen(char ch)
{
    char flag;
    if (ch > %a_MAX_CHANNEL)
    {
        return (255);  //error trying to open a non-existent channel
    }

  #if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)
    //make sure we are addressing this device
    Internet_WR_ADDR = %a_W3100A_WR;

    %a_STATUS[ch] = 0;
    Internet_WriteReg(0, REG_Cx_CR(ch), CLISTEN);

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)

    Internet_WriteByte(W5100_Sn_CR(ch), W5100_Sn_CR_LISTEN); // Send command 'listen'

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

    #error "TODO: MIAC stuff here"

  #else

    #error "Error - no Internet bus type defined."

  #endif

    %a_InitialiseGlobals(ch);

    flag = 1 << ch; // Eg: CHANNEL_FLAG_0
    %a_CHANNEL_FLAG = %a_CHANNEL_FLAG | flag;
    flag = flag * 0x10;
    %a_CHANNEL_FLAG = %a_CHANNEL_FLAG & (~flag);
    return (0);
}

void %a_InitialiseGlobals(char ch)
{
    %a_DATA_SENT_CNT[ch] = 0;
    %a_SAME_STATE_CNT[ch] = 0;
    %a_LAST_STATE[ch] = 0;
}

char %a_CheckForProbs(char ch)
{
    char TA_TW_Result;

    if (ch > %a_MAX_CHANNEL)
    {
        return (255);  //error trying to open a non-existant channel
    }

  #if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)
    //make sure we are addressing this device
    Internet_WR_ADDR = %a_W3100A_WR;

    char sockState = Internet_ReadReg(0, REG_Cx_SSR(ch));

    if ((sockState == SOCK_ESTABLISHED) && (%a_DATA_SENT_CNT[ch] > 0))
    {
        //socket is established, so we are waiting for the Tx buffer inside
        //the device to empty before we can close the socket
        %a_ReadTAPtrReg(ch);

        TA_TW_Result = %a_TW_TA_Comp(ch);
        if (TA_TW_Result == 0)
        {
            Internet_DisplayDebugInfo(ch, 5, 'h', 0);

            //transmission has finished, so close the channel
            Internet_WriteReg(0, REG_Cx_CR(ch), CCLOSE);

        } else {
            Internet_DisplayDebugInfo(ch, 4, TA_TW_Result, 0);

            %a_DATA_SENT_CNT[ch] = %a_DATA_SENT_CNT[ch] + 1;
            if (%a_DATA_SENT_CNT[ch] > %a_MAX_BUFFER_CHECKS)
            {
                Internet_DisplayDebugInfo(ch, 0, 'i', 0);

                //give up: close the socket and start over
                %a_RecreateSocket(ch);
                %a_Listen(ch);
            }
        }

    //} else if ((sockState != SOCK_LISTEN) && (sockState != SOCK_CLOSED)) {
    } else if (sockState != SOCK_LISTEN) {

        if (sockState == %a_LAST_STATE[ch])
        {
            %a_SAME_STATE_CNT[ch] = %a_SAME_STATE_CNT[ch] + 1;
            if (%a_SAME_STATE_CNT[ch] > %a_MAX_NON_LISTEN)
            {
                Internet_DisplayDebugInfo(ch, 0, 'j', 0);

                //give up: close the socket and start over
                %a_RecreateSocket(ch);
                %a_Listen(ch);
            }
        }

    } else {

        //things are ok, so reset the same state count
        %a_SAME_STATE_CNT[ch] = 0;
    }


    //keep a track of the last state
    %a_LAST_STATE[ch] = sockState;

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)

    // TODO: Add code to reset internet state if failed to send

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

    #error "TODO: MIAC stuff here"

  #else

    #error "Error - no Internet bus type defined."

  #endif
    // Edited for V3.0.1
    // needs to return a value
    //assumed it to be TA_TW_Result
    return TA_TW_Result;
}

char %a_ProcessSocket(char ch)
{
    char flag;
    char rx_page;

    if (ch > %a_MAX_CHANNEL)
    {
        return (255);  //error trying to open a non-existant channel
    }

  #if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)
    //the following is for debug purposes...
    switch (ch)
    {
      case 0:
        ch = 0;
        break;

      case 1:
        ch = 1;
        break;

      case 2:
        ch = 2;
        break;

      case 3:
        ch = 3;
        break;
    }

    //make sure we are addressing this device
    Internet_WR_ADDR = %a_W3100A_WR;

    if (%a_STATUS[ch] & SESTABLISHED)
    {

        Internet_DisplayDebugInfo(ch, 0, 'b', 1);

        //a connection has been established
        %a_STATUS[ch] = %a_STATUS[ch] & (~SESTABLISHED);
        Internet_WriteReg(0, REG_Cx_CR(ch), CRECV);
    }

    if (%a_STATUS[ch] & SRECV_OK)
    {
        Internet_DisplayDebugInfo(ch, 1, 'c', 0);

        //we have received a message
        %a_STATUS[ch] = %a_STATUS[ch] & (~SRECV_OK);

        //Modified to account for multiple HTML pages...
        //SendHTMLPage(ch);

        rx_page = %a_GetHTMLReq(ch);

        //read the TX Write register
        %a_ReadTWPtrReg(ch);
        %a_CHANNEL = ch;

        //now fill the TX buffer...
        if ((rx_page > 0) && (rx_page <= %a_MAX_PAGES))
        {
            //send the header text...
            %a_WriteString("HTTP/1.0 200 OK");
            %a_WriteString("Content-type: text/html");
            %a_WriteString("");
            %a_WriteString("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">");

            //now send the actual html...
            switch (rx_page)
            {
              #ifdef %a_USE_PAGE1
                case 1:
                    %a_SendHTMLPage1();
                    break;
              #endif

              #ifdef %a_USE_PAGE2
                case 2:
                    %a_SendHTMLPage2();
                    break;
              #endif

              #ifdef %a_USE_PAGE3
                case 3:
                    %a_SendHTMLPage3();
                    break;
              #endif

              #ifdef %a_USE_PAGE4
                case 4:
                    %a_SendHTMLPage4();
                    break;
              #endif

                default:
                    %a_SendHTMLPage1();
                    break;
            }

        } else {
            //not found
            %a_WriteString("HTTP/1.0 404 Not Found");
        }


        //finally, transmit the buffer
        %a_FlushBuffer(ch);

        Internet_DisplayDebugInfo(ch, 2, 'f', 0);
    }

    if (%a_STATUS[ch] & SSEND_OK)
    {
        Internet_DisplayDebugInfo(ch, 3, 'd', 0);

        //data transmission is in progress
        %a_STATUS[ch] = %a_STATUS[ch] & (~SSEND_OK);
        %a_ReadTWPtrReg(ch);
        %a_DATA_SENT_CNT[ch] = 1;
    }

    if (%a_STATUS[ch] & SCLOSED)
    {
        Internet_DisplayDebugInfo(ch, 6, 'e', 0);

        flag = 1 << ch; // Eg: CHANNEL_FLAG_0
        %a_CHANNEL_FLAG = %a_CHANNEL_FLAG & (~flag);
        flag = flag * 0x10;
        %a_CHANNEL_FLAG = %a_CHANNEL_FLAG | flag;

        //reset the socket
        %a_STATUS[ch] = %a_STATUS[ch] & (~SCLOSED);

        //reopen the socket and listen
        %a_RecreateSocket(ch);
        %a_Listen(ch);

        Internet_DisplayDebugInfo(ch, 7, 'r', 0);
    }
  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)

    // If it is, then send the HTTP for this
    %a_CHANNEL = ch;
    // Check if 'GET /' is recieved
    rx_page = %a_GetHTMLReq(ch);
    {
	    // Read the Tx Write Pointer
	    unsigned short tx_ptr_val;
	    tx_ptr_val = Internet_ReadWord(W5100_Sn_TX_WR(%a_CHANNEL));
	    Internet_gTxPtr[%a_CHANNEL] = W5100_TXMEM(%a_CHANNEL) + (tx_ptr_val & W5100_TXMEM_MASK);
	    Internet_gTxSize[%a_CHANNEL] = 0;
	}

    //now fill the TX buffer...
    if ((rx_page > 0) && (rx_page <= %a_MAX_PAGES))
    {
        //send the header text...
        %a_WriteString("HTTP/1.0 200 OK");
        %a_WriteString("Content-type: text/html");
        %a_WriteString("");
        %a_WriteString("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">");

        //now send the actual html...
        switch (rx_page)
        {
          #ifdef %a_USE_PAGE1
            case 1:
                %a_SendHTMLPage1();
                break;
          #endif

          #ifdef %a_USE_PAGE2
            case 2:
                %a_SendHTMLPage2();
                break;
          #endif

          #ifdef %a_USE_PAGE3
            case 3:
                %a_SendHTMLPage3();
                break;
          #endif

          #ifdef %a_USE_PAGE4
            case 4:
                %a_SendHTMLPage4();
                break;
          #endif

            default:
                %a_SendHTMLPage1();
                break;
        }

    } else {
        //not found
        %a_WriteString("HTTP/1.0 404 Not Found");
    }

    //finally, transmit the buffer
    %a_FlushBuffer(ch);
    // Updated transmit the data - used %a_IncTxPointer()

    // Reset sent size
    Internet_gTxSize[%a_CHANNEL] = 0;
    // Mark as 'sent on this'
    flag = 1 << ch; // Eg: CHANNEL_FLAG_0;
    %a_CHANNEL_FLAG = %a_CHANNEL_FLAG & (~flag);
    flag = flag * 0x10;
    %a_CHANNEL_FLAG = %a_CHANNEL_FLAG | flag;

    //reopen the socket and listen
    delay_ms(1);
    %a_RecreateSocket(ch);
    %a_Listen(ch);

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

    #error "TODO: MIAC stuff here"

  #else

    #error "Error - no Internet bus type defined."

  #endif

    return (0);
}

char %a_TW_TA_Comp(char ch)
{
    char retVal;

    //make sure we are addressing this device
    Internet_WR_ADDR = %a_W3100A_WR;

    if (ch > %a_MAX_CHANNEL)
    {
        return (255);  //error trying to open a non-existent channel
    }
    retVal = 0;

  #if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)
    //compares TWptr and TAPtr, and returns 0 if they are equal or a char identifier otherwise
    char idxBase = ch*4;
    char ack0 = %a_ACK_PTR[idxBase];
    char tx0 = %a_TX_PTR[idxBase];

    if (%a_ACK_PTR[idxBase+3] == %a_TX_PTR[idxBase+3])
    {
        if (%a_ACK_PTR[idxBase+2] == %a_TX_PTR[idxBase+2])
        {
            if (%a_ACK_PTR[idxBase+1] == %a_TX_PTR[idxBase+1])
            {
                if (ack0 == tx0)
                {
                    retVal = 0;
                } else {
                    if (tx0 > ack0)
                    {
                        retVal = 'B';
                    } else {
                        retVal = 'C';
                    }
                }
            } else {
                retVal = 'D';
            }
        } else {
            retVal = 'E';
        }
    } else {
        retVal = 'F';
    }
  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)

    // No code needed here - function unused

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

    #error "TODO: MIAC stuff here"

  #else

    #error "Error - no Internet bus type defined."

  #endif
    return (retVal);
}

void %a_ReadTWPtrReg(char ch)
{
  #if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)
    //make sure we are addressing this device
    Internet_WR_ADDR = %a_W3100A_WR;

    if (ch <= %a_MAX_CHANNEL)
    {
        char idxBase = ch*4;

        Internet_ReadPtrWithShadow(Cx_STW_PTR(ch), REG_Cx_TW_PR(ch));

        %a_TX_PTR[idxBase+0] = Internet_PTR_VAL[0];
        %a_TX_PTR[idxBase+1] = Internet_PTR_VAL[1];
        %a_TX_PTR[idxBase+2] = Internet_PTR_VAL[2];
        %a_TX_PTR[idxBase+3] = Internet_PTR_VAL[3];
    }
  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)

    // Unused, no code

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

    #error "TODO: MIAC stuff here"

  #else

    #error "Error - no Internet bus type defined."

  #endif
}

void %a_ReadTAPtrReg(char ch)
{
  #if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)
    //make sure we are addressing this device
    Internet_WR_ADDR = %a_W3100A_WR;

    if (ch <= %a_MAX_CHANNEL)
    {
        char idxBase = ch*4;

        Internet_ReadPtrWithShadow(Cx_STA_PTR(ch), REG_Cx_TA_PR(ch));

        %a_ACK_PTR[idxBase+0] = Internet_PTR_VAL[0];
        %a_ACK_PTR[idxBase+1] = Internet_PTR_VAL[1];
        %a_ACK_PTR[idxBase+2] = Internet_PTR_VAL[2];
        %a_ACK_PTR[idxBase+3] = Internet_PTR_VAL[3];
    }
  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)

    // Unused, no code

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

    #error "TODO: MIAC stuff here"

  #else

    #error "Error - no Internet bus type defined."

  #endif
}

void %a_WriteString(const char* string)
{
    char i = 0;
    char addr_h;
    char addr_l;
    char offset;
    char mask;
    char sub_idx;
    char digit;
    char newval;
    char force10;
    char idxBase = %a_CHANNEL;
  #if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)
    idxBase = idxBase*4;
  #endif

    //make sure we are addressing this device
    Internet_WR_ADDR = %a_W3100A_WR;

    //calculate the address
    //assume channel 0
    offset = %a_MEM_TX0_OFFSET;
    mask   = %a_MEM_TX0_MASK;
    if (%a_CHANNEL == 1)
    {
        offset = %a_MEM_TX1_OFFSET;
        mask   = %a_MEM_TX1_MASK;
    } else if (%a_CHANNEL == 2) {
        offset = %a_MEM_TX2_OFFSET;
        mask   = %a_MEM_TX2_MASK;
    } else if (%a_CHANNEL == 3) {
        offset = %a_MEM_TX3_OFFSET;
        mask   = %a_MEM_TX3_MASK;
    }

    addr_l = %a_TX_PTR[idxBase+0];
    addr_h = (%a_TX_PTR[idxBase+1] & mask) + offset;

  #if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)
    //send the i2c start bit and the address
    Internet_SendI2CStart();
    Internet_SendI2CByte(%a_W3100A_WR);    //write to W3100A
    Internet_SendI2CByte(addr_h);            //addr (hi byte)
    Internet_SendI2CByte(addr_l);            //addr (low byte)
  #endif

    //send the data
    while (string[i] != 0)
    {
        if (string[i] == '%')
        {
            if (string[i+1] == '%')
            {
                //"%%" should be sent as a single "%"
                Internet_SendHtmlChar(string[i]);   //data
                %a_IncTxPointer(idxBase);

            } else {

                if ((string[i+1] >= '1') && (string[i+1] <= ('0'+%a_MAX_SUBSTITUTIONS)))
                {
                    //need to send the value "%a_SUB_VAL" instead...
                    sub_idx = string[i+1] - '1';

                    digit = %a_SUB_VAL[sub_idx] / 100;
                    newval = %a_SUB_VAL[sub_idx] % 100;
                    if (digit > 0)
                    {
                        Internet_SendHtmlChar('0'+digit);   //100's
                        %a_IncTxPointer(idxBase);
                        force10 = 1;
                    } else {
                        force10 = 0;
                    }

                    digit = newval / 10;
                    newval = newval % 10;
                    if ((digit > 0) || (force10 != 0))
                    {
                        Internet_SendHtmlChar('0'+digit);   //10's
                        %a_IncTxPointer(idxBase);
                    }

                    Internet_SendHtmlChar('0'+newval);      //1's
                    %a_IncTxPointer(idxBase);
                } else {

                    //illegal substitution: just ignore it
                    Internet_SendHtmlChar(string[i]);
                    %a_IncTxPointer(idxBase);
                    Internet_SendHtmlChar(string[i+1]);
                    %a_IncTxPointer(idxBase);
                }
            }

            //inc by 2 (e.g. "%2", "%%")
            i = i + 2;

        } else {

            Internet_SendHtmlChar(string[i]);   //data
            %a_IncTxPointer(idxBase);
            i++;
        }
    }

    //append "/r/n" to the end (CR + LF)
    Internet_SendHtmlChar(0x0D);
    %a_IncTxPointer(idxBase);
    Internet_SendHtmlChar(0x0A);
    %a_IncTxPointer(idxBase);

  #if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)
    //send the i2c stop bit
    Internet_SendI2CStop();
  #endif
}

void %a_IncTxPointer(char idxBase)
{
  #if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)

    %a_TX_PTR[idxBase+0] = %a_TX_PTR[idxBase+0] + 1;
    if (%a_TX_PTR[idxBase+0] == 0)
    {
        %a_TX_PTR[idxBase+1] = %a_TX_PTR[idxBase+1] + 1;
        if (%a_TX_PTR[idxBase+1] == 0)
        {
            %a_TX_PTR[idxBase+2] = %a_TX_PTR[idxBase+2] + 1;
            if (%a_TX_PTR[idxBase+2] == 0)
            {
                %a_TX_PTR[idxBase+3] = %a_TX_PTR[idxBase+3] + 1;
            }
        }
    }

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)

	//increment the pointer and wrap it around
	Internet_gTxPtr[%a_CHANNEL]++;
	Internet_gTxSize[%a_CHANNEL]++;
	if (Internet_gTxSize[%a_CHANNEL] >= W5100_TXMEM_MASK)
	{
	    %a_FlushBuffer(%a_CHANNEL);
	}
	if (Internet_gTxPtr[%a_CHANNEL] > W5100_TXMEM(%a_CHANNEL) + W5100_TXMEM_MASK)
	{
		Internet_gTxPtr[%a_CHANNEL] = W5100_TXMEM(%a_CHANNEL); // Wrap pointer
	}

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

	#error "TODO: MIAC stuff here"

  #else

	#error "Error - no Internet bus type defined."

  #endif
}

void %a_FlushBuffer(char ch)
{
  #if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)
    //make sure we are addressing this device
    Internet_WR_ADDR = %a_W3100A_WR;

    if (ch <= %a_MAX_CHANNEL)
    {
        char idxBase = ch*4;

        Internet_PTR_VAL[0] = %a_TX_PTR[idxBase+0];
        Internet_PTR_VAL[1] = %a_TX_PTR[idxBase+1];
        Internet_PTR_VAL[2] = %a_TX_PTR[idxBase+2];
        Internet_PTR_VAL[3] = %a_TX_PTR[idxBase+3];

        //Update TW_PR
        Internet_WritePtr(0, REG_Cx_TW_PR(ch));

        //transmit the data
        Internet_WriteReg(0, REG_Cx_CR(ch), CSEND);
        delay_ms(1);
    }
  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)
	    // Buffer will overflow, loosing data
	    unsigned short tx_ptr_cur;

        // Set the position to write as the position to read from, marking the end of the data
	    tx_ptr_cur = Internet_ReadWord(W5100_Sn_TX_WR(ch)); // Current pointer
	    tx_ptr_cur += Internet_gTxSize[ch];                     // Add buffered size
	    //tx_ptr_cur = tx_ptr_cur & W5100_TXMEM_MASK;          // Dont wrap buffer pointer, only wrapped when calculating address
	    Internet_WriteWord(W5100_Sn_TX_WR(ch), tx_ptr_cur);

        // Now Send the SEND command
        Internet_WriteByte(W5100_Sn_CR(ch), W5100_Sn_CR_SEND);

        // Wait for Sending Process
        while(Internet_ReadByte(W5100_Sn_CR(ch)));
        // Reset sent size
        Internet_gTxSize[ch] = 0;

        // Wait for Sending Process
        while(Internet_ReadByte(W5100_Sn_CR(%a_CHANNEL)));
        //make sure Tx buffer is empty...
        {
          char is_equal;
          char timeout;
          is_equal = 0;
          timeout = 100;
          while (is_equal == 0)
          {
              if((Internet_ReadByte(W5100_Sn_IR(%a_CHANNEL)) & W5100_Sn_IR_SEND_OK) != 0)
              {
                  is_equal = 1;
              } else {
                  delay_ms(1);
                  timeout--;
                  if (timeout == 0)
                  {
                      is_equal = 1;
                  }
              }
          }
        }
        delay_ms(1);

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

	#error "TODO: MIAC stuff here"

  #else

	#error "Error - no Internet bus type defined."

  #endif
}

char %a_GetHTMLReq(char ch)
{
    char i;
    char name_length = 0;
    char retval = 0;
    char rxpage[%a_MAX_PAGENAME_LENGTH];
    char rx_char;
    char ch0, ch1;

  #if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)
    char addr_l;
    char addr_h;
    char offset;
    char mask;

    char rr_ptr[4];
    char rw_ptr[4];

    ch0 = 0; // For HTML chars
    ch1 = 1;

    //make sure we are addressing this device
    Internet_WR_ADDR = %a_W3100A_WR;

    //calculate the address
    //assume channel 0
    offset = %a_MEM_RX0_OFFSET;
    mask   = %a_MEM_RX0_MASK;
    if (ch == 1) {
        offset = %a_MEM_RX1_OFFSET;
        mask   = %a_MEM_RX1_MASK;
    } else if (ch == 2) {
        offset = %a_MEM_RX2_OFFSET;
        mask   = %a_MEM_RX2_MASK;
    } else if (ch == 3) {
        offset = %a_MEM_RX3_OFFSET;
        mask   = %a_MEM_RX3_MASK;
    }

    //get the RW pointer
    Internet_ReadPtrWithShadow(Cx_SRW_PTR(ch), REG_Cx_RW_PR(ch));
    rw_ptr[0] = Internet_PTR_VAL[0];
    rw_ptr[1] = Internet_PTR_VAL[1];
    rw_ptr[2] = Internet_PTR_VAL[2];
    rw_ptr[3] = Internet_PTR_VAL[3];

    //get the RR pointer
    Internet_ReadPtrWithShadow(Cx_SRR_PTR(ch), REG_Cx_RR_PR(ch));
    rr_ptr[0] = Internet_PTR_VAL[0];
    rr_ptr[1] = Internet_PTR_VAL[1];
    rr_ptr[2] = Internet_PTR_VAL[2];
    rr_ptr[3] = Internet_PTR_VAL[3];

    addr_l = rr_ptr[0];
    addr_h = (rr_ptr[1] & mask) + offset;

    //send the i2c start bit and the address
    Internet_SendI2CStart();
    Internet_SendI2CByte(%a_W3100A_WR);        //write to W3100A
    Internet_SendI2CByte(addr_h);                //addr (hi byte)
    Internet_SendI2CByte(addr_l);                //addr (low byte)
    Internet_SendI2CStop();

    Internet_SendI2CStart();
    Internet_SendI2CByte(%a_W3100A_RD);        //read from W3100A

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)

    // TCP start
    unsigned short address;
    unsigned short size;
    // Write the data size to the storage
    size = Internet_ReadWord(W5100_Sn_RX_RSR(%a_CHANNEL));
    // Claculate the address
    address = Internet_ReadWord(W5100_Sn_RX_RD(%a_CHANNEL));
    address = address & W5100_RXMEM_MASK;
    address = W5100_RXMEM(%a_CHANNEL) + address;

    ch0 = %a_CHANNEL; // For HTML chars
    ch1 = %a_CHANNEL;

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

	#error "TODO: MIAC stuff here"

  #else

	#error "Error - no Internet bus type defined."

  #endif

    // The remainder of this code is generic
    rx_char = Internet_ReadHtmlChar(ch0);
    if (rx_char != 'G') goto %a_rx_fin;
    rx_char = Internet_ReadHtmlChar(ch0);
    if (rx_char != 'E') goto %a_rx_fin;
    rx_char = Internet_ReadHtmlChar(ch0);
    if (rx_char != 'T') goto %a_rx_fin;
    rx_char = Internet_ReadHtmlChar(ch0);
    if (rx_char != ' ') goto %a_rx_fin;
    rx_char = Internet_ReadHtmlChar(ch0);
    if (rx_char != '/') goto %a_rx_fin;

    //Internet_DisplayDebugInfo(ch, 0, ' ', 1);    //erase the debug area
    for (i=0; i<16; i++)
    {
        rx_char = Internet_ReadHtmlChar(ch0);      //read data
        rxpage[i] = rx_char;
        if (rx_char == ' ')
        {
            name_length = i;
            goto %a_rx_fin;
        }

        //Internet_DisplayDebugInfo(ch, i, rx_char, 0);
    }

    %a_rx_fin:
    rx_char = Internet_ReadHtmlChar(ch1);          //read last byte
    Internet_ReadHtmlCharNull();

    //work out which page has been requested...

    if (name_length == 0)
    {
        //blank, so assume it's "index.htm"
        name_length = 9;
        rxpage[0] = 'i';
        rxpage[1] = 'n';
        rxpage[2] = 'd';
        rxpage[3] = 'e';
        rxpage[4] = 'x';
        rxpage[5] = '.';
        rxpage[6] = 'h';
        rxpage[7] = 't';
        rxpage[8] = 'm';
    }

    %a_try_page_1:
  #ifdef %a_USE_PAGE1
    //try first page...
    for (i=0; i<name_length; i++)
    {
        if (rxpage[i] != %a_HTML_PAGE1[i])
        {
            goto %a_try_page_2;
        }
    }
    retval = 1;
    goto %a_page_found;
  #endif

    %a_try_page_2:
  #ifdef %a_USE_PAGE2
    //try second page...
    for (i=0; i<name_length; i++)
    {
        if (rxpage[i] != %a_HTML_PAGE2[i])
        {
            goto %a_try_page_3;
        }
    }
    retval = 2;
    goto %a_page_found;
  #endif

    %a_try_page_3:
  #ifdef %a_USE_PAGE3
    //try second page...
    for (i=0; i<name_length; i++)
    {
        if (rxpage[i] != %a_HTML_PAGE3[i])
        {
            goto %a_try_page_4;
        }
    }
    retval = 3;
    goto %a_page_found;
  #endif

    %a_try_page_4:
  #ifdef %a_USE_PAGE4
    //try second page...
    for (i=0; i<name_length; i++)
    {
        if (rxpage[i] != %a_HTML_PAGE4[i])
        {
            retval = 0;
            goto %a_page_found;
        }
    }
    retval = 4;
    goto %a_page_found;
  #endif

    //if we get here, then the page has probably not been found...
    retval = 0;

    %a_page_found:
    return (retval);
}


#ifdef %a_USE_PAGE1
  void %a_SendHTMLPage1()
  {
      //Send a HTML page
      %o
  }
#endif


#ifdef %a_USE_PAGE2
  void %a_SendHTMLPage2()
  {
      //Send a HTML page
      %p
  }
#endif


#ifdef %a_USE_PAGE3
  void %a_SendHTMLPage3()
  {
      //Send a HTML page
      %q
  }
#endif


#ifdef %a_USE_PAGE4
  void %a_SendHTMLPage4()
  {
      //Send a HTML page
      %r
  }
#endif

/*DefinesCode_End*/

/*InitialisationCode_Start*/
/*InitialisationCode_End*/

/*InterruptCode_Start*/
/*InterruptCode_End*/

/********************************************************************
 * FUNCTIONS
 ********************************************************************/

void Initialize()
{
/*Macro_Initialize_Start*/
#if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)
    //make sure we are addressing this device
    Internet_WR_ADDR = %a_W3100A_WR;

	//Init I2C
  #ifndef MX_MI2C
	#error "Chip does not have master I2C capability"
  #endif

    //initialise i2c
    //trisc=0x19;     //sck, sda and int as i/p
  #ifdef MX_I2C_B
    set_bit(trisb, MX_I2C_SDA);
    set_bit(trisb, MX_I2C_SCL);
    set_bit(%a_INT_TRIS, %a_INT_PIN);
  #endif

  #ifdef MX_I2C_C
    set_bit(trisc, MX_I2C_SDA);
    set_bit(trisc, MX_I2C_SCL);
    set_bit(%a_INT_TRIS, %a_INT_PIN);
  #endif

    //sspadd=48;      //48 = 100kHz, 11 = 400kHz
    sspadd = %a_I2C_BAUD;

    //sspstat = 0x80;
    sspstat = 0;
    st_bit(sspstat,SMP);

    sspcon2 = 0;

	#ifdef _BOOSTC
		#ifdef SSPCON
			//sspcon=0x38;    //i2c master mode, SPEN
			sspcon = 0;
			st_bit(sspcon,SSPM3);
			st_bit(sspcon,CKP);
			st_bit(sspcon,SSPEN);
		#endif
		#ifdef SSPCON1
			//sspcon1=0x38;    //i2c master mode, SPEN
			sspcon1 = 0;
			st_bit(sspcon1,SSPM3);
			st_bit(sspcon1,CKP);
			st_bit(sspcon1,SSPEN);
		#endif
	#endif

	#ifdef HI_TECH_C
		#ifdef sspcon
			//sspcon=0x38;    //i2c master mode, SPEN
			sspcon = 0;
			st_bit(sspcon,SSPM3);
			st_bit(sspcon,CKP);
			st_bit(sspcon,SSPEN);
		#endif
		#ifdef sspcon1
			//sspcon1=0x38;    //i2c master mode, SPEN
			sspcon1 = 0;
			st_bit(sspcon1,SSPM3);
			st_bit(sspcon1,CKP);
			st_bit(sspcon1,SSPEN);
		#endif
	#endif

    //InitW3100A();
    //TODO: may need to generate a random number here for %a_SEQ_NUM
    %a_SEQ_NUM[0] = 12;
    %a_SEQ_NUM[1] = 34;
    %a_SEQ_NUM[2] = 56;
    %a_SEQ_NUM[3] = 78;

    Internet_WriteReg(0, REG_Cx_CR(0), CSW_RESET);  //send a software reset

    #ifdef %a_USE_CH0
    %a_STATUS[0] = 0;
    #endif
    #ifdef %a_USE_CH1
    %a_STATUS[1] = 0;
    #endif
    #ifdef %a_USE_CH2
    %a_STATUS[2] = 0;
    #endif
    #ifdef %a_USE_CH3
    %a_STATUS[3] = 0;
    #endif


    //InitNetConfig();
    //Set the gateway IP address
    Internet_WriteReg(0, REG_GAR+0, %a_GAR_0);
    Internet_WriteReg(0, REG_GAR+1, %a_GAR_1);
    Internet_WriteReg(0, REG_GAR+2, %a_GAR_2);
    Internet_WriteReg(0, REG_GAR+3, %a_GAR_3);

    //Set the subnet mask
    Internet_WriteReg(0, REG_SMR+0, %a_SMR_0);
    Internet_WriteReg(0, REG_SMR+1, %a_SMR_1);
    Internet_WriteReg(0, REG_SMR+2, %a_SMR_2);
    Internet_WriteReg(0, REG_SMR+3, %a_SMR_3);

    //Set the MAC address
    Internet_WriteReg(0, REG_SHAR+0, %a_SHAR_0);
    Internet_WriteReg(0, REG_SHAR+1, %a_SHAR_1);
    Internet_WriteReg(0, REG_SHAR+2, %a_SHAR_2);
    Internet_WriteReg(0, REG_SHAR+3, %a_SHAR_3);
    Internet_WriteReg(0, REG_SHAR+4, %a_SHAR_4);
    Internet_WriteReg(0, REG_SHAR+5, %a_SHAR_5);

    //Set the source IP address
    Internet_WriteReg(0, REG_SIPR+0, %a_SIPR_0);
    Internet_WriteReg(0, REG_SIPR+1, %a_SIPR_1);
    Internet_WriteReg(0, REG_SIPR+2, %a_SIPR_2);
    Internet_WriteReg(0, REG_SIPR+3, %a_SIPR_3);

    //Set the memory size registers
    Internet_WriteReg(0, REG_RMSR, %a_RMSR_VAL);
    Internet_WriteReg(0, REG_TMSR, %a_TMSR_VAL);

    //Set the TCP timeout values
    //Internet_WriteReg(0, REG_IRTR, IRTR_HI);
    //Internet_WriteReg(0, REG_IRTR+1, IRTR_LO);
    //Internet_WriteReg(0, REG_RCR, RCR_VAL);

    //send a sys init
    %a_STATUS[0] = 0;
    Internet_WriteReg(0, REG_Cx_CR(0), CSYS_INIT);

    //wait until a SSYS_INIT_OK interrupt
    while (!(%a_STATUS[0] & SSYS_INIT_OK))
    {
        %a_Check_Int();
    }

#elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)

    //initialise spi
	#ifndef MX_SPI
		#error "Chip does not have SPI capability"
	#endif

	//initialise spi
	#ifdef MX_SPI_B
		set_bit(trisb,MX_SPI_SDI);
		clear_bit(trisb,MX_SPI_SDO);
		clear_bit(trisb,MX_SPI_SCK);
	#endif

	#ifdef MX_SPI_C
		set_bit(trisc,MX_SPI_SDI);
		clear_bit(trisc,MX_SPI_SDO);
		clear_bit(trisc,MX_SPI_SCK);
	#endif

	#ifdef MX_SPI_BCB
		set_bit(trisb,MX_SPI_SDI);
		clear_bit(trisc,MX_SPI_SDO);
		clear_bit(trisb,MX_SPI_SCK);
	#endif

	clear_bit(%a_CS_TRIS,%a_CS_PIN);		//chip select is an output
	set_bit(%a_CS_PORT, %a_CS_PIN);		//disable SPI device

	sspstat = 64;

	#ifdef _BOOSTC
		#ifdef SSPCON							// SPI Speed Initialise
			sspcon = 1;
			st_bit(sspcon, SSPEN);
		#endif
		#ifdef SSPCON1
			sspcon1 = 1;
			st_bit(sspcon1, SSPEN);
		#endif
	#endif

	#ifdef HI_TECH_C
		#ifdef sspcon							// SPI Speed Initialise
			sspcon = 1;
			st_bit(sspcon, SSPEN);
		#endif
		#ifdef sspcon1
			sspcon1 = 1;
			st_bit(sspcon1, SSPEN);
		#endif
	#endif


    //Init W5100
    Internet_WriteByte(W5100_MR, W5100_MR_RST);  //send a software reset

    //TODO: Set the interrupt mask
    Internet_WriteByte(W5100_IMR+0, 0);

    //Set the gateway IP address
    Internet_WriteByte(W5100_GAR+0, %a_GAR_0);
    Internet_WriteByte(W5100_GAR+1, %a_GAR_1);
    Internet_WriteByte(W5100_GAR+2, %a_GAR_2);
    Internet_WriteByte(W5100_GAR+3, %a_GAR_3);

    //Set the subnet mask
    Internet_WriteByte(W5100_SUBR+0, %a_SMR_0);
    Internet_WriteByte(W5100_SUBR+1, %a_SMR_1);
    Internet_WriteByte(W5100_SUBR+2, %a_SMR_2);
    Internet_WriteByte(W5100_SUBR+3, %a_SMR_3);

    //Set the MAC address
    Internet_WriteByte(W5100_SHAR+0, %a_SHAR_0);
    Internet_WriteByte(W5100_SHAR+1, %a_SHAR_1);
    Internet_WriteByte(W5100_SHAR+2, %a_SHAR_2);
    Internet_WriteByte(W5100_SHAR+3, %a_SHAR_3);
    Internet_WriteByte(W5100_SHAR+4, %a_SHAR_4);
    Internet_WriteByte(W5100_SHAR+5, %a_SHAR_5);

    //Set the source IP address
    Internet_WriteByte(W5100_SIPR+0, %a_SIPR_0);
    Internet_WriteByte(W5100_SIPR+1, %a_SIPR_1);
    Internet_WriteByte(W5100_SIPR+2, %a_SIPR_2);
    Internet_WriteByte(W5100_SIPR+3, %a_SIPR_3);

    //Set the memory size registers
    Internet_WriteByte(W5100_RMSR, RMSR_VAL);
    Internet_WriteByte(W5100_TMSR, TMSR_VAL);

    //Set the TCP timeout values
    Internet_WriteByte(W5100_RTR+0, IRTR_HI);
    Internet_WriteByte(W5100_RTR+1, IRTR_LO);
    Internet_WriteByte(W5100_RCR, RCR_VAL);


#elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

	#error "TODO: MIAC stuff here"

#else

	#error "Error - no Internet bus type defined."

#endif

    %a_CHANNEL_FLAG = 0;

/*Macro_Initialize_End*/
}


char CreateServerSocket(char channel, char port_hi, char port_lo)
{
/*Macro_CreateServerSocket_Start*/
    if (channel > %a_MAX_CHANNEL)
    {
        return (0);  //error trying to open a non-existent channel
    }

  #if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)
    //make sure we are addressing this device
    Internet_WR_ADDR = %a_W3100A_WR;

    //set up the socket protocol and option flags
    Internet_WriteReg(0, REG_Cx_SOPR(channel), SOCK_STREAM);

    //set up the port
    Internet_WriteReg(0, REG_Cx_SPR(channel), port_hi);
    Internet_WriteReg(0, REG_Cx_SPR(channel)+1, port_lo);

    //initialise the socket
    %a_STATUS[channel] = 0;
    Internet_WriteReg(0, REG_Cx_CR(channel), CSOCK_INIT);

    //check for result
    while (%a_STATUS[channel] == 0)
    {
        %a_Check_Int();
    }

    //check the result
    if ((%a_STATUS[channel] & SSOCK_INIT_OK) == 0)
    {
        return (0);  //failed to open the socket
    }
  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)

	// Make sure we close the socket first
	if (Internet_ReadByte(W5100_Sn_SR(channel)) != W5100_SOCK_CLOSED)
	{
		Internet_WriteByte(W5100_Sn_CR(channel), W5100_Sn_CR_CLOSE);
		// Waiting until the S0_CR is clear
		while(Internet_ReadByte(W5100_Sn_CR(channel)));
	}
	// Open the socket
	{
	    char state = Internet_ReadByte(W5100_Sn_SR(channel));
	    while(state != W5100_SOCK_INIT)
	    {
		    //Create the TCP layer socket
		    //set up the socket protocol and option flags
		    Internet_WriteByte(W5100_Sn_MR(channel), W5100_Sn_MR_TCP);
            // Set the port value
            Internet_WriteByte(W5100_Sn_PORT(channel) + 0, port_hi);
            Internet_WriteByte(W5100_Sn_PORT(channel) + 1, port_lo);
		    //initialise the socket
		    Internet_WriteByte(W5100_Sn_CR(channel), W5100_Sn_CR_OPEN);
		    // Waiting until the S0_CR is clear
		    while(Internet_ReadByte(W5100_Sn_CR(channel)));
		    state = Internet_ReadByte(W5100_Sn_SR(channel));
	    }
	}

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

    #error "TODO: MIAC stuff here"

  #else

    #error "Error - no Internet bus type defined."

  #endif
    %a_UpdateSeqnum(channel);

    %a_Listen(channel);

    return (1);
/*Macro_CreateServerSocket_End*/
}

char CheckSocketActivity()
{
/*Macro_CheckSocketActivity_Start*/
    char retval = 0;

  #if (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W3100)
    char channels = %a_Check_Int();

    //make sure we are addressing this device
    Internet_WR_ADDR = %a_W3100A_WR;

    if (channels != 0)
    {
        //there has been an interrupt, so process it

        if (channels & 0x01)
        {
            channels = channels & 0xFE;
            %a_ProcessSocket(0);
        }

        if (channels & 0x02)
        {
            #ifdef %a_USE_CH1
              channels = channels & 0xFD;
              %a_ProcessSocket(1);
            #else
              //TODO: what if channel interrupts, but we don't use it?
            #endif
        }

        if (channels & 0x04)
        {
            #ifdef %a_USE_CH2
              channels = channels & 0xFB;
              %a_ProcessSocket(2);
            #else
              //TODO: what if channel interrupts, but we don't use it?
            #endif
        }

        if (channels & 0x08)
        {
            #ifdef %a_USE_CH3
              channels = channels & 0xF7;
              %a_ProcessSocket(3);
            #else
              //TODO: what if channel interrupts, but we don't use it?
            #endif
        }

    } else {

        //have a look at the socket state to see if things have gone wrong
        #ifdef %a_USE_CH0
            if (%a_CHANNEL_FLAG & CHANNEL_FLAG_0)
            {
                %a_CheckForProbs(0);
            }
        #endif
        #ifdef %a_USE_CH1
            if (%a_CHANNEL_FLAG & CHANNEL_FLAG_1)
            {
                %a_CheckForProbs(1);
            }
        #endif
        #ifdef %a_USE_CH2
            if (%a_CHANNEL_FLAG & CHANNEL_FLAG_2)
            {
                %a_CheckForProbs(2);
            }
        #endif
        #ifdef %a_USE_CH3
            if (%a_CHANNEL_FLAG & CHANNEL_FLAG_3)
            {
                %a_CheckForProbs(3);
            }
        #endif

        //delay here, so we don't poll the device too quickly
        delay_ms(25);
    }
    retval = (%a_CHANNEL_FLAG & 0xF0);
    retval = retval >> 4;
  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_W5100)

	char channels = Internet_ReadByte(W5100_IR); // W5100_Sn_SR(channel));
	if ((channels & 0xF) != 0) // Interrupt mask 0xF is for W5100_IR_SOCK(0)...W5100_IR_SOCK(3)
	{
        if (channels & W5100_IR_SOCK(0))
        {
            %a_ProcessSocket(0);
        }

        if (channels & W5100_IR_SOCK(1))
        {
            #ifdef %a_USE_CH1
              %a_ProcessSocket(1);
            #else
              //TODO: what if channel interrupts, but we don't use it?
            #endif
        }

        if (channels & W5100_IR_SOCK(2))
        {
            #ifdef %a_USE_CH2
              %a_ProcessSocket(2);
            #else
              //TODO: what if channel interrupts, but we don't use it?
            #endif
        }

        if (channels & W5100_IR_SOCK(3))
        {
            #ifdef %a_USE_CH3
              %a_ProcessSocket(3);
            #else
              //TODO: what if channel interrupts, but we don't use it?
            #endif
        }
	}
	else
	{
	  // TODO: Check for problems and reset channels if flagged
	}
    retval = (%a_CHANNEL_FLAG & 0xF0);
    retval = retval >> 4;
    //delay here, so we don't poll the device too quickly
    delay_ms(25);

  #elif (MX_TCPIP_HWTYPE == MX_TCPIP_HWTYPE_MIAC)

    #error "TODO: MIAC stuff here"

  #else

    #error "Error - no Internet bus type defined."

  #endif

    return (retval);
/*Macro_CheckSocketActivity_End*/
}
